@@ -35,6 +35,7 @@
 #include "libs/setpgrp.h"
 #include "libs/Picture.h"
 #include "libs/PictureUtils.h"
+#include "libs/FGettext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "colorset.h"
@@ -499,9 +500,34 @@ static void do_title_style(F_CMD_ARGS, Bool do_add)
 					height = 0;
 				}
 			}
-			if (decor->title_height != height)
+			if (decor->title_height != height ||
+			    decor->min_title_height != 0)
 			{
 				decor->title_height = height;
+				decor->min_title_height = 0;
+				decor->flags.has_title_height_changed = 1;
+			}
+			if (action)
+				action += next;
+		}
+		else if (!do_add && StrEquals(parm,"MinHeight"))
+		{
+			int height = 0;
+			int next = 0;
+
+			if (!action ||
+			    sscanf(action, "%d%n", &height, &next) <= 0 ||
+			    height < MIN_FONT_HEIGHT ||
+			    height > MAX_FONT_HEIGHT)
+			{
+				if (height < MIN_FONT_HEIGHT)
+					height = MIN_FONT_HEIGHT;
+				else if (height > MAX_FONT_HEIGHT)
+					height = 0;
+			}
+			if (decor->min_title_height != height)
+			{
+				decor->min_title_height = height;
 				decor->flags.has_title_height_changed = 1;
 			}
 			if (action)
@@ -978,6 +1004,149 @@ static void do_button_style(F_CMD_ARGS, Bool do_add)
 	return;
 }
 
+static
+int update_decorface_colorset(DecorFace *df, int cset)
+{
+	DecorFace *tdf;
+	int has_changed = 0;
+
+	for(tdf = df; tdf != NULL; tdf = tdf->next)
+	{
+		if (DFS_FACE_TYPE(tdf->style) == ColorsetButton &&
+		    tdf->u.acs.cs == cset)
+		{
+			tdf->flags.has_changed = 1;
+			has_changed = 1;
+		}
+	}
+	
+	return has_changed;
+}
+
+static
+int update_titlebutton_colorset(TitleButton *tb, int cset)
+{
+	int i;
+	int has_changed = 0;
+
+	for(i = 0; i < BS_MaxButtonState; i++)
+	{
+		tb->state[i].flags.has_changed =
+			update_decorface_colorset(&(tb->state[i]), cset);
+		has_changed |= tb->state[i].flags.has_changed;
+	}
+	return has_changed;
+}
+
+static
+void update_decors_colorset(int cset)
+{
+	int i;
+	int has_changed;
+	FvwmDecor *decor = &Scr.DefaultDecor;
+
+#ifdef USEDECOR
+	for(decor = &Scr.DefaultDecor; decor != NULL; decor = decor->next)
+#endif
+	{
+		has_changed = 0;
+		for(i = 0; i < NUMBER_OF_BUTTONS; i++)
+		{
+			decor->flags.has_changed |= update_titlebutton_colorset(
+				&(decor->buttons[i]), cset);
+		}
+		decor->flags.has_changed |= update_titlebutton_colorset(
+			&(decor->titlebar), cset);
+		decor->flags.has_changed |= update_decorface_colorset(
+			&(decor->BorderStyle.active), cset);
+		decor->flags.has_changed |= update_decorface_colorset(
+			&(decor->BorderStyle.inactive), cset);
+		if (decor->flags.has_changed)
+		{
+			Scr.flags.do_need_window_update = 1;
+		}
+	}
+}
+
+static Bool __parse_vector_line_one_coord(
+	char **ret_action, int *pcoord, int *poff, char *action)
+{
+	int offset;
+	int n;
+
+	*ret_action = action;
+	n = sscanf(action, "%d%n", pcoord, &offset);
+	if (n < 1)
+	{
+		return False;
+	}
+	action += offset;
+	/* check for offest */
+	if (*action == '+' || *action == '-')
+	{
+		n = sscanf(action, "%dp%n", poff, &offset);
+		if (n < 1)
+		{
+			return False;
+		}
+		if (*poff < -128)
+		{
+			*poff = -128;
+		}
+		else if (*poff > 127)
+		{
+			*poff = 127;
+		}
+		action += offset;
+	}
+	else
+	{
+		*poff = 0;
+	}
+	*ret_action = action;
+
+	return True;
+}
+
+static Bool __parse_vector_line(
+	char **ret_action, int *px, int *py, int *pxoff, int *pyoff, int *pc,
+	char *action)
+{
+	Bool is_valid = True;
+	int offset;
+	int n;
+
+	*ret_action = action;
+	if (__parse_vector_line_one_coord(&action, px, pxoff, action) == False)
+	{
+		return False;
+	}
+	if (*action != 'x')
+	{
+		return False;
+	}
+	action++;
+	if (__parse_vector_line_one_coord(&action, py, pyoff, action) == False)
+	{
+		return False;
+	}
+	if (*action != '@')
+	{
+		return False;
+	}
+	action++;
+	/* read the line style */
+	n = sscanf(action, "%d%n", pc, &offset);
+	if (n < 1)
+	{
+		return False;
+	}
+	action += offset;
+	*ret_action = action;
+
+	return is_valid;
+}
+
 /* ---------------------------- interface functions ------------------------- */
 
 void refresh_window(Window w, Bool window_update)
@@ -1159,6 +1328,15 @@ void FreeDecorFace(Display *dpy, DecorFace *df)
 		{
 			free (df->u.vector.y);
 		}
+		/* free offsets for coord */
+		if (df->u.vector.xoff)
+		{
+			free(df->u.vector.xoff);
+		}
+		if (df->u.vector.yoff)
+		{
+			free(df->u.vector.yoff);
+		}
 		if (df->u.vector.c)
 		{
 			free (df->u.vector.c);
@@ -1275,7 +1453,7 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 				num = sscanf(style,"%d",&num_coords);
 			}
 
-			if (num != 1 || num_coords<2 ||
+			if (num < 1 || num_coords<2 ||
 			    num_coords > MAX_TITLE_BUTTON_VECTOR_LINES)
 			{
 				if (verbose)
@@ -1292,6 +1470,8 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 			vc->use_fgbg = 0;
 			vc->x = safemalloc(sizeof(char) * num_coords);
 			vc->y = safemalloc(sizeof(char) * num_coords);
+			vc->xoff = safemalloc(sizeof(char) * num_coords);
+			vc->yoff = safemalloc(sizeof(char) * num_coords);
 			vc->c = safemalloc(sizeof(char) * num_coords);
 
 			/* get the points */
@@ -1299,27 +1479,15 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 			{
 				int x;
 				int y;
+				int xoff = 0;
+				int yoff = 0;
 				int c;
 
-				/* X x Y @ line_style */
-				num = sscanf(
-					s,"%dx%d@%d%n", &x, &y, &c, &offset);
-				if (num != 3)
+				if (__parse_vector_line(
+					    &s, &x, &y, &xoff, &yoff, &c, s) ==
+				    False)
 				{
-					if (verbose)
-					{
-						fvwm_msg(
-							ERR, "ReadDecorFace",
-							"Bad button style (3)"
-							" in line %s", action);
-					}
-					free(vc->x);
-					free(vc->y);
-					free(vc->c);
-					vc->x = NULL;
-					vc->y = NULL;
-					vc->c = NULL;
-					return False;
+					break;
 				}
 				if (x < 0)
 				{
@@ -1344,11 +1512,33 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 				vc->x[i] = x;
 				vc->y[i] = y;
 				vc->c[i] = c;
+				vc->xoff[i] = xoff;
+				vc->yoff[i] = yoff;
 				if (c == 2 || c == 3)
 				{
 					vc->use_fgbg = 1;
 				}
-				s += offset;
+			}
+			if (i < vc->num)
+			{
+				if (verbose)
+				{
+					fvwm_msg(
+						ERR, "ReadDecorFace",
+						"Bad button style (3) in line"
+						" %s", action);
+				}
+				free(vc->x);
+				free(vc->y);
+				free(vc->c);
+				free(vc->xoff);
+				free(vc->yoff);
+				vc->x = NULL;
+				vc->y = NULL;
+				vc->c = NULL;
+				vc->xoff = NULL;
+				vc->yoff = NULL;
+				return False;
 			}
 			memset(&df->style, 0, sizeof(df->style));
 			DFS_FACE_TYPE(df->style) = VectorButton;
@@ -1494,6 +1684,31 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 			/* pixmap read in when the window is created */
 			df->u.p = NULL;
 		}
+		else if (strncasecmp (style, "Colorset", 12) == 0)
+		{
+			int val[2];
+			int n;
+
+			n = GetIntegerArguments(s, NULL, val, 2);
+			if (n == 0)
+			{
+			}
+			memset(&df->style, 0, sizeof(df->style));
+			if (n > 0 && val[0] >= 0)
+			{
+				
+				df->u.acs.cs = val[0];
+				alloc_colorset(val[0]);
+				DFS_FACE_TYPE(df->style) = ColorsetButton;
+			}
+			df->u.acs.alpha_percent = 100;
+			if (n > 1)
+			{
+				df->u.acs.alpha_percent =
+					max(0, min(100,val[1]));
+			}
+			s = SkipNTokens(s, n);
+		}
 		else
 		{
 			if (verbose)
@@ -1769,6 +1984,20 @@ void reset_decor_changes(void)
 	return;
 }
 
+void update_fvwm_colorset(int cset)
+{
+	if (cset == Scr.DefaultColorset)
+	{
+		Scr.flags.do_need_window_update = 1;
+		Scr.flags.has_default_color_changed = 1;
+	}
+	UpdateMenuColorset(cset);
+	update_style_colorset(cset);
+	update_decors_colorset(cset);
+
+	return;
+}
+
 /* ---------------------------- builtin commands ---------------------------- */
 
 void CMD_Beep(F_CMD_ARGS)
@@ -2182,7 +2411,7 @@ void CMD_Wait(F_CMD_ARGS)
 					e.xkey.keycode, e.xkey.state,
 					GetUnusedModifiers(),
 					GetContext(&t, t, &e, &nonewin),
-					KEY_BINDING);
+					BIND_KEYPRESS);
 				if (escape != NULL)
 				{
 					if (!strcasecmp(escape,"escapefunc"))
@@ -2259,6 +2488,10 @@ void CMD_PrintInfo(F_CMD_ARGS)
 	{
 		PicturePrintColorInfo(verbose);
 	}
+	else if (StrEquals(subject, "Locale"))
+	{
+		FlocalePrintLocaleInfo(dpy, verbose);
+	}
 	else
 	{
 		fvwm_msg(ERR, "PrintInfo",
@@ -2339,6 +2572,13 @@ void CMD_PixmapPath(F_CMD_ARGS)
 	return;
 }
 
+void CMD_LocalePath(F_CMD_ARGS)
+{
+	FGettextSetLocalePath( action );
+
+	return;
+}
+
 void CMD_ModulePath(F_CMD_ARGS)
 {
 	static int need_to_free = 0;
@@ -2349,6 +2589,7 @@ void CMD_ModulePath(F_CMD_ARGS)
 	return;
 }
 
+
 void CMD_ModuleTimeout(F_CMD_ARGS)
 {
 	int timeout;
@@ -2464,13 +2705,7 @@ void CMD_Colorset(F_CMD_ARGS)
 		return;
 	}
 	parse_colorset(n, token);
-	if (n == Scr.DefaultColorset)
-	{
-		Scr.flags.do_need_window_update = 1;
-		Scr.flags.has_default_color_changed = 1;
-	}
-	UpdateMenuColorset(n);
-	update_style_colorset(n);
+	update_fvwm_colorset(n);
 
 	return;
 }
@@ -3787,7 +4022,7 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 	/* check for a binding */
 	stroke_action = CheckBinding(
 		Scr.AllBindings, sequence, 0, modifiers, GetUnusedModifiers(),
-		exc->w.wcontext, STROKE_BINDING);
+		exc->w.wcontext, BIND_STROKE);
 
 	/* execute the action */
 	if (stroke_action != NULL)
