@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -5,16 +6,15 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/****************************************************************************
- * This module is all new
+/* This module is all new
  * by Rob Nation
  *
  * This code handles colormaps for fvwm.
@@ -22,7 +22,9 @@
  * Copyright 1994 Robert Nation. No restrictions are placed on this code,
  * as long as the copyright notice is preserved . No guarantees or
  * warrantees of any sort whatsoever are given or implied or anything.
- ****************************************************************************/
+ */
+
+/* ---------------------------- included header files ----------------------- */
 
 #include "config.h"
 
@@ -32,51 +34,71 @@
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
+#include "execcontext.h"
+#include "eventhandler.h"
 #include "functions.h"
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
 #include "colormaps.h"
 
-FvwmWindow *colormap_win;
-extern FvwmWindow *Fw;
+/* ---------------------------- local definitions --------------------------- */
+
+/* ---------------------------- local macros -------------------------------- */
+
+/* ---------------------------- imports ------------------------------------- */
+
+/* ---------------------------- included code files ------------------------- */
+
+/* ---------------------------- local types --------------------------------- */
+
+/* ---------------------------- forward declarations ------------------------ */
+
+/* ---------------------------- local variables ----------------------------- */
 
 static Bool client_controls_colormaps = False;
 static Colormap last_cmap = None;
 
+/* ---------------------------- exported variables (globals) ---------------- */
+
+const FvwmWindow *colormap_win;
+
+/* ---------------------------- local functions ----------------------------- */
+
+/* ---------------------------- interface functions ------------------------- */
+
 void set_client_controls_colormaps(Bool flag)
 {
 	client_controls_colormaps = flag;
+
 	return;
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *	HandleColormapNotify - colormap notify event handler
+/* HandleColormapNotify - colormap notify event handler
  *
  * This procedure handles both a client changing its own colormap, and
  * a client explicitly installing its colormap itself (only the window
  * manager should do that, so we must set it correctly).
- *
- ***********************************************************************/
-void HandleColormapNotify(void)
+ */
+void colormap_handle_colormap_notify(const evh_args_t *ea)
 {
-	XColormapEvent *cevent = (XColormapEvent *)&Event;
+	XEvent evdummy;
+	XColormapEvent *cevent = (XColormapEvent *)ea->exc->x.etrigger;
 	Bool ReInstall = False;
 	XWindowAttributes attr;
+	FvwmWindow *fw = ea->exc->w.fw;
 
-	if (!Fw)
+	if (!fw)
 	{
 		return;
 	}
 	if (cevent->new)
 	{
-		if (XGetWindowAttributes(dpy, FW_W(Fw), &attr) != 0)
+		if (XGetWindowAttributes(dpy, FW_W(fw), &attr) != 0)
 		{
-			Fw->attr_backup.colormap = attr.colormap;
-			if (Fw == colormap_win &&
-			    Fw->number_cmap_windows == 0)
+			fw->attr_backup.colormap = attr.colormap;
+			if (fw == colormap_win &&
+			    fw->number_cmap_windows == 0)
 			{
 				last_cmap = attr.colormap;
 			}
@@ -90,35 +112,35 @@ void HandleColormapNotify(void)
 		ReInstall = True;
 	}
 
-	while (XCheckTypedEvent(dpy,ColormapNotify,&Event))
+	while (FCheckTypedEvent(dpy, ColormapNotify, &evdummy))
 	{
 		if (XFindContext(
 			    dpy, cevent->window, FvwmContext,
-			    (caddr_t *) &Fw) == XCNOENT)
+			    (caddr_t *) &fw) == XCNOENT)
 		{
-			Fw = NULL;
+			fw = NULL;
 		}
-		if ((Fw)&&(cevent->new))
+		if ((fw)&&(cevent->new))
 		{
-			if (XGetWindowAttributes(dpy, FW_W(Fw), &attr) != 0)
+			if (XGetWindowAttributes(dpy, FW_W(fw), &attr) != 0)
 			{
-				Fw->attr_backup.colormap = attr.colormap;
-				if (Fw == colormap_win &&
-				    Fw->number_cmap_windows == 0)
+				fw->attr_backup.colormap = attr.colormap;
+				if (fw == colormap_win &&
+				    fw->number_cmap_windows == 0)
 				{
 					last_cmap = attr.colormap;
 				}
 				ReInstall = True;
 			}
 		}
-		else if ((Fw)&&
+		else if ((fw)&&
 			(cevent->state == ColormapUninstalled)&&
 			(last_cmap == cevent->colormap))
 		{
 			/* Some window installed its colormap, change it back */
 			ReInstall = True;
 		}
-		else if ((Fw)&&
+		else if ((fw)&&
 			(cevent->state == ColormapInstalled)&&
 			(last_cmap == cevent->colormap))
 		{
@@ -140,11 +162,7 @@ void HandleColormapNotify(void)
 	return;
 }
 
-/************************************************************************
- *
- * Re-Install the active colormap
- *
- *************************************************************************/
+/* Re-Install the active colormap */
 void ReInstallActiveColormap(void)
 {
 	InstallWindowColormaps(colormap_win);
@@ -152,19 +170,15 @@ void ReInstallActiveColormap(void)
 	return;
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *	InstallWindowColormaps - install the colormaps for one fvwm window
+/*  Procedure:
+ *      InstallWindowColormaps - install the colormaps for one fvwm window
  *
  *  Inputs:
- *	type	- type of event that caused the installation
- *	tmp	- for a subset of event types, the address of the
- *		  window structure, whose colormaps are to be installed.
- *
- ************************************************************************/
-
-void InstallWindowColormaps(FvwmWindow *fw)
+ *      type    - type of event that caused the installation
+ *      tmp     - for a subset of event types, the address of the
+ *                window structure, whose colormaps are to be installed.
+ */
+void InstallWindowColormaps(const FvwmWindow *fw)
 {
 	int i;
 	XWindowAttributes attributes;
@@ -206,7 +220,7 @@ void InstallWindowColormaps(FvwmWindow *fw)
 
 			/*
 			 * On Sun X servers, don't install 24 bit TrueColor
-			 * colourmaps.	Despite what the server says, these
+			 * colourmaps.  Despite what the server says, these
 			 * colourmaps are always installed. */
 			if (last_cmap != attributes.colormap
 #if defined(sun) && defined(TRUECOLOR_ALWAYS_INSTALLED)
@@ -239,20 +253,17 @@ void InstallWindowColormaps(FvwmWindow *fw)
 }
 
 
-/***********************************************************************
+/* Force (un)loads root colormap(s)
  *
- *  Procedures:
- *	<Uni/I>nstallRootColormap - Force (un)loads root colormap(s)
+ * These matching routines provide a mechanism to insure that
+ * the root colormap(s) is installed during operations like
+ * rubber banding that require colors from
+ * that colormap.  Calls may be nested arbitrarily deeply,
+ * as long as there is one UninstallRootColormap call per
+ * InstallRootColormap call.
  *
- *	   These matching routines provide a mechanism to insure that
- *	   the root colormap(s) is installed during operations like
- *	   rubber banding that require colors from
- *	   that colormap.  Calls may be nested arbitrarily deeply,
- *	   as long as there is one UninstallRootColormap call per
- *	   InstallRootColormap call.
- *
- *	   {Uni,I}nstall{Root,Fvwm}Colormap calls may be freely intermixed
- ***********************************************************************/
+ * {Uni,I}nstall{Root,Fvwm}Colormap calls may be freely intermixed
+ */
 void InstallRootColormap(void)
 {
 	if (last_cmap != DefaultColormap(dpy, Scr.screen))
@@ -265,13 +276,10 @@ void InstallRootColormap(void)
 	return;
 }
 
-/***************************************************************************
- *
- * Unstacks one layer of root colormap pushing
+/* Unstacks one layer of root colormap pushing
  * If we peel off the last layer, re-install the application colormap
  * or the fvwm colormap if fvwm has a menu posted
- *
- ***************************************************************************/
+ */
 void UninstallRootColormap(void)
 {
 	if (Scr.root_pushes)
@@ -295,14 +303,11 @@ void UninstallRootColormap(void)
 	return;
 }
 
-/***********************************************************************
- *
- *  Procedures:
- *	<Uni/I>nstallFvwmColormap - Force (un)loads fvwm colormap(s)
- *	This is used to ensure the fvwm colormap is installed during
- *	menu operations
- *
- ***********************************************************************/
+/*  Procedures:
+ *      {Uni/I}nstallFvwmColormap - Force (un)loads fvwm colormap(s)
+ *      This is used to ensure the fvwm colormap is installed during
+ *      menu operations
+ */
 void InstallFvwmColormap(void)
 {
 	if (last_cmap != Pcmap)
@@ -338,19 +343,17 @@ void UninstallFvwmColormap(void)
 	return;
 }
 
-/*****************************************************************************
+/* Gets the WM_COLORMAP_WINDOWS property from the window
  *
- * Gets the WM_COLORMAP_WINDOWS property from the window
- *   This property typically doesn't exist, but a few applications
- *   use it. These seem to occur mostly on SGI machines.
- *
- ****************************************************************************/
+ * This property typically doesn't exist, but a few applications
+ * use it. These seem to occur mostly on SGI machines.
+ */
 void FetchWmColormapWindows (FvwmWindow *fw)
 {
-	XWindowAttributes      getattribs;
-	XSetWindowAttributes   setattribs;
-	long int	       i;
-	unsigned long	       valuemask;
+	XWindowAttributes getattribs;
+	XSetWindowAttributes setattribs;
+	long int i;
+	unsigned long valuemask;
 
 	if (fw->cmap_windows != (Window *)NULL)
 	{
@@ -385,16 +388,13 @@ void FetchWmColormapWindows (FvwmWindow *fw)
 	return;
 }
 
-/*****************************************************************************
- *
- * Looks through the window list for any matching COLORMAP_WINDOWS
+/* Looks through the window list for any matching COLORMAP_WINDOWS
  *   windows and installs the colormap if one exists.
- *
- ****************************************************************************/
+ */
 void EnterSubWindowColormap(Window win)
 {
-	FvwmWindow	   *t;
-	long int	    i;
+	FvwmWindow         *t;
+	long int            i;
 	XWindowAttributes   attribs;
 
 	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
