@@ -46,6 +46,7 @@
 #include "screen.h"
 #include "module_interface.h"
 #include "execcontext.h"
+#include "builtins.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -89,7 +90,7 @@ static char *gray = "gray";
 
 static struct junklist *junk = NULL;
 static Bool cleanup_scheduled = False;
-static struct root_pic root_pic = {None, 0, 0};
+static struct root_pic root_pic = {None, None, 0, 0};
 
 static char *csetopts[] =
 {
@@ -1055,15 +1056,49 @@ void parse_colorset(int n, char *line)
 	/*
 	 * ---------- change the background colour ----------
 	 */
-	if (has_bg_changed || has_pixmap_changed)
+	if (has_bg_changed ||
+	    (has_pixmap_changed && (cs->color_flags & BG_AVERAGE) &&
+	     cs->pixmap != None &&
+	     cs->pixmap != ParentRelative &&
+	     !pixmap_is_a_bitmap))
 	{
 		Bool do_set_default_background = False;
+		Pixmap average_pix = None;
+
+		if (cs->color_flags & BG_AVERAGE)
+		{
+			average_pix =
+			   (cs->picture != NULL &&
+			    cs->picture->picture != None) ?
+				cs->picture->picture : cs->pixmap;
+			if (average_pix == root_pic.pixmap)
+			{
+				int w,h;
+				XID dummy;
+
+				XGrabServer(dpy);
+				if (!XGetGeometry(
+				    dpy, average_pix, &dummy,
+				    (int *)&dummy, (int *)&dummy,
+				    (unsigned int *)&w, (unsigned int *)&h,
+				    (unsigned int *)&dummy,
+				    (unsigned int *)&dummy))
+				{
+					average_pix = None;
+				}
+				if (w != cs->width || h != cs->height)
+				{
+					average_pix = None;
+				}
+				if (average_pix == None)
+				{
+					XUngrabServer(dpy);
+				}
+			}
+		}
 
 		/* note: no average for bitmap */
-		if ((cs->color_flags & BG_AVERAGE) &&
-		    cs->pixmap != None &&
-		    cs->pixmap != ParentRelative &&
-		    !pixmap_is_a_bitmap)
+		if ((cs->color_flags & BG_AVERAGE) && average_pix)
 		{
 			/* calculate average background color */
 			XColor *colors;
@@ -1073,19 +1108,14 @@ void parse_colorset(int n, char *line)
 			unsigned long red = 0, blue = 0, green = 0;
 			unsigned long tred, tblue, tgreen;
 			double dred = 0.0, dblue = 0.0, dgreen = 0.0;
-			Pixmap pix;
 
-			/* chose the good pixmap */
-			pix =
-			   (cs->picture != NULL &&
-			    cs->picture->picture != None) ?
-				cs->picture->picture : cs->pixmap;
+			has_bg_changed = True;
 			/* create an array to store all the pixmap colors in */
 			colors = (XColor *)safemalloc(
 				cs->width * cs->height * sizeof(XColor));
 			/* get the pixmap and mask into an image */
 			image = XGetImage(
-				dpy, pix, 0, 0, cs->width, cs->height,
+				dpy, average_pix, 0, 0, cs->width, cs->height,
 				AllPlanes, ZPixmap);
 			if (cs->mask != None)
 			{
@@ -1093,6 +1123,10 @@ void parse_colorset(int n, char *line)
 					dpy, cs->mask, 0, 0, cs->width,
 					cs->height, AllPlanes, ZPixmap);
 			}
+			if (average_pix == root_pic.pixmap)
+			{
+				XUngrabServer(dpy);
+			}
 			/* only fetch the pixels that are not masked out */
 			for (i = 0; i < cs->width; i++)
 			{
@@ -1187,10 +1221,7 @@ void parse_colorset(int n, char *line)
 				have_pixels_changed = True;
 			}
 		} /* user specified */
-		else if ((bg == NULL && has_bg_changed) ||
-			 ((cs->color_flags & BG_AVERAGE) &&
-			  (cs->pixmap == None || cs->pixmap == ParentRelative ||
-			   pixmap_is_a_bitmap)))
+		else if (bg == NULL && has_bg_changed)
 		{
 			/* default */
 			do_set_default_background = True;
@@ -1205,11 +1236,14 @@ void parse_colorset(int n, char *line)
 			{
 				have_pixels_changed = True;
 			}
+			has_bg_changed = True;
 		}
-		has_bg_changed = True;
 
-		/* save the bg color for tinting */
-		cs->bg_saved = cs->bg;
+		if (has_bg_changed)
+		{
+			/* save the bg color for tinting */
+			cs->bg_saved = cs->bg;
+		}
 	} /* has_bg_changed */
 
 
@@ -1690,15 +1724,22 @@ void update_root_transparent_colorset(Atom prop)
 #endif
 	for (i=0; i < nColorsets; i++)
 	{
+		Bool root_trans = False;
 		cs = &Colorset[i];
 		if (cs->is_maybe_root_transparent &&
 		    cs->allows_buffered_transparency)
 		{
 			parse_colorset(i, "RootTransparent buffer");
+			root_trans = True;
 		}
 		else if (cs->is_maybe_root_transparent)
 		{
 			parse_colorset(i, "RootTransparent");
+			root_trans = True;
+		}
+		if (root_trans)
+		{
+			update_fvwm_colorset(i);
 		}
 	}
 }
