@@ -71,11 +71,12 @@ enum
 	F_EDGE_SCROLL,
 	F_EMULATE,
 	F_ESCAPE_FUNC,
-	F_EWMH_BASE_STRUT,
+	F_EWMH_BASE_STRUTS,
 	F_EWMH_NUMBER_OF_DESKTOPS,
 	F_EXEC,
 	F_EXEC_SETUP,
 	F_FAKE_CLICK,
+	F_FOCUSSTYLE,
 	F_FUNCTION,
 	F_GLOBAL_OPTS,
 	F_GOTO_DESK,
@@ -90,6 +91,7 @@ enum
 	F_KEY,
 	F_KILL_MODULE,
 	F_LAYER,
+	F_LOCALE_PATH,
 	F_MENUSTYLE,
 	F_MODULE,
 	F_MODULE_PATH,
@@ -181,9 +183,12 @@ enum
 	F_RESIZE_MAXIMIZE,
 	F_RESIZEMOVE,
 	F_RESIZEMOVE_MAXIMIZE,
+	F_RESTACKTRANSIENTS,
 	F_SEND_STRING,
 	F_STATE,
 	F_STICK,
+	F_STICKDESK,
+	F_STICKPAGE,
 	F_UPDATE_DECOR,
 	F_WARP,
 	F_WINDOWID,
@@ -262,13 +267,14 @@ void CMD_EdgeScroll(F_CMD_ARGS);
 void CMD_EdgeThickness(F_CMD_ARGS);
 void CMD_Emulate(F_CMD_ARGS);
 void CMD_EscapeFunc(F_CMD_ARGS);
-void CMD_EwmhBaseStrut(F_CMD_ARGS);
+void CMD_EwmhBaseStruts(F_CMD_ARGS);
 void CMD_EwmhNumberOfDesktops(F_CMD_ARGS);
 void CMD_Exec(F_CMD_ARGS);
 void CMD_ExecUseShell(F_CMD_ARGS);
 void CMD_FakeClick(F_CMD_ARGS);
 void CMD_FlipFocus(F_CMD_ARGS);
 void CMD_Focus(F_CMD_ARGS);
+void CMD_FocusStyle(F_CMD_ARGS);
 void CMD_Function(F_CMD_ARGS);
 void CMD_GlobalOpts(F_CMD_ARGS);
 #ifdef GNOME
@@ -289,6 +295,7 @@ void CMD_ImagePath(F_CMD_ARGS);
 void CMD_Key(F_CMD_ARGS);
 void CMD_KillModule(F_CMD_ARGS);
 void CMD_Layer(F_CMD_ARGS);
+void CMD_LocalePath(F_CMD_ARGS);
 void CMD_Lower(F_CMD_ARGS);
 void CMD_Maximize(F_CMD_ARGS);
 void CMD_Menu(F_CMD_ARGS);
@@ -334,6 +341,7 @@ void CMD_Resize(F_CMD_ARGS);
 void CMD_ResizeMaximize(F_CMD_ARGS);
 void CMD_ResizeMove(F_CMD_ARGS);
 void CMD_ResizeMoveMaximize(F_CMD_ARGS);
+void CMD_RestackTransients(F_CMD_ARGS);
 void CMD_Restart(F_CMD_ARGS);
 void CMD_SaveQuitSession(F_CMD_ARGS);
 void CMD_SaveSession(F_CMD_ARGS);
@@ -352,6 +360,8 @@ void CMD_SnapAttraction(F_CMD_ARGS);
 void CMD_SnapGrid(F_CMD_ARGS);
 void CMD_State(F_CMD_ARGS);
 void CMD_Stick(F_CMD_ARGS);
+void CMD_StickDesk(F_CMD_ARGS);
+void CMD_StickPage(F_CMD_ARGS);
 #ifdef HAVE_STROKE
 void CMD_Stroke(F_CMD_ARGS);
 void CMD_StrokeFunc(F_CMD_ARGS);
