@@ -7,12 +7,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #ifndef _EWMH_
@@ -26,16 +26,15 @@ Bool EWMH_CMD_Style(char *token, window_style *ptmpstyle);
 
 /* for maximize and placement ewmh style */
 #define EWMH_IGNORE_WORKING_AREA      0
-#define EWMH_USE_WORKING_AREA	      1
+#define EWMH_USE_WORKING_AREA         1
 #define EWMH_USE_DYNAMIC_WORKING_AREA 2
-#define EWMH_WORKING_AREA_MASK	      3
+#define EWMH_WORKING_AREA_MASK        3
 
 #ifdef HAVE_EWMH
 /* Extended window manager hints support */
 
 #include <X11/Xmd.h>
 
-
 void EWMH_SetCurrentDesktop(void);
 void EWMH_SetNumberOfDesktops(void);
 
@@ -48,10 +47,10 @@ void EWMH_ManageKdeSysTray(Window w, int type);
 void EWMH_SetClientList(void);
 void EWMH_SetClientListStacking(void);
 void EWMH_UpdateWorkArea(void);
-void EWMH_GetWorkAreaIntersection(FvwmWindow *fwin,
-				  int *x, int *y, int *h, int *w, int func);
-float EWMH_GetStrutIntersection(int x11, int y11, int x12, int y12,
-				Bool use_percent);
+void EWMH_GetWorkAreaIntersection(
+	FvwmWindow *fwin, int *x, int *y, int *h, int *w, int func);
+float EWMH_GetStrutIntersection(
+	int x11, int y11, int x12, int y12, Bool use_percent);
 void EWMH_SetFrameStrut(FvwmWindow *fwin);
 void EWMH_SetAllowedActions(FvwmWindow *fwin);
 
@@ -69,8 +68,8 @@ void EWMH_ExitStuff(void);
 /* ewmh_conf.c */
 
 /* ewmh_events.c */
-Bool EWMH_ProcessClientMessage(FvwmWindow *fwin, XEvent *ev);
-void EWMH_ProcessPropertyNotify(FvwmWindow *fwin, XEvent *ev);
+Bool EWMH_ProcessClientMessage(const exec_context_t *exc);
+void EWMH_ProcessPropertyNotify(const exec_context_t *exc);
 
 /* ewmh_icon.c */
 void EWMH_DeleteWmIcon(FvwmWindow *fwin, Bool mini_icon, Bool icon);
@@ -118,8 +117,8 @@ void EWMH_SetDesktopNames(void);
 /* ewmh_conf.c */
 
 /* ewmh_events.c */
-#define EWMH_ProcessClientMessage(x, y) 0
-#define EWMH_ProcessPropertyNotify(x, y)
+#define EWMH_ProcessClientMessage(x) 0
+#define EWMH_ProcessPropertyNotify(x)
 
 /* ewmh_icon.c */
 #define EWMH_DoUpdateWmIcon(x,y,z)
@@ -128,7 +127,7 @@ void EWMH_SetDesktopNames(void);
 
 /* ewmh_name.c */
 #define EWMH_SetVisibleName(x,y)
-#define EWMH_WMName(x,y,z,t)	 0
+#define EWMH_WMName(x,y,z,t)     0
 #define EWMH_WMIconName(x,y,z,t) 0
 #define EWMH_SetDesktopNames()
 #endif
