@@ -6,12 +6,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 /* ---------------------------- included header files ----------------------- */
 
@@ -33,6 +33,7 @@
 #include "virtual.h"
 #include "colorset.h"
 #include "schedule.h"
+#include "libs/FGettext.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -57,6 +58,7 @@ static char *function_vars[] =
 	"hilight.cs",
 	"shadow.cs",
 	"fgsh.cs",
+	"gt.",
 	"desk.name",
 	"desk.width",
 	"desk.height",
@@ -106,6 +108,7 @@ enum
 	VAR_HILIGHT_CS,
 	VAR_SHADOW_CS,
 	VAR_FGSH_CS,
+	VAR_GT,
 	VAR_DESK_NAME,
 	VAR_DESK_WIDTH,
 	VAR_DESK_HEIGHT,
@@ -153,7 +156,7 @@ enum
 
 static int expand_vars_extended(
 	char *var_name, char *output, FvwmWindow *fw,
-        fvwm_cond_func_rc *cond_rc)
+	fvwm_cond_func_rc *cond_rc)
 {
 	char *s;
 	char *rest;
@@ -217,6 +220,25 @@ static int expand_vars_extended(
 			break;
 		}
 		return pixel_to_color_string(dpy, Pcmap, pixel, target, False);
+	case VAR_GT:
+	{
+		const char *gt;
+
+		if (rest == NULL)
+		{
+			return 0;
+		}
+		gt = _(rest);
+		if (gt == NULL)
+		{
+			return 0;
+		}
+		if (output)
+		{
+			strcpy(output, gt);
+		}
+		return strlen(gt);
+	}
 	case VAR_DESK_NAME:
 		if (sscanf(rest, "%d%n", &cs, &n) < 1)
 		{
@@ -230,8 +252,9 @@ static int expand_vars_extended(
 		s = GetDesktopName(cs);
 		if (s == NULL)
 		{
-			s = (char *)safemalloc(23 * sizeof(char));
-			sprintf(s, "Desk %i", cs);
+			const char *ddn = _("Desk");
+			s = (char *)safemalloc(19 + strlen(ddn));
+			sprintf(s, "%s %i", ddn, cs);
 			l = strlen(s);
 			if (output)
 			{
@@ -531,7 +554,7 @@ static int expand_vars_extended(
 		case VAR_POINTER_CX:
 			is_x = True;
 		}
-		if (XQueryPointer(dpy, context_w, &JunkRoot, &JunkChild,
+		if (FQueryPointer(dpy, context_w, &JunkRoot, &JunkChild,
 				  &JunkX, &JunkY, &x, &y, &JunkMask) == False)
 		{
 			/* pointer is on a different screen, don't expand */
@@ -616,9 +639,9 @@ char *expand_vars(
 						xlen = expand_vars_extended(
 							var, NULL, fw, cond_rc);
 						if (xlen > 0)
-                                                {
+						{
 							l2 += xlen - (k + 2);
-                                                }
+						}
 					}
 					i += k + 2;
 					input[m] = ']';
@@ -636,13 +659,13 @@ char *expand_vars(
 			case '9':
 			case '*':
 				if (input[i+1] == '*')
-                                {
+				{
 					n = 0;
-                                }
+				}
 				else
-                                {
+				{
 					n = input[i+1] - '0' + 1;
-                                }
+				}
 				if (arguments[n] != NULL)
 				{
 					l2 += strlen(arguments[n])-2;
@@ -704,9 +727,9 @@ char *expand_vars(
 				for (k = 0; string[k] != 0; k++, l2++)
 				{
 					if (string[k] == '\'')
-                                        {
+					{
 						l2++;
-                                        }
+					}
 				}
 				string = NULL;
 			}
