@@ -23,26 +23,18 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
-#include "libs/Flocale.h"
-#include <libs/gravity.h>
-#include "libs/fvwmrect.h"
-#include "libs/gravity.h"
 #include "fvwm.h"
 #include "externs.h"
+#include "execcontext.h"
 #include "events.h"
-#include "cursor.h"
-#include "functions.h"
-#include "bindings.h"
 #include "misc.h"
 #include "screen.h"
-#include "defaults.h"
 #include "geometry.h"
 #include "module_interface.h"
-#include "gnome.h"
 #include "focus.h"
-#include "ewmh.h"
 #include "borders.h"
 #include "frame.h"
+extern void GetWindowSizeHints(FvwmWindow *tmp);
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -82,10 +74,12 @@ typedef struct
 	int current_step;
 	int curr_titlebar_compression;
 	direction_type shade_dir;
+	window_parts trans_parts;
 	struct
 	{
 		unsigned do_force : 1;
 		unsigned do_not_configure_client : 1;
+		unsigned do_not_draw : 1;
 		unsigned do_restore_gravity : 1;
 		unsigned do_set_bit_gravity : 1;
 		unsigned do_update_shape : 1;
@@ -212,6 +206,8 @@ static void get_resize_decor_gravities_one_axis(
 	case FRAME_MR_SETUP:
 	case FRAME_MR_SETUP_BY_APP:
 		ret_grav->client_grav = neg_grav;
+	case FRAME_MR_DONT_DRAW:
+		/* can not happen, just a dummy to keep -Wall happy */
 		break;
 	}
 	ret_grav->parent_grav = ret_grav->client_grav;
@@ -405,6 +401,8 @@ static void __frame_setup_window(
 	}
 	else if (is_moved)
 	{
+		unsigned int draw_parts = PART_NONE;
+
 		/* inform the application of the change
 		 *
 		 * According to the July 27, 1988 ICCCM draft, we should send a
@@ -412,6 +410,15 @@ static void __frame_setup_window(
 		 * was moved but not resized. */
 		XMoveWindow(dpy, FW_W_FRAME(fw), frame_g->x, frame_g->y);
 		fw->frame_g = *frame_g;
+		if ((draw_parts = border_get_transparent_decorations_part(fw))
+		    != PART_NONE)
+		{
+			border_draw_decorations(
+				fw, draw_parts,
+				((fw == get_focus_window())) ? True : False,
+				True, CLEAR_ALL, NULL, NULL);
+		}
+		fw->frame_g = *frame_g;
 		do_send_configure_notify = True;
 	}
 	/* must not send events to shaded windows because this might cause them
@@ -593,6 +600,25 @@ static void frame_set_decor_gravities(
 	return;
 }
 
+/* Just restore the win and bit gravities on the client window. */
+static void frame_restore_client_gravities(FvwmWindow *fw)
+{
+	XSetWindowAttributes xcwa;
+	long valuemask;
+
+	valuemask = CWWinGravity;
+	if (fw->attr_backup.is_bit_gravity_stored)
+	{
+		fw->attr_backup.is_bit_gravity_stored = 0;
+		xcwa.bit_gravity = fw->attr_backup.bit_gravity;
+		valuemask = CWBitGravity;
+	}
+	xcwa.win_gravity = fw->hints.win_gravity;
+	XChangeWindowAttributes(dpy, FW_W(fw), valuemask, &xcwa);
+
+	return;
+}
+
 /* Prepares the structure for the next animation step. */
 static void frame_next_move_resize_args(
 	frame_move_resize_args mr_args)
@@ -885,7 +911,11 @@ static void frame_mrs_setup_draw_decorations(
 	frame_setup_titlebar(fw, &mra->next_g, setup_parts, &mra->dstep_g);
 	frame_setup_border(fw, &mra->next_g, PART_ALL, &mra->dstep_g);
 	/* draw the border and the titlebar */
-	if (!mra->flags.is_shading || !mra->flags.is_lazy_shading)
+	if (mra->flags.do_not_draw == 1)
+	{
+		/* nothing */
+	}
+	else if (!mra->flags.is_shading || !mra->flags.is_lazy_shading)
 	{
 		/* draw as usual */
 		border_draw_decorations(
@@ -1073,7 +1103,11 @@ static void frame_has_handles_and_tiled_border(
 	{
 		*ret_has_handles = 0;
 	}
-	*ret_has_tiled_border = (DFS_FACE_TYPE(df->style) == TiledPixmapButton);
+	*ret_has_tiled_border =
+		(DFS_FACE_TYPE(df->style) == TiledPixmapButton) ||
+		(DFS_FACE_TYPE(df->style) == ColorsetButton &&
+		 !CSET_IS_TRANSPARENT_PR(df->u.acs.cs) &&
+		 CSET_HAS_PIXMAP(df->u.acs.cs));
 
 	return;
 }
@@ -1311,10 +1345,6 @@ void frame_get_titlebar_dimensions(
 		tb_w = tb_length;
 		tb_h = tb_thick;
 	}
-	title_layout->titlebar_g.x = tb_x;
-	title_layout->titlebar_g.y = tb_y;
-	title_layout->titlebar_g.width = tb_w;
-	title_layout->titlebar_g.height = tb_h;
 	/* configure left buttons */
 	for (i = 0; i < NUMBER_OF_BUTTONS; i += 2)
 	{
@@ -1597,6 +1627,15 @@ frame_move_resize_args frame_create_move_resize_args(
 	/* set some variables */
 	mra = (mr_args_internal *)safecalloc(1, sizeof(mr_args_internal));
 	memset(mra, 0, sizeof(*mra));
+	if (mr_mode & FRAME_MR_DONT_DRAW)
+	{
+		mr_mode &= ~FRAME_MR_DONT_DRAW;
+		mra->flags.do_not_draw = 1;
+	}
+	else
+	{
+		mra->flags.do_not_draw = 0;
+	}
 	if (mr_mode == FRAME_MR_SETUP_BY_APP)
 	{
 		mr_mode = FRAME_MR_SETUP;
@@ -1681,6 +1720,7 @@ frame_move_resize_args frame_create_move_resize_args(
 	 * if either the border uses a tiled pixmap background. */
 	mra->flags.had_handles = HAS_HANDLES(fw);
 	mra->flags.is_lazy_shading = frame_get_shading_laziness(fw, mra);
+	mra->trans_parts = border_get_transparent_decorations_part(fw);
 	if (mra->flags.is_lazy_shading)
 	{
 		SET_HAS_HANDLES(fw, 0);
@@ -1756,8 +1796,15 @@ void frame_free_move_resize_args(
 			(mra->w_with_focus != None) ? True : False, True,
 			CLEAR_ALL, &mra->current_g, &mra->next_g);
 	}
+	else if (mra->trans_parts != PART_NONE)
+	{
+		border_draw_decorations(
+			fw, mra->trans_parts,
+			(mra->w_with_focus != None) ? True : False, True,
+			CLEAR_ALL, &mra->current_g, &mra->next_g);
+	}
 	update_absolute_geometry(fw);
-	frame_reparent_hide_windows(Scr.Root);
+	frame_reparent_hide_windows(Scr.NoFocusWin);
 	if (mra->w_with_focus != None)
 	{
 		/* domivogt (28-Dec-1999): For some reason the XMoveResize() on
@@ -1776,10 +1823,16 @@ void frame_free_move_resize_args(
 	frame_setup_shape(fw, mra->end_g.width, mra->end_g.height);
 	if (mra->flags.do_restore_gravity)
 	{
+		mra->grav.client_grav = fw->hints.win_gravity;
 		frame_set_decor_gravities(
 			fw, &mra->grav,
 			(mra->flags.do_set_bit_gravity) ? 2 : 0);
 	}
+	else
+	{
+		frame_restore_client_gravities(fw);
+	}
+
 	focus_grab_buttons_on_layer(fw->layer);
 	/* free the memory */
 	free(mr_args);
