@@ -31,17 +31,14 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <errno.h>
-#include "libs/ftime.h"
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
+#include "libs/ftime.h"
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
-#include "libs/Module.h"
-#include "libs/queue.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -49,7 +46,6 @@
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
-#include "colorset.h"
 #include "module_interface.h"
 #include "read.h"
 #include "events.h"
@@ -202,8 +198,8 @@ void ClosePipes(void)
 
 static int do_execute_module(F_CMD_ARGS, Bool desperate)
 {
-	int fvwm_to_app[2],app_to_fvwm[2];
-	int i,val,nargs = 0;
+	int fvwm_to_app[2], app_to_fvwm[2];
+	int i, val, nargs = 0;
 	int ret_pipe;
 	char *cptr;
 	char **args;
@@ -215,6 +211,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 	char *token;
 	extern char *ModulePath;
 	Window win;
+	FvwmWindow * const fw = exc->w.fw;
 
 	args = (char **)safemalloc(7 * sizeof(char *));
 
@@ -237,10 +234,6 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 		win = None;
 	}
 
-	/* If we execute a module, don't wait for buttons to come up,
-	 * that way, a pop-up menu could be implemented */
-	*Module = 0;
-
 	action = GetNextToken(action, &cptr);
 	if (!cptr)
 	{
@@ -248,7 +241,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 		return -1;
 	}
 
-	arg1 = searchPath( ModulePath, cptr, EXECUTABLE_EXTENSION, X_OK );
+	arg1 = searchPath(ModulePath, cptr, EXECUTABLE_EXTENSION, X_OK);
 
 	if (arg1 == NULL)
 	{
@@ -272,8 +265,8 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 	{
 		char *p;
 
-		p = arg1 + strlen( arg1 ) - strlen( EXECUTABLE_EXTENSION );
-		if (strcmp(p, EXECUTABLE_EXTENSION) ==  0)
+		p = arg1 + strlen(arg1) - strlen(EXECUTABLE_EXTENSION);
+		if (strcmp(p, EXECUTABLE_EXTENSION) == 0)
 		{
 			*p = 0;
 		}
@@ -281,14 +274,14 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 #endif
 
 	/* Look for an available pipe slot */
-	i=0;
+	i = 0;
 	while ((i<npipes) && (writePipes[i] >=0))
 	{
 		i++;
 	}
 	if (i>=npipes)
 	{
-		fvwm_msg(ERR,"executeModule","Too many Accessories!");
+		fvwm_msg(ERR, "executeModule", "Too many Accessories!");
 		free(arg1);
 		free(cptr);
 		free(args);
@@ -299,9 +292,9 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 	/* I want one-ended pipes, so I open two two-ended pipes,
 	 * and close one end of each. I need one ended pipes so that
 	 * I can detect when the module crashes/malfunctions */
-	if (pipe(fvwm_to_app)!=0)
+	if (pipe(fvwm_to_app) != 0)
 	{
-		fvwm_msg(ERR,"executeModule","Failed to open pipe");
+		fvwm_msg(ERR, "executeModule", "Failed to open pipe");
 		free(arg1);
 		free(cptr);
 		free(args);
@@ -309,7 +302,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 	}
 	if (pipe(app_to_fvwm)!=0)
 	{
-		fvwm_msg(ERR,"executeModule","Failed to open pipe2");
+		fvwm_msg(ERR, "executeModule", "Failed to open pipe2");
 		free(arg1);
 		free(cptr);
 		close(fvwm_to_app[0]);
@@ -320,10 +313,10 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 
 	pipeName[i] = stripcpy(cptr);
 	free(cptr);
-	sprintf(arg2,"%d",app_to_fvwm[1]);
-	sprintf(arg3,"%d",fvwm_to_app[0]);
-	sprintf(arg5,"%lx",(unsigned long)win);
-	sprintf(arg6,"%lx",(unsigned long)context);
+	sprintf(arg2, "%d", app_to_fvwm[1]);
+	sprintf(arg3, "%d", fvwm_to_app[0]);
+	sprintf(arg5, "%lx", (unsigned long)win);
+	sprintf(arg6, "%lx", (unsigned long)exc->w.wcontext);
 	args[0] = arg1;
 	args[1] = arg2;
 	args[2] = arg3;
@@ -398,7 +391,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 				ERR, "executeModule",
 				"module close-on-exec failed");
 		}
-		for (i=6;i<nargs;i++)
+		for (i = 6; i < nargs; i++)
 		{
 			if (args[i] != 0)
 			{
@@ -408,7 +401,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 	}
 	else if (val ==0)
 	{
-		/* this is  the child */
+		/* this is the child */
 		/* this fork execs the module */
 #ifdef FORK_CREATES_CHILD
 		close(fvwm_to_app[1]);
@@ -427,8 +420,8 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 			putenv(colormap);
 		}
 
-		/** Why is this execvp??  We've already searched the module
-		 * path! **/
+		/* Why is this execvp??  We've already searched the module
+		 * path! */
 		execvp(arg1,args);
 		fvwm_msg(
 			ERR, "executeModule", "Execution of module failed: %s",
@@ -442,9 +435,9 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 	}
 	else
 	{
-		fvwm_msg(ERR,"executeModule","Fork failed");
+		fvwm_msg(ERR, "executeModule", "Fork failed");
 		free(arg1);
-		for (i=6;i<nargs;i++)
+		for (i = 6; i < nargs; i++)
 		{
 			if (args[i] != 0)
 			{
@@ -486,7 +479,6 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 	Bool need_ungrab = False;
 	char *escape = NULL;
 	XEvent tmpevent;
-	extern FvwmWindow *Fw;
 
 	if (!action)
 	{
@@ -616,15 +608,17 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 		}
 
 		/* Check for "escape function" */
-		if (XPending(dpy) &&
-		    XCheckMaskEvent(dpy, KeyPressMask, &tmpevent))
+		if (FPending(dpy) &&
+		    FCheckMaskEvent(dpy, KeyPressMask, &tmpevent))
 		{
 			escape = CheckBinding(
 				Scr.AllBindings, STROKE_ARG(0)
 				tmpevent.xkey.keycode, tmpevent.xkey.state,
 				GetUnusedModifiers(),
-				GetContext(Fw, &tmpevent, &targetWindow),
-				KEY_BINDING);
+				GetContext(
+					NULL, exc->w.fw, &tmpevent,
+					&targetWindow),
+				BIND_KEYPRESS);
 			if (escape != NULL)
 			{
 				if (!strcasecmp(escape,"escapefunc"))
@@ -643,6 +637,65 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 	return;
 }
 
+/* run the command as if it cames from a button press or release */
+static void ExecuteModuleCommand(Window w, int module, char *text)
+{
+	XEvent e;
+	const exec_context_t *exc;
+	exec_context_changes_t ecc;
+
+	if (XFindContext(dpy, w, FvwmContext, (caddr_t *)&ecc.w.fw) == XCNOENT)
+	{
+		ecc.w.fw = NULL;
+		w = None;
+	}
+	/* Query the pointer, the pager-drag-out feature doesn't work properly.
+	 * This is OK now that the Pager uses "Move pointer"
+	 * A real fix would be for the modules to pass the button press coords
+	 */
+	if (FQueryPointer(
+		    dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX,&JunkY,
+		    &e.xbutton.x_root, &e.xbutton.y_root, &JunkMask) ==
+	    False)
+	{
+		/* pointer is not on this screen */
+		/* If a module does XUngrabPointer(), it can now get proper
+		 * Popups */
+		e.xbutton.window = Scr.Root;
+		ecc.w.fw = NULL;
+	}
+	else
+	{
+		e.xbutton.window = w;
+	}
+	e.xbutton.subwindow = None;
+	e.xbutton.button = 1;
+	/* If a module does XUngrabPointer(), it can now get proper Popups */
+	if (StrEquals(text, "popup"))
+	{
+		e.xbutton.type = ButtonPress;
+	}
+	else
+	{
+		e.xbutton.type = ButtonRelease;
+	}
+	e.xbutton.x = 0;
+	e.xbutton.y = 0;
+	fev_fake_event(&e);
+	ecc.type = EXCT_MODULE;
+	ecc.w.w = w;
+	ecc.w.wcontext = GetContext(NULL, ecc.w.fw, &e, &w);
+	ecc.x.etrigger = &e;
+	ecc.m.modnum = module;
+	exc = exc_create_context(
+		&ecc, ECC_TYPE | ECC_ETRIGGER | ECC_FW | ECC_W | ECC_WCONTEXT |
+		ECC_MODNUM);
+	execute_function(NULL, exc, text, 0);
+	exc_destroy_context(exc);
+
+	return;
+}
+
 /* read input from a module and either execute it or queue it
  * returns True and does NOP if the module input matches the expect string */
 Bool HandleModuleInput(Window w, int module, char *expect, Bool queue)
@@ -658,9 +711,10 @@ Bool HandleModuleInput(Window w, int module, char *expect, Bool queue)
 	n = read(channel, &size, sizeof(size));
 	if (n < sizeof(size))
 	{
-		fvwm_msg(ERR, "HandleModuleInput",
-			 "Fail to read (Module: %i, read: %i, size: %i)",
-			 module, n, sizeof(size));
+		fvwm_msg(
+			ERR, "HandleModuleInput",
+			"Fail to read (Module: %i, read: %i, size: %i)",
+			module, n, sizeof(size));
 		KillModule(module);
 		return False;
 	}
@@ -726,63 +780,9 @@ Bool HandleModuleInput(Window w, int module, char *expect, Bool queue)
 	return False;
 }
 
-/* run the command as if it cames from a button press or release */
-void ExecuteModuleCommand(Window w, int module, char *text)
-{
-	int context;
-	FvwmWindow *fw;
-
-	if (XFindContext (dpy, w, FvwmContext, (caddr_t *) &fw) == XCNOENT)
-	{
-		fw = NULL;
-		w = None;
-	}
-
-	/* Query the pointer, the pager-drag-out feature doesn't work properly.
-	 * This is OK now that the Pager uses "Move pointer"
-	 * A real fix would be for the modules to pass the button press coords
-	 */
-	if (XQueryPointer(
-		    dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX,&JunkY,
-		    &Event.xbutton.x_root,&Event.xbutton.y_root,&JunkMask) ==
-	    False)
-	{
-		/* pointer is not on this screen */
-		/* If a module does XUngrabPointer(), it can now get proper
-		 * Popups */
-		Event.xbutton.window = Scr.Root;
-		Event.xbutton.button = 1;
-		Event.xbutton.subwindow = None;
-		fw = NULL;
-	}
-	else
-	{
-		Event.xbutton.window = w;
-		Event.xbutton.button = 1;
-		Event.xbutton.subwindow = None;
-		context = GetContext(fw,&Event,&w);
-	}
-	/* If a module does XUngrabPointer(), it can now get proper Popups */
-	if (StrEquals(text, "popup"))
-	{
-		Event.xbutton.type = ButtonPress;
-	}
-	else
-	{
-		Event.xbutton.type = ButtonRelease;
-	}
-	Event.xbutton.x = 0;
-	Event.xbutton.y = 0;
-	context = GetContext(fw, &Event, &w);
-	old_execute_function(NULL, text, fw, &Event, context, module, 0, NULL);
-
-	return;
-}
-
-
 RETSIGTYPE DeadPipe(int sig)
-{}
-
+{
+}
 
 void KillModule(int channel)
 {
@@ -860,7 +860,6 @@ static unsigned long *
 make_vpacket(unsigned long *body, unsigned long event_type,
 	     unsigned long num, va_list ap)
 {
-	extern Time lastTimestamp;
 	unsigned long *bp = body;
 
 	/* truncate long packets */
@@ -871,7 +870,7 @@ make_vpacket(unsigned long *body, unsigned long event_type,
 	*(bp++) = START_FLAG;
 	*(bp++) = event_type;
 	*(bp++) = num+FvwmPacketHeaderSize;
-	*(bp++) = lastTimestamp;
+	*(bp++) = fev_get_evtime();
 
 	for (; num > 0; --num)
 	{
@@ -897,7 +896,6 @@ static unsigned long
 make_new_vpacket(unsigned char *body, unsigned long event_type,
 		 unsigned long num, va_list ap)
 {
-	extern Time lastTimestamp;
 	unsigned long arglen;
 	unsigned long bodylen = 0;
 	unsigned long *bp = (unsigned long *)body;
@@ -910,7 +908,7 @@ make_new_vpacket(unsigned char *body, unsigned long event_type,
 	*(bp++) = event_type;
 	/*  Skip length field, we don't know it yet. */
 	bp++;
-	*(bp++) = lastTimestamp;
+	*(bp++) = fev_get_evtime();
 
 	for (; num > 0; --num)
 	{
@@ -1330,37 +1328,39 @@ BroadcastName(unsigned long event_type,
 }
 
 void
-BroadcastWindowIconNames(FvwmWindow *t, Bool window, Bool icon)
+BroadcastWindowIconNames(FvwmWindow *fw, Bool window, Bool icon)
 {
 	if (window)
 	{
 		BroadcastName(
-			M_WINDOW_NAME,FW_W(t), FW_W_FRAME(t), (unsigned long)t,
-			t->name.name);
+			M_WINDOW_NAME, FW_W(fw), FW_W_FRAME(fw),
+			(unsigned long)fw, fw->name.name);
 		BroadcastName(
-			M_VISIBLE_NAME,FW_W(t),FW_W_FRAME(t),(unsigned long)t,
-			t->visible_name);
+			M_VISIBLE_NAME, FW_W(fw), FW_W_FRAME(fw),
+			(unsigned long)fw, fw->visible_name);
 	}
 	if (icon)
 	{
 		BroadcastName(
-			M_ICON_NAME, FW_W(t), FW_W_FRAME(t), (unsigned long)t,
-			t->icon_name.name);
+			M_ICON_NAME, FW_W(fw), FW_W_FRAME(fw),
+			(unsigned long)fw, fw->icon_name.name);
 		BroadcastName(
-			MX_VISIBLE_ICON_NAME,FW_W(t),FW_W_FRAME(t),
-			(unsigned long)t, t->visible_icon_name);
+			MX_VISIBLE_ICON_NAME, FW_W(fw), FW_W_FRAME(fw),
+			(unsigned long)fw, fw->visible_icon_name);
 	}
 
 	return;
 }
 
 void
-SendFvwmPicture(int module, unsigned long event_type,
-		unsigned long data1, unsigned long data2,
-		unsigned long data3, FvwmPicture *picture, char *name)
+SendFvwmPicture(
+	int module, unsigned long event_type, unsigned long data1,
+	unsigned long data2, unsigned long data3, FvwmPicture *picture,
+	char *name)
 {
 	unsigned long *body;
-	unsigned int data4 = 0, data5 = 0, data6 = 0,
+	unsigned int
+		data4 = 0, data5 = 0, data6 = 0,
 		data7 = 0, data8 = 0, data9 = 0;
 	int l;
 
@@ -1393,9 +1393,9 @@ SendFvwmPicture(int module, unsigned long event_type,
 }
 
 void
-BroadcastFvwmPicture(unsigned long event_type,
-		     unsigned long data1, unsigned long data2,
-		     unsigned long data3, FvwmPicture *picture, char *name)
+BroadcastFvwmPicture(
+	unsigned long event_type, unsigned long data1, unsigned long data2,
+	unsigned long data3, FvwmPicture *picture, char *name)
 {
 	unsigned long *body;
 	unsigned int data4, data5, data6, data7, data8, data9;
@@ -1533,6 +1533,7 @@ void CMD_SendToModule(F_CMD_ARGS)
 	char *module,*str;
 	unsigned long data0, data1, data2;
 	int i;
+	FvwmWindow * const fw = exc->w.fw;
 
 	/* FIXME: Without this, popup menus can't be implemented properly in
 	 *  modules.  Olivier: Why ? */
@@ -1929,118 +1930,112 @@ void ExecuteCommandQueue(void)
 void CMD_Send_WindowList(F_CMD_ARGS)
 {
 	FvwmWindow *t;
+	int mod = exc->m.modnum;
 
-	if (*Module >= 0)
+	if (mod < 0)
+	{
+		return;
+	}
+	SendPacket(mod, M_NEW_DESK, 1, Scr.CurrentDesk);
+	SendPacket(
+		mod, M_NEW_PAGE, 5, Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax,
+		Scr.VyMax);
+
+	if (Scr.Hilite != NULL)
 	{
-		SendPacket(*Module, M_NEW_DESK, 1, Scr.CurrentDesk);
 		SendPacket(
-			*Module, M_NEW_PAGE, 5,
-			Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax, Scr.VyMax);
+			mod, M_FOCUS_CHANGE, 5, FW_W(Scr.Hilite),
+			FW_W_FRAME(Scr.Hilite), (unsigned long)True,
+			Scr.Hilite->hicolors.fore, Scr.Hilite->hicolors.back);
+	}
+	else
+	{
+		SendPacket(
+			mod, M_FOCUS_CHANGE, 5, 0, 0, (unsigned long)True,
+			GetColor(DEFAULT_FORE_COLOR),
+			GetColor(DEFAULT_BACK_COLOR));
+	}
+	if (Scr.DefaultIcon != NULL)
+	{
+		SendName(mod, M_DEFAULTICON, 0, 0, 0, Scr.DefaultIcon);
+	}
 
-		if (Scr.Hilite != NULL)
-		{
-			SendPacket(
-				*Module, M_FOCUS_CHANGE, 5, FW_W(Scr.Hilite),
-				FW_W_FRAME(Scr.Hilite), (unsigned long)True,
-				Scr.Hilite->hicolors.fore,
-				Scr.Hilite->hicolors.back);
-		}
-		else
-		{
-			SendPacket(
-				*Module, M_FOCUS_CHANGE, 5, 0, 0,
-				(unsigned long)True,
-				GetColor(DEFAULT_FORE_COLOR),
-				GetColor(DEFAULT_BACK_COLOR));
-		}
-		if (Scr.DefaultIcon != NULL)
+	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	{
+		SendConfig(mod,M_CONFIGURE_WINDOW,t);
+		SendName(
+			mod, M_WINDOW_NAME, FW_W(t), FW_W_FRAME(t),
+			(unsigned long)t, t->name.name);
+		SendName(
+			mod, M_ICON_NAME, FW_W(t), FW_W_FRAME(t),
+			(unsigned long)t, t->icon_name.name);
+		SendName(
+			mod, M_VISIBLE_NAME, FW_W(t), FW_W_FRAME(t),
+			(unsigned long)t, t->visible_name);
+		SendName(
+			mod, MX_VISIBLE_ICON_NAME, FW_W(t), FW_W_FRAME(t),
+			(unsigned long)t,t->visible_icon_name);
+		if (t->icon_bitmap_file != NULL
+		    && t->icon_bitmap_file != Scr.DefaultIcon)
 		{
 			SendName(
-				*Module, M_DEFAULTICON, 0, 0, 0,
-				Scr.DefaultIcon);
+				mod, M_ICON_FILE, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, t->icon_bitmap_file);
 		}
 
-		for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+		SendName(
+			mod, M_RES_CLASS, FW_W(t), FW_W_FRAME(t),
+			(unsigned long)t, t->class.res_class);
+		SendName(
+			mod, M_RES_NAME, FW_W(t), FW_W_FRAME(t),
+			(unsigned long)t, t->class.res_name);
+
+		if (IS_ICONIFIED(t) && !IS_ICON_UNMAPPED(t))
 		{
-			SendConfig(*Module,M_CONFIGURE_WINDOW,t);
-			SendName(
-				*Module, M_WINDOW_NAME, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, t->name.name);
-			SendName(
-				*Module, M_ICON_NAME, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, t->icon_name.name);
-			SendName(
-				*Module, M_VISIBLE_NAME, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, t->visible_name);
-			SendName(
-				*Module, MX_VISIBLE_ICON_NAME, FW_W(t),
-				FW_W_FRAME(t),
-				(unsigned long)t,t->visible_icon_name);
-			if (t->icon_bitmap_file != NULL
-			    && t->icon_bitmap_file != Scr.DefaultIcon)
-			{
-				SendName(*Module, M_ICON_FILE, FW_W(t),
-					 FW_W_FRAME(t), (unsigned long)t,
-					 t->icon_bitmap_file);
-			}
+			rectangle r;
+			Bool rc;
 
-			SendName(
-				*Module, M_RES_CLASS, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, t->class.res_class);
-			SendName(
-				*Module, M_RES_NAME, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, t->class.res_name);
-
-			if ((IS_ICONIFIED(t))&&(!IS_ICON_UNMAPPED(t)))
-			{
-				rectangle r;
-				Bool rc;
-
-				rc = get_visible_icon_geometry(t, &r);
-				if (rc == True)
-				{
-					SendPacket(
-						*Module, M_ICONIFY, 7, FW_W(t),
-						FW_W_FRAME(t), (unsigned long)t,
-						r.x, r.y, r.width, r.height);
-				}
-			}
-
-			if ((IS_ICONIFIED(t))&&(IS_ICON_UNMAPPED(t)))
+			rc = get_visible_icon_geometry(t, &r);
+			if (rc == True)
 			{
 				SendPacket(
-					*Module, M_ICONIFY, 7, FW_W(t),
-					FW_W_FRAME(t), (unsigned long)t, 0, 0,
-					0, 0);
-			}
-			if (FMiniIconsSupported && t->mini_icon != NULL)
-			{
-				SendFvwmPicture(
-					*Module, M_MINI_ICON, FW_W(t),
-					FW_W_FRAME(t), (unsigned long)t,
-					t->mini_icon, t->mini_pixmap_file);
+					mod, M_ICONIFY, 7, FW_W(t),
+					FW_W_FRAME(t), (unsigned long)t, r.x,
+					r.y, r.width, r.height);
 			}
 		}
-
-		if (Scr.Hilite == NULL)
+		if ((IS_ICONIFIED(t))&&(IS_ICON_UNMAPPED(t)))
 		{
-			BroadcastPacket(
-				M_FOCUS_CHANGE, 5, 0, 0, (unsigned long)True,
-				GetColor(DEFAULT_FORE_COLOR),
-				GetColor(DEFAULT_BACK_COLOR));
+			SendPacket(
+				mod, M_ICONIFY, 7, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, 0, 0, 0, 0);
 		}
-		else
+		if (FMiniIconsSupported && t->mini_icon != NULL)
 		{
-			BroadcastPacket(
-				M_FOCUS_CHANGE, 5, FW_W(Scr.Hilite),
-				FW_W(Scr.Hilite), (unsigned long)True,
-				Scr.Hilite->hicolors.fore,
-				Scr.Hilite->hicolors.back);
+			SendFvwmPicture(
+				mod, M_MINI_ICON, FW_W(t), FW_W_FRAME(t),
+				(unsigned long)t, t->mini_icon,
+				t->mini_pixmap_file);
 		}
-
-		SendPacket(*Module, M_END_WINDOWLIST, 0);
 	}
 
+	if (Scr.Hilite == NULL)
+	{
+		BroadcastPacket(
+			M_FOCUS_CHANGE, 5, 0, 0, (unsigned long)True,
+			GetColor(DEFAULT_FORE_COLOR),
+			GetColor(DEFAULT_BACK_COLOR));
+	}
+	else
+	{
+		BroadcastPacket(
+			M_FOCUS_CHANGE, 5, FW_W(Scr.Hilite), FW_W(Scr.Hilite),
+			(unsigned long)True, Scr.Hilite->hicolors.fore,
+			Scr.Hilite->hicolors.back);
+	}
+
+	SendPacket(mod, M_END_WINDOWLIST, 0);
+
 	return;
 }
 
@@ -2048,12 +2043,15 @@ void CMD_set_mask(F_CMD_ARGS)
 {
 	unsigned long val;
 
-	/*GetIntegerArguments(action, NULL, &val, 1);*/
-	if (!action || sscanf(action,"%lu",&val) != 1)
+	if (exc->m.modnum < 0)
+	{
+		return;
+	}
+	if (!action || sscanf(action, "%lu", &val) != 1)
 	{
 		val = 0;
 	}
-	set_message_mask(&PipeMask[*Module], (unsigned long)val);
+	set_message_mask(&PipeMask[exc->m.modnum], (unsigned long)val);
 
 	return;
 }
@@ -2062,11 +2060,15 @@ void CMD_set_sync_mask(F_CMD_ARGS)
 {
 	unsigned long val;
 
+	if (exc->m.modnum < 0)
+	{
+		return;
+	}
 	if (!action || sscanf(action,"%lu",&val) != 1)
 	{
 		val = 0;
 	}
-	set_message_mask(&SyncMask[*Module], (unsigned long)val);
+	set_message_mask(&SyncMask[exc->m.modnum], (unsigned long)val);
 
 	return;
 }
@@ -2075,11 +2077,15 @@ void CMD_set_nograb_mask(F_CMD_ARGS)
 {
 	unsigned long val;
 
+	if (exc->m.modnum < 0)
+	{
+		return;
+	}
 	if (!action || sscanf(action,"%lu",&val) != 1)
 	{
 		val = 0;
 	}
-	set_message_mask(&NoGrabMask[*Module], (unsigned long)val);
+	set_message_mask(&NoGrabMask[exc->m.modnum], (unsigned long)val);
 
 	return;
 }
