@@ -29,7 +29,11 @@ void AnimatedMoveFvwmWindow(
 Bool __move_loop(
 	const exec_context_t *exc, int XOffset, int YOffset, int Width,
 	int Height, int *FinalX, int *FinalY,Bool do_move_opaque);
-void handle_stick(F_CMD_ARGS, int toggle, int do_not_draw, int do_silently);
+int is_window_sticky_on_page(FvwmWindow *fw);
+int is_window_sticky_on_desk(FvwmWindow *fw);
+void handle_stick(
+	F_CMD_ARGS, int toggle_page, int toggle_desk, int do_not_draw,
+	int do_silently);
 void resize_geometry_window(void);
 void __move_icon(
 	FvwmWindow *fw, int x, int y, int old_x, int old_y,
