@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /****************************************************************************
@@ -30,11 +30,10 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
+#include "execcontext.h"
 #include "cursor.h"
-#include "functions.h"
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
@@ -42,12 +41,11 @@
 #include "geometry.h"
 #include "update.h"
 #include "style.h"
-#include "borders.h"
 #include "move_resize.h"
 #include "virtual.h"
 #include "stack.h"
-#include "add_window.h"
 #include "ewmh.h"
+#include "icons.h"
 
 #ifndef MIN
 #define MIN(A,B) ((A)<(B)? (A):(B))
@@ -72,9 +70,9 @@ static int SmartPlacement(
   FvwmWindow *t, rectangle *screen_g,
   int width, int height, int *x, int *y, int pdeltax, int pdeltay)
 {
-  int PageLeft	 = screen_g->x - pdeltax;
-  int PageTop	 = screen_g->y - pdeltay;
-  int PageRight	 = PageLeft + screen_g->width;
+  int PageLeft   = screen_g->x - pdeltax;
+  int PageTop    = screen_g->y - pdeltay;
+  int PageRight  = PageLeft + screen_g->width;
   int PageBottom = PageTop + screen_g->height;
   int temp_h;
   int temp_w;
@@ -108,10 +106,10 @@ static int SmartPlacement(
 	if (t == test_fw || IS_EWMH_DESKTOP(FW_W(test_fw)))
 	  continue;
 
-	/*  RBW - account for sticky windows...	 */
-	if (test_fw->Desk == t->Desk || IS_STICKY(test_fw))
+	/*  RBW - account for sticky windows...  */
+	if (test_fw->Desk == t->Desk || IS_STICKY_ON_DESK(test_fw))
 	{
-	  if (IS_STICKY(test_fw))
+	  if (IS_STICKY_ON_PAGE(test_fw))
 	  {
 	    stickyx = pdeltax;
 	    stickyy = pdeltay;
@@ -158,8 +156,8 @@ static int SmartPlacement(
 
 /* CleverPlacement by Anthony Martin <amartin@engr.csulb.edu>
  * This function will place a new window such that there is a minimum amount
- * of interference with other windows.	If it can place a window without any
- * interference, fine.	Otherwise, it places it so that the area of of
+ * of interference with other windows.  If it can place a window without any
+ * interference, fine.  Otherwise, it places it so that the area of of
  * interference between the new window and the other windows is minimized */
 static void CleverPlacement(
   FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
@@ -253,11 +251,11 @@ static int get_next_x(
   /* Test the values of the right edges of every window */
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)) ||
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY_ON_DESK(testw)) ||
 	IS_EWMH_DESKTOP(FW_W(testw)))
       continue;
 
-    if (IS_STICKY(testw))
+    if (IS_STICKY_ON_PAGE(testw))
     {
       stickyx = pdeltax;
       stickyy = pdeltay;
@@ -349,11 +347,11 @@ static int get_next_y(
   /* Test the values of the bottom edge of every window */
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY_ON_DESK(testw))
 	|| IS_EWMH_DESKTOP(FW_W(testw)))
       continue;
 
-    if (IS_STICKY(testw))
+    if (IS_STICKY_ON_PAGE(testw))
     {
       stickyy = pdeltay;
     }
@@ -424,7 +422,7 @@ static float test_fit(
   float anew;
   float cover_factor = 0;
   float avoidance_factor;
-  int PageRight	 = screen_g->x + screen_g->width - pdeltax;
+  int PageRight  = screen_g->x + screen_g->width - pdeltax;
   int PageBottom = screen_g->y + screen_g->height - pdeltay;
   int stickyx, stickyy;
   rectangle g;
@@ -439,11 +437,11 @@ static float test_fit(
     return -2;
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY_ON_DESK(testw))
 	|| IS_EWMH_DESKTOP(FW_W(testw)))
        continue;
 
-    if (IS_STICKY(testw))
+    if (IS_STICKY_ON_PAGE(testw))
     {
       stickyx = pdeltax;
       stickyy = pdeltay;
@@ -474,7 +472,7 @@ static float test_fit(
 	avoidance_factor = ONTOP_PLACEMENT_PENALTY(testw);
       else if(compare_window_layers(testw, t) < 0)
 	avoidance_factor = BELOW_PLACEMENT_PENALTY(testw);
-      else if(IS_STICKY(testw))
+      else if(IS_STICKY_ON_PAGE(testw) || IS_STICKY_ON_DESK(testw))
 	avoidance_factor = STICKY_PLACEMENT_PENALTY(testw);
       else
 	avoidance_factor = NORMAL_PLACEMENT_PENALTY(testw);
@@ -538,7 +536,7 @@ static float test_fit(
  *
  **************************************************************************/
 Bool PlaceWindow(
-	FvwmWindow *fw, style_flags *sflags, rectangle *attr_g,
+	const exec_context_t *exc, style_flags *sflags, rectangle *attr_g,
 	int Desk, int PageX, int PageY, int XineramaScreen, int mode,
 	initial_window_options_type *win_opts)
 {
@@ -567,6 +565,7 @@ Bool PlaceWindow(
     unsigned do_not_use_wm_placement : 1;
   } flags;
   extern Bool Restarting;
+  FvwmWindow *fw = exc->w.fw;
 
   memset(&flags, 0, sizeof(flags));
 
@@ -576,7 +575,7 @@ Bool PlaceWindow(
    * 3. Put it on the desk it was on before the restart.
    * 4. Transients go on the same desk as their parents.
    * 5. Window groups stay together (if the KeepWindowGroupsOnDesk style is
-   *	used).
+   *    used).
    */
 
   /*
@@ -671,7 +670,7 @@ Bool PlaceWindow(
   {
     fw->Desk = Scr.CurrentDesk;
   }
-  if (SIS_STICKY(*sflags))
+  if (S_IS_STICKY_ON_DESK(SFC(*sflags)))
     fw->Desk = Scr.CurrentDesk;
   else if (SUSE_START_ON_DESK(sflags) && Desk && flags.do_honor_starts_on_page)
     fw->Desk = (Desk > -1) ? Desk - 1 : Desk;
@@ -738,7 +737,7 @@ Bool PlaceWindow(
 
   /* I think it would be good to switch to the selected desk
    * whenever a new window pops up, except during initialization */
-  /*  RBW - 11/02/1998	--  I dont. */
+  /*  RBW - 11/02/1998  --  I dont. */
   if ((!win_opts->flags.do_override_ppos)&&(!DO_NOT_SHOW_ON_MAP(fw)))
   {
     goto_desk(fw->Desk);
@@ -748,7 +747,8 @@ Bool PlaceWindow(
    * adjust the coordinates later. Otherwise, just switch to the target
    * page - it's ever so much simpler.
    */
-  if (!SIS_STICKY(*sflags) && SUSE_START_ON_DESK(sflags))
+  if (!S_IS_STICKY_ON_DESK(SFC(*sflags)) &&
+      !S_IS_STICKY_ON_PAGE(SFC(*sflags)) && SUSE_START_ON_DESK(sflags))
   {
     if (PageX && PageY)
     {
@@ -763,11 +763,11 @@ Bool PlaceWindow(
       else if (flags.do_honor_starts_on_page)
       {
 	/*  Save the delta from current page */
-	pdeltax	    = Scr.Vx - px;
-	pdeltay	    = Scr.Vy - py;
+	pdeltax     = Scr.Vx - px;
+	pdeltay     = Scr.Vy - py;
 	PageLeft   -= pdeltax;
 	PageRight  -= pdeltax;
-	PageTop	   -= pdeltay;
+	PageTop    -= pdeltay;
 	PageBottom -= pdeltay;
       }
     }
@@ -904,8 +904,8 @@ Bool PlaceWindow(
 	  XMapRaised(dpy, Scr.SizeWindow);
 	}
 	FScreenGetScrRect(NULL, FSCREEN_GLOBAL, &mx, &my, NULL, NULL);
-	if (moveLoop(fw, mx, my, DragWidth, DragHeight,
-		     &xl, &yt, False))
+	if (__move_loop(
+		    exc, mx, my, DragWidth, DragHeight, &xl, &yt, False))
 	{
 	  /* resize too */
 	  rc = True;
@@ -996,13 +996,13 @@ Bool PlaceWindow(
       }
 
       /* the left and top sides are more important in huge windows */
-      if (attr_g->x < 0)
+      if (attr_g->x < PageLeft)
       {
-	attr_g->x = 0;
+	attr_g->x = PageLeft;
       }
-      if (attr_g->y < 0)
+      if (attr_g->y < PageTop)
       {
-	attr_g->y = 0;
+	attr_g->y = PageTop;
       }
       break;
     case PLACE_MINOVERLAP:
@@ -1050,7 +1050,7 @@ Bool PlaceWindow(
     }
 
     /*
-     *	If SkipMapping, and other legalities are observed, adjust for
+     *  If SkipMapping, and other legalities are observed, adjust for
      * StartsOnPage.
      */
     if ( ( DO_NOT_SHOW_ON_MAP(fw) && flags.do_honor_starts_on_page )  &&
@@ -1061,7 +1061,7 @@ Bool PlaceWindow(
 	   ((SUSE_NO_PPOSITION(sflags)) ||
 	    !(fw->hints.flags & PPosition)) &&
 
-	   /*  RBW - allow StartsOnPage to go through, even if iconic.	*/
+	   /*  RBW - allow StartsOnPage to go through, even if iconic.  */
 	   ( ((!((fw->wmhints)&&
 		 (fw->wmhints->flags & StateHint)&&
 		 (fw->wmhints->initial_state == IconicState)))
@@ -1141,7 +1141,6 @@ Bool PlaceWindow(
   return rc;
 }
 
-
 void CMD_PlaceAgain(F_CMD_ARGS)
 {
 	char *token;
@@ -1150,38 +1149,61 @@ void CMD_PlaceAgain(F_CMD_ARGS)
 	window_style style;
 	rectangle attr_g;
 	XWindowAttributes attr;
-	initial_window_options_type win_opts;
+	Bool do_move_animated = False;
+	Bool do_place_icon = False;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
-	{
-		return;
-	}
 	if (!XGetWindowAttributes(dpy, FW_W(fw), &attr))
 	{
 		return;
 	}
-	memset(&win_opts, 0, sizeof(win_opts));
-	lookup_style(fw, &style);
-	attr_g.x = attr.x;
-	attr_g.y = attr.y;
-	attr_g.width = attr.width;
-	attr_g.height = attr.height;
-	PlaceWindow(
-		fw, &style.flags, &attr_g, SGET_START_DESK(style),
-		SGET_START_PAGE_X(style), SGET_START_PAGE_Y(style),
-		SGET_START_SCREEN(style), PLACE_AGAIN, &win_opts);
-
-	/* Possibly animate the movement */
-	token = PeekToken(action, NULL);
-	if (token && StrEquals("Anim", token))
+	while ((token = PeekToken(action, &action)) != NULL)
 	{
-		ppctMovement = NULL;
+		if (StrEquals("Anim", token))
+		{
+			ppctMovement = NULL;
+			do_move_animated = True;
+		}
+		else if (StrEquals("icon", token))
+		{
+			do_place_icon = True;
+		}
 	}
+	if (IS_ICONIFIED(fw) && do_place_icon)
+	{
+		rectangle new_g;
+		rectangle old_g;
 
-	AnimatedMoveFvwmWindow(
-		fw, FW_W_FRAME(fw), -1, -1, attr_g.x, attr_g.y, False, -1,
-		ppctMovement);
+		if (IS_ICON_SUPPRESSED(fw))
+		{
+			return;
+		}
+		get_icon_geometry(fw, &old_g);
+		SET_ICON_MOVED(fw, 0);
+		AutoPlaceIcon(fw, NULL, False);
+		get_icon_geometry(fw, &new_g);
+		__move_icon(
+			fw, new_g.x, new_g.y, old_g.x, old_g.y,
+			do_move_animated, False);
+	}
+	else
+	{
+		initial_window_options_type win_opts;
+
+		memset(&win_opts, 0, sizeof(win_opts));
+		lookup_style(fw, &style);
+		attr_g.x = attr.x;
+		attr_g.y = attr.y;
+		attr_g.width = attr.width;
+		attr_g.height = attr.height;
+		PlaceWindow(
+			exc, &style.flags, &attr_g, SGET_START_DESK(style),
+			SGET_START_PAGE_X(style), SGET_START_PAGE_Y(style),
+			SGET_START_SCREEN(style), PLACE_AGAIN, &win_opts);
+		AnimatedMoveFvwmWindow(
+			fw, FW_W_FRAME(fw), -1, -1, attr_g.x, attr_g.y, False,
+			-1, ppctMovement);
+	}
 
 	return;
 }
