@@ -15,9 +15,7 @@ extern const char *get_current_read_dir(void);
  * Parameters eventp, fw, context, and Module supply the context
  * for executing the commands.
  **/
-extern void run_command_stream( FILE* f,
-				XEvent *eventp, FvwmWindow *fw,
-				unsigned long context, int Module );
+void run_command_stream(FILE *f, const exec_context_t *exc);
 
 
 /**
@@ -27,8 +25,6 @@ extern void run_command_stream( FILE* f,
  * fvwm_userdir (set in main()) or in FVWM_DATADIR.  Return TRUE
  * if the file was found and executed.
  **/
-extern int run_command_file( char* filename,
-			     XEvent *eventp, FvwmWindow *fw,
-			     unsigned long context, int Module );
+int run_command_file(char *filename, const exec_context_t *exc);
 
 #endif
