@@ -82,7 +82,8 @@ typedef enum
     MultiPixmap               ,
 #endif
     MiniIconButton            ,
-    SolidButton
+    SolidButton               ,
+    ColorsetButton
 } DecorFaceType;
 
 typedef enum
@@ -136,6 +137,11 @@ typedef struct DecorFace
 		FvwmPicture **multi_pixmaps;
 		short multi_stretch_flags;
 #endif
+		struct
+		{
+			int cs;
+			unsigned short alpha_percent;
+		} acs;
 		Pixel back;
 		struct
 		{
@@ -151,6 +157,8 @@ typedef struct DecorFace
 			int num;
 			signed char *x;
 			signed char *y;
+		        signed char *xoff;
+		        signed char *yoff;
 			signed char *c;
 			unsigned use_fgbg : 1;
 		} vector;
@@ -249,24 +257,25 @@ typedef struct
 typedef struct FvwmDecor
 {
 #ifdef USEDECOR
-  char *tag;                    /* general style tag */
+	char *tag;                    /* general style tag */
 #endif
-  int title_height;           /* explicitly specified title bar height */
-  /* titlebar buttons */
-  TitleButton buttons[NUMBER_OF_BUTTONS];
-  TitleButton titlebar;
-  struct BorderStyle
-  {
-    DecorFace active, inactive;
-  } BorderStyle;
+	int title_height;           /* explicitly specified title bar height */
+	int min_title_height;
+	/* titlebar buttons */
+	TitleButton buttons[NUMBER_OF_BUTTONS];
+	TitleButton titlebar;
+	struct BorderStyle
+	{
+		DecorFace active, inactive;
+	} BorderStyle;
 #ifdef USEDECOR
-  struct FvwmDecor *next;       /* additional user-defined styles */
+	struct FvwmDecor *next;       /* additional user-defined styles */
 #endif
-  struct
-  {
-    unsigned has_changed : 1;
-    unsigned has_title_height_changed : 1;
-  } flags;
+	struct
+	{
+		unsigned has_changed : 1;
+		unsigned has_title_height_changed : 1;
+	} flags;
 } FvwmDecor;
 
 typedef struct DesktopsInfo
@@ -350,7 +359,7 @@ typedef struct ScreenInfo
   GC ScratchGC3;
   GC ScratchGC4;
   GC TitleGC;
-  GC TileGC;                    /* Used for tiling (clipmask and tile) */
+  GC BordersGC;
   int SizeStringWidth;          /* minimum width of size window */
 
   FvwmDecor DefaultDecor;       /* decoration style(s) */
