@@ -45,6 +45,7 @@
 #include "icccm2.h"
 #include "virtual.h"
 #include "geometry.h"
+#include "move_resize.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -375,7 +376,7 @@ SaveWindowStates(FILE *f)
 	     ewin != &Scr.FvwmRoot;
 	     ewin = get_next_window_in_stack_ring(ewin))
 	{
-		Bool is_icon_sticky;
+		Bool is_icon_sticky_on_page;
 
 		if (!XGetGeometry(
 			    dpy, FW_W(ewin), &JunkRoot, &JunkX, &JunkY,
@@ -385,8 +386,7 @@ SaveWindowStates(FILE *f)
 			 * (i.e. modules)! */
 			continue;
 		}
-		is_icon_sticky = (IS_STICKY(ewin) ||
-				  (IS_ICONIFIED(ewin) && IS_ICON_STICKY(ewin)));
+		is_icon_sticky_on_page = is_window_sticky_on_page(ewin);
 		fprintf(f, "[CLIENT] %lx\n", FW_W(ewin));
 
 		client_id = GetClientID(FW_W(ewin));
@@ -446,7 +446,7 @@ SaveWindowStates(FILE *f)
 		gravity_get_naked_geometry(
 			ewin->hints.win_gravity, ewin, &save_g,
 			&ewin->normal_g);
-		if (IS_STICKY(ewin))
+		if (IS_STICKY_ON_PAGE(ewin))
 		{
 			save_g.x -= Scr.Vx;
 			save_g.y -= Scr.Vy;
@@ -459,8 +459,8 @@ SaveWindowStates(FILE *f)
 			ewin->max_g.x, ewin->max_g.y, ewin->max_g.width,
 			ewin->max_g.height, ewin->max_g_defect.width,
 			ewin->max_g_defect.height,
-			ig.x + ((!is_icon_sticky) ? Scr.Vx : 0),
-			ig.y + ((!is_icon_sticky) ? Scr.Vy : 0),
+			ig.x + ((!is_icon_sticky_on_page) ? Scr.Vx : 0),
+			ig.y + ((!is_icon_sticky_on_page) ? Scr.Vy : 0),
 			ewin->hints.win_gravity,
 			ewin->max_offset.x, ewin->max_offset.y);
 		fprintf(f, "  [DESK] %i\n", ewin->Desk);
@@ -1053,8 +1053,10 @@ LoadWindowStates(char *filename)
 {
 	FILE *f;
 	char s[4096], s1[4096];
+	char *s2;
 	int i, pos, pos1;
 	unsigned long w;
+	int n;
 
 	if (!VerifyVersionInfo(filename))
 	{
@@ -1081,7 +1083,8 @@ LoadWindowStates(char *filename)
 
 	while (fgets(s, sizeof(s), f))
 	{
-		sscanf(s, "%4000s", s1);
+		n = 0;
+		sscanf(s, "%4000s%n", s1, &n);
 		/* migo: temporarily */
 		if (
 #ifndef SESSION
@@ -1182,27 +1185,52 @@ LoadWindowStates(char *filename)
 		}
 		else if (!strcmp(s1, "[CLIENT_ID]"))
 		{
-			sscanf(s, "%*s %[^\n]", s1);
+			s2 = s + n;
+			if (*s2 != 0)
+			{
+				s2++;
+			}
+			sscanf(s2, "%[^\n]", s1);
 			matches[num_match - 1].client_id = duplicate(s1);
 		}
 		else if (!strcmp(s1, "[WINDOW_ROLE]"))
 		{
-			sscanf(s, "%*s %[^\n]", s1);
+			s2 = s + n;
+			if (*s2 != 0)
+			{
+				s2++;
+			}
+			sscanf(s2, "%[^\n]", s1);
 			matches[num_match - 1].window_role = duplicate(s1);
 		}
 		else if (!strcmp(s1, "[RES_NAME]"))
 		{
-			sscanf(s, "%*s %[^\n]", s1);
+			s2 = s + n;
+			if (*s2 != 0)
+			{
+				s2++;
+			}
+			sscanf(s2, "%[^\n]", s1);
 			matches[num_match - 1].res_name = duplicate(s1);
 		}
 		else if (!strcmp(s1, "[RES_CLASS]"))
 		{
-			sscanf(s, "%*s %[^\n]", s1);
+			s2 = s + n;
+			if (*s2 != 0)
+			{
+				s2++;
+			}
+			sscanf(s2, "%[^\n]", s1);
 			matches[num_match - 1].res_class = duplicate(s1);
 		}
 		else if (!strcmp(s1, "[WM_NAME]"))
 		{
-			sscanf(s, "%*s %[^\n]", s1);
+			s2 = s + n;
+			if (*s2 != 0)
+			{
+				s2++;
+			}
+			sscanf(s2, "%[^\n]", s1);
 			matches[num_match - 1].wm_name = duplicate(s1);
 		}
 		else if (!strcmp(s1, "[WM_COMMAND]"))
@@ -1270,8 +1298,12 @@ MatchWinToSM(
 					FW_FOCUS_POLICY(ewin),
 					FP_IS_LENIENT(FW_FOCUS_POLICY(
 							      &(matches[i]))));
-				SET_ICON_STICKY(
-					ewin, IS_ICON_STICKY(&(matches[i])));
+				SET_ICON_STICKY_ON_PAGE(
+					ewin,
+					IS_ICON_STICKY_ON_PAGE(&(matches[i])));
+				SET_ICON_STICKY_ON_DESK(
+					ewin,
+					IS_ICON_STICKY_ON_DESK(&(matches[i])));
 				SET_DO_SKIP_ICON_CIRCULATE(
 					ewin, DO_SKIP_ICON_CIRCULATE(
 						&(matches[i])));
@@ -1311,9 +1343,9 @@ MatchWinToSM(
 				win_opts->flags.use_initial_icon_xy = 1;
 				win_opts->initial_icon_x = matches[i].icon_x;
 				win_opts->initial_icon_y = matches[i].icon_y;
-				if (!IS_STICKY(&(matches[i])) &&
+				if (!IS_STICKY_ON_PAGE(&(matches[i])) &&
 				    !(IS_ICONIFIED(&(matches[i])) &&
-				      IS_ICON_STICKY(&(matches[i]))))
+				      IS_ICON_STICKY_ON_PAGE(&(matches[i]))))
 				{
 					win_opts->initial_icon_x -= Scr.Vx;
 					win_opts->initial_icon_y -= Scr.Vy;
@@ -1332,8 +1364,11 @@ MatchWinToSM(
 				matches[i].height_defect_max;
 			ewin->max_offset.x = matches[i].max_x_offset;
 			ewin->max_offset.y = matches[i].max_y_offset;
-			SET_STICKY(ewin, IS_STICKY(&(matches[i])));
-			ewin->Desk = (IS_STICKY(ewin)) ?
+			SET_STICKY_ON_PAGE(
+				ewin, IS_STICKY_ON_PAGE(&(matches[i])));
+			SET_STICKY_ON_DESK(
+				ewin, IS_STICKY_ON_DESK(&(matches[i])));
+			ewin->Desk = (IS_STICKY_ON_DESK(ewin)) ?
 				Scr.CurrentDesk : matches[i].desktop;
 			set_layer(ewin, matches[i].layer);
 			/* Note: the Modal, skip pager, skip taskbar and
