@@ -22,16 +22,15 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
-#include <libs/gravity.h>
+#include "libs/FGettext.h"
 #include "fvwm.h"
 #include "externs.h"
+#include "execcontext.h"
 #include "cursor.h"
-#include "functions.h"
-#include "bindings.h"
 #include "events.h"
+#include "eventmask.h"
 #include "misc.h"
 #include "screen.h"
-#include "defaults.h"
 #include "virtual.h"
 #include "module_interface.h"
 #include "focus.h"
@@ -335,8 +334,7 @@ static void UnmapDesk(int desk, Bool grab)
 	     t != &Scr.FvwmRoot; t = get_prev_window_in_stack_ring(t))
 	{
 		/* Only change mapping for non-sticky windows */
-		if (!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
-		   !(IS_STICKY(t)) && !IS_ICON_UNMAPPED(t))
+		if (!is_window_sticky_on_desk(t) && !IS_ICON_UNMAPPED(t))
 		{
 			if (t->Desk == desk)
 			{
@@ -390,8 +388,7 @@ static void MapDesk(int desk, Bool grab)
 	     t != &Scr.FvwmRoot; t = get_next_window_in_stack_ring(t))
 	{
 		/* Only change mapping for non-sticky windows */
-		if (!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
-		   !(IS_STICKY(t)) && !IS_ICON_UNMAPPED(t))
+		if (!is_window_sticky_on_desk(t) && !IS_ICON_UNMAPPED(t))
 		{
 			if (t->Desk == desk)
 			{
@@ -420,9 +417,8 @@ static void MapDesk(int desk, Bool grab)
 		  Autoplace any sticky icons, so that sticky icons from the old
 		  desk don't land on top of stationary ones on the new desk.
 		*/
-		if ((IS_STICKY(t) || IS_ICON_STICKY(t)) &&
-		   IS_ICONIFIED(t) && !IS_ICON_MOVED(t) &&
-		   !IS_ICON_UNMAPPED(t))
+		if (is_window_sticky_on_desk(t) && IS_ICONIFIED(t) &&
+		    !IS_ICON_MOVED(t) && !IS_ICON_UNMAPPED(t))
 		{
 			AutoPlaceIcon(t, NULL, True);
 		}
@@ -466,7 +462,7 @@ static void MapDesk(int desk, Bool grab)
  * -1: no need to call the function again before a new event arrives
  ***************************************************************************/
 int HandlePaging(
-	int HorWarpSize, int VertWarpSize, int *xl, int *yt,
+	XEvent *pev, int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 	int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
 	Bool do_continue_previous)
 {
@@ -487,10 +483,10 @@ int HandlePaging(
 		/* don't call me again until something has happened */
 		return -1;
 	}
-	if (!is_timestamp_valid && Event.type == MotionNotify)
+	if (!is_timestamp_valid && pev->type == MotionNotify)
 	{
-		x = Event.xmotion.x_root;
-		y = Event.xmotion.y_root;
+		x = pev->xmotion.x_root;
+		y = pev->xmotion.y_root;
 		/* need to move the viewport */
 		if ((Scr.VxMax == 0 ||
 		     (x >= edge_thickness &&
@@ -518,49 +514,45 @@ int HandlePaging(
 	if (!is_timestamp_valid)
 	{
 		is_timestamp_valid = True;
-		my_timestamp = lastTimestamp;
+		my_timestamp = fev_get_evtime();
 		is_last_position_valid = False;
 		add_time = 0;
 		last_x = -1;
 		last_y = -1;
 	}
-	else if (my_last_timestamp != lastTimestamp)
+	else if (my_last_timestamp != fev_get_evtime())
 	{
 		add_time = 0;
 	}
-	my_last_timestamp = lastTimestamp;
+	my_last_timestamp = fev_get_evtime();
 
 	do
 	{
-		if (XPending(dpy) > 0 &&
-		    (XCheckWindowEvent(
-			    dpy, Scr.PanFrameTop.win, LeaveWindowMask,
-			    &Event) ||
-		     XCheckWindowEvent(
+		if (FPending(dpy) > 0 &&
+		    (FCheckWindowEvent(
+			    dpy, Scr.PanFrameTop.win, LeaveWindowMask, pev) ||
+		     FCheckWindowEvent(
 			     dpy, Scr.PanFrameBottom.win, LeaveWindowMask,
-			     &Event) ||
-		     XCheckWindowEvent(
-			     dpy, Scr.PanFrameLeft.win, LeaveWindowMask,
-			     &Event) ||
-		     XCheckWindowEvent(
-			     dpy, Scr.PanFrameRight.win, LeaveWindowMask,
-			     &Event)))
+			     pev) ||
+		     FCheckWindowEvent(
+			     dpy, Scr.PanFrameLeft.win, LeaveWindowMask, pev) ||
+		     FCheckWindowEvent(
+			     dpy, Scr.PanFrameRight.win, LeaveWindowMask, pev)))
 		{
-			StashEventTime(&Event);
 			is_timestamp_valid = False;
 			add_time = 0;
 			return 0;
 		}
-		else if (XCheckMaskEvent(
+		else if (FCheckMaskEvent(
 				 dpy, ButtonPressMask | ButtonReleaseMask, &e))
 		{
-			XPutBackEvent(dpy, &e);
+			FPutBackEvent(dpy, &e);
 			is_timestamp_valid = False;
 			add_time = 0;
 			return 0;
 		}
 		/* get pointer location */
-		GetLocationFromEventOrQuery(dpy, Scr.Root, &Event, &x, &y);
+		fev_get_evpos_or_query(dpy, Scr.Root, pev, &x, &y);
 		/* check actual pointer location since PanFrames can get buried
 		 * under window being moved or resized - mab */
 		if (x >= edge_thickness &&
@@ -591,7 +583,7 @@ int HandlePaging(
 			 * set since we can't be sure that HandlePaging will be
 			 * called again. */
 			is_timestamp_valid = True;
-			my_timestamp = lastTimestamp;
+			my_timestamp = fev_get_evtime();
 			add_time = 0;
 			last_x = x;
 			last_y = y;
@@ -603,10 +595,10 @@ int HandlePaging(
 		usleep(10000);
 		add_time += 10;
 	} while (fLoop &&
-		 lastTimestamp - my_timestamp + add_time <
+		 fev_get_evtime() - my_timestamp + add_time <
 		 Scr.ScrollResistance);
 
-	if (lastTimestamp - my_timestamp + add_time < Scr.ScrollResistance)
+	if (fev_get_evtime() - my_timestamp + add_time < Scr.ScrollResistance)
 	{
 		return 0;
 	}
@@ -742,9 +734,9 @@ int HandlePaging(
 	}
 	/* Turn off the rubberband if its on */
 	switch_move_resize_grid(False);
-	XWarpPointer(dpy,None,Scr.Root,0,0,0,0,*xl,*yt);
+	FWarpPointer(dpy,None,Scr.Root,0,0,0,0,*xl,*yt);
 	MoveViewport(Scr.Vx + *delta_x,Scr.Vy + *delta_y,False);
-	if (XQueryPointer(
+	if (FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &JunkChild, xl, yt, &JunkX,
 		    &JunkY, &JunkMask) == False)
 	{
@@ -1055,6 +1047,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 	int PageTop, PageLeft;
 	int PageBottom, PageRight;
 	int txl, txr, tyt, tyb;
+	XEvent e;
 
 	if (grab)
 	{
@@ -1126,8 +1119,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 			tyt = t->frame_g.y;
 			txr = t->frame_g.x + t->frame_g.width - 1;
 			tyb = t->frame_g.y + t->frame_g.height - 1;
-			if ((IS_STICKY(t) ||
-			     (IS_ICONIFIED(t) && IS_ICON_STICKY(t))) &&
+			if (is_window_sticky_on_page(t) &&
 			    !IS_VIEWPORT_MOVED(t))
 			{
 				/* the absolute position has changed */
@@ -1148,8 +1140,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 				/* If the window is iconified, and sticky
 				 * Icons is set, then the window should
 				 * essentially be sticky */
-				if (!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
-				    !IS_STICKY(t))
+				if (!is_window_sticky_on_page(t))
 				{
 					if (IS_ICONIFIED(t))
 					{
@@ -1188,8 +1179,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 				/* If the window is iconified, and sticky
 				 * Icons is set, then the window should
 				 * essentially be sticky */
-				if (!(IS_ICONIFIED(t1) &&
-				      IS_ICON_STICKY(t1)) && !IS_STICKY(t1))
+				if (!is_window_sticky_on_page(t1))
 				{
 					if (IS_ICONIFIED(t1))
 					{
@@ -1220,9 +1210,8 @@ void MoveViewport(int newx, int newy, Bool grab)
 			/* If its an icon, and its sticking, autoplace it so
 			 * that it doesn't wind up on top a a stationary
 			 * icon */
-			if ((IS_STICKY(t) || IS_ICON_STICKY(t)) &&
-			   IS_ICONIFIED(t) && !IS_ICON_MOVED(t) &&
-			   !IS_ICON_UNMAPPED(t))
+			if (is_window_sticky_on_page(t) && IS_ICONIFIED(t) &&
+			    !IS_ICON_MOVED(t) && !IS_ICON_UNMAPPED(t))
 			{
 				AutoPlaceIcon(t, NULL, True);
 			}
@@ -1233,9 +1222,9 @@ void MoveViewport(int newx, int newy, Bool grab)
 	focus_grab_buttons_all();
 
 	/* do this with PanFrames too ??? HEDU */
-	while (XCheckTypedEvent(dpy, MotionNotify, &Event))
+	while (FCheckTypedEvent(dpy, MotionNotify, &e))
 	{
-		StashEventTime(&Event);
+		/* nothing */
 	}
 	if (grab)
 	{
@@ -1297,8 +1286,7 @@ void do_move_window_to_desk(FvwmWindow *fw, int desk)
 	 * Set the window's desktop, and map or unmap it as needed.
 	 */
 	/* Only change mapping for non-sticky windows */
-	if (!(IS_ICONIFIED(fw) && IS_ICON_STICKY(fw)) &&
-	    !IS_STICKY(fw) /*&& !IS_ICON_UNMAPPED(fw)*/)
+	if (!is_window_sticky_on_desk(fw) /*&& !IS_ICON_UNMAPPED(fw)*/)
 	{
 		if (fw->Desk == Scr.CurrentDesk)
 		{
@@ -1589,7 +1577,7 @@ void CMD_EdgeThickness(F_CMD_ARGS)
 
 void CMD_EdgeScroll(F_CMD_ARGS)
 {
-	int val1, val2, val1_unit,val2_unit,n;
+	int val1, val2, val1_unit, val2_unit, n;
 
 	n = GetTwoArguments(action, &val1, &val2, &val1_unit, &val2_unit);
 	if (n != 2)
@@ -1601,7 +1589,7 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 	}
 
 	/*
-	 * if edgescroll >1000 and < 100000m
+	 * if edgescroll >1000 and <100000
 	 * wrap at edges of desktop (a "spherical" desktop)
 	 */
 	if (val1 >= 1000)
@@ -1623,8 +1611,8 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 		Scr.flags.edge_wrap_y = 0;
 	}
 
-	Scr.EdgeScrollX = val1*val1_unit/100;
-	Scr.EdgeScrollY = val2*val2_unit/100;
+	Scr.EdgeScrollX = val1 * val1_unit / 100;
+	Scr.EdgeScrollY = val2 * val2_unit / 100;
 
 	checkPanFrames();
 
@@ -1881,12 +1869,8 @@ void CMD_GotoPage(F_CMD_ARGS)
 void CMD_MoveToDesk(F_CMD_ARGS)
 {
 	int desk;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
-	{
-		return;
-	}
 	desk = GetDeskNumber(action);
 	if (desk == fw->Desk)
 	{
@@ -2055,6 +2039,8 @@ void CMD_DesktopName(F_CMD_ARGS)
 	if (!fFvwmInStartup)
 	{
 		char *msg;
+		const char *default_desk_name = _("Desk");
+
 		/* should send the info to the FvwmPager and set the EWMH
 		 * desktop names */
 		if (action != NULL && *action && *action != '\n')
@@ -2064,8 +2050,10 @@ void CMD_DesktopName(F_CMD_ARGS)
 		}
 		else
 		{
-			msg = (char *)safemalloc(84);
-			sprintf(msg, "DesktopName %d Desk %d", desk, desk);
+			msg = (char *)safemalloc(strlen(default_desk_name)+44);
+			sprintf(
+				msg, "DesktopName %d %s %d", desk,
+				default_desk_name, desk);
 		}
 		BroadcastConfigInfoString(msg);
 		free(msg);
