@@ -5,21 +5,25 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #ifndef _VIRTUAL_
 #define _VIRTUAL_
 
-int HandlePaging(int, int, int *, int *, int *, int *, Bool, Bool, Bool);
+int HandlePaging(
+	XEvent *pev, int HorWarpSize, int VertWarpSize, int *xl, int *yt,
+	int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
+	Bool do_continue_previous);
 void checkPanFrames(void);
 void raisePanFrames(void);
 void initPanFrames(void);
+Bool is_pan_frame(Window w);
 void MoveViewport(int newx, int newy,Bool);
 void goto_desk(int desk);
 void do_move_window_to_desk(FvwmWindow *fw, int desk);
