@@ -33,6 +33,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
+#include "libs/FGettext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "functions.h"
@@ -49,7 +50,8 @@
 #define SHOW_ALLDESKS		(1<<1)
 #define SHOW_NORMAL		(1<<2)
 #define SHOW_ICONIC		(1<<3)
-#define SHOW_STICKY		(1<<4)
+#define SHOW_STICKY_ON_PAGE	(1<<4)
+#define SHOW_STICKY_ON_DESK	(1<<5)
 #define NO_DESK_SORT		(1<<6)
 #define SHOW_ICONNAME		(1<<7)
 #define SHOW_ALPHABETIC		(1<<8)
@@ -66,7 +68,7 @@
 #define NO_LAYER		(1<<19)
 #define SHOW_SCREEN		(1<<20)
 #define SHOW_DEFAULT (SHOW_GEOMETRY | SHOW_ALLDESKS | SHOW_NORMAL | \
-	SHOW_ICONIC | SHOW_STICKY)
+	SHOW_ICONIC | SHOW_STICKY_ON_PAGE | SHOW_STICKY_ON_DESK)
 
 static char *get_desk_title(int desk, unsigned long flags, Bool is_top_title)
 {
@@ -89,20 +91,20 @@ static char *get_desk_title(int desk, unsigned long flags, Bool is_top_title)
 		{
 			sprintf(tlabel, "%s%s", desk_name,
 				(is_top_title && (flags & SHOW_GEOMETRY)) ?
-				"\tGeometry" : "");
+				_("\tGeometry") : "");
 		}
 		else
 		{
 			sprintf(tlabel,"%d: %s%s", desk, desk_name,
 				(is_top_title && (flags & SHOW_GEOMETRY)) ?
-				"\tGeometry" : "");
+				_("\tGeometry") : "");
 		}
 	}
 	else
 	{
-		sprintf(tlabel,"Desk: %d%s",desk,
+		sprintf(tlabel,_("Desk: %d%s"),desk,
 			(is_top_title && (flags & SHOW_GEOMETRY)) ?
-			"\tGeometry" : "");
+			_("\tGeometry") : "");
 	}
 
 	return tlabel;
@@ -392,15 +394,42 @@ void CMD_WindowList(F_CMD_ARGS)
 			}
 			else if (StrEquals(tok,"NoSticky"))
 			{
-				flags &= ~SHOW_STICKY;
+				flags &= ~(SHOW_STICKY_ON_PAGE);
+				flags &= ~(SHOW_STICKY_ON_DESK);
+			}
+			else if (StrEquals(tok,"NoStickyPage"))
+			{
+				flags &= ~(SHOW_STICKY_ON_PAGE);
+			}
+			else if (StrEquals(tok,"NoStickyDesk"))
+			{
+				flags &= ~(SHOW_STICKY_ON_DESK);
 			}
 			else if (StrEquals(tok,"Sticky"))
 			{
-				flags |= SHOW_STICKY;
+				flags |= SHOW_STICKY_ON_PAGE;
+				flags |= SHOW_STICKY_ON_DESK;
+			}
+			else if (StrEquals(tok,"StickyPage"))
+			{
+				flags |= SHOW_STICKY_ON_PAGE;
+			}
+			else if (StrEquals(tok,"StickyDesk"))
+			{
+				flags |= SHOW_STICKY_ON_DESK;
 			}
 			else if (StrEquals(tok,"OnlySticky"))
 			{
-				flags = SHOW_STICKY;
+				flags = SHOW_STICKY_ON_PAGE;
+				flags = SHOW_STICKY_ON_DESK;
+			}
+			else if (StrEquals(tok,"OnlyStickyPage"))
+			{
+				flags = SHOW_STICKY_ON_PAGE;
+			}
+			else if (StrEquals(tok,"OnlyStickyDesk"))
+			{
+				flags = SHOW_STICKY_ON_DESK;
 			}
 			else if (StrEquals(tok,"UseListSkip"))
 			{
@@ -723,13 +752,22 @@ void CMD_WindowList(F_CMD_ARGS)
 					/* don't want icons - skip */
 					continue;
 				}
-				if (!(flags & SHOW_STICKY) && (IS_STICKY(t)))
+				if (!(flags & SHOW_STICKY_ON_PAGE) &&
+				    (IS_STICKY_ON_PAGE(t)))
+				{
+					/* don't want sticky ones - skip */
+					continue;
+				}
+				if (!(flags & SHOW_STICKY_ON_DESK) &&
+				    (IS_STICKY_ON_DESK(t)))
 				{
 					/* don't want sticky ones - skip */
 					continue;
 				}
 				if (!(flags & SHOW_NORMAL) &&
-				    !((IS_ICONIFIED(t)) || (IS_STICKY(t))))
+				    !(IS_ICONIFIED(t) ||
+				      IS_STICKY_ON_PAGE(t) ||
+				      IS_STICKY_ON_DESK(t)))
 				{
 					/* don't want "normal" ones - skip */
 					continue;
@@ -944,7 +982,8 @@ void CMD_WindowList(F_CMD_ARGS)
 					}
 					strcat(tname, loc);
 
-					if (IS_STICKY(t))
+					if (IS_STICKY_ON_PAGE(t) ||
+					    IS_STICKY_ON_DESK(t))
 					{
 						strcat(tname, " S");
 					}
