@@ -21,14 +21,10 @@
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
-#include "libs/FShape.h"
-#include "libs/fvwmrect.h"
-#include "libs/gravity.h"
 #include "fvwm.h"
 #include "externs.h"
+#include "execcontext.h"
 #include "misc.h"
-#include "cursor.h"
-#include "functions.h"
 #include "commands.h"
 #include "screen.h"
 #include "module_interface.h"
@@ -80,17 +76,12 @@ void CMD_WindowShade(F_CMD_ARGS)
 	char *naction;
 	Bool do_force_shading;
 	Bool has_dir;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(
-		    eventp,&w,&fw,&context, CRS_SELECT,ButtonRelease))
+	if (IS_ICONIFIED(fw))
 	{
 		return;
 	}
-	if (fw == NULL || IS_ICONIFIED(fw))
-	{
-		return;
-	}
-
 	token = PeekToken(action, &naction);
 	if (StrEquals("shadeagain", token))
 	{
