@@ -17,7 +17,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
@@ -30,38 +30,38 @@
  Port by Dan Espen, no additional copyright
 */
 
-#include "config.h"			/* must be first */
+#include "config.h"                     /* must be first */
 
 #include <stdio.h>
-#include <X11/Xproto.h>			/* for X functions in general */
-#include "fvwmlib.h"			/* prototype GetShadow GetHilit */
+#include <X11/Xproto.h>                 /* for X functions in general */
+#include "fvwmlib.h"                    /* prototype GetShadow GetHilit */
 #include "Colorset.h"
+#include "PictureBase.h"
+#include "PictureUtils.h"
 
-#define	PCT_BRIGHTNESS			(6 * 0xffff / 100)
+#define PCT_BRIGHTNESS                  (6 * 0xffff / 100)
 
 /* How much lighter/darker to make things in default routine */
 
-#define	PCT_DARK_BOTTOM		70	/* lighter (less dark, actually) */
-#define	PCT_DARK_TOP		50	/* lighter */
-#define	PCT_LIGHT_BOTTOM	55	/* darker */
-#define	PCT_LIGHT_TOP		80	/* darker */
-#define	PCT_MEDIUM_BOTTOM_BASE	40	/* darker */
-#define	PCT_MEDIUM_BOTTOM_RANGE	25
-#define	PCT_MEDIUM_TOP_BASE	60	/* lighter */
-#define	PCT_MEDIUM_TOP_RANGE	-30
+#define PCT_DARK_BOTTOM         70      /* lighter (less dark, actually) */
+#define PCT_DARK_TOP            50      /* lighter */
+#define PCT_LIGHT_BOTTOM        55      /* darker */
+#define PCT_LIGHT_TOP           80      /* darker */
+#define PCT_MEDIUM_BOTTOM_BASE  40      /* darker */
+#define PCT_MEDIUM_BOTTOM_RANGE 25
+#define PCT_MEDIUM_TOP_BASE     60      /* lighter */
+#define PCT_MEDIUM_TOP_RANGE    -30
 
 /* The "brightness" of an RGB color.  The "right" way seems to use
  * empirical values like the default thresholds below, but it boils down
  * to red is twice as bright as blue and green is thrice blue.
  */
 
-#define	BRIGHTNESS(r,g,b) (2*(int)(r) + 3*(int)(g) + 1*(int)(b))
+#define BRIGHTNESS(r,g,b) (2*(int)(r) + 3*(int)(g) + 1*(int)(b))
 
 /* From Xm.h on Solaris */
-#define XmDEFAULT_DARK_THRESHOLD	15
-#define XmDEFAULT_LIGHT_THRESHOLD	85
-extern Colormap Pcmap;
-extern Display *Pdpy;
+#define XmDEFAULT_DARK_THRESHOLD        15
+#define XmDEFAULT_LIGHT_THRESHOLD       85
 
 static XColor color;
 
@@ -266,11 +266,15 @@ XColor *GetShadowColor(Pixel background)
 
 Pixel GetShadow(Pixel background)
 {
-  XColor *colorp;
+	XColor *colorp;
 
-  colorp = GetShadowColor(background);
-  XAllocColor (Pdpy, Pcmap, colorp);
-  return colorp->pixel;
+	colorp = GetShadowColor(background);
+	PictureAllocColor(Pdpy, Pcmap, colorp, True);
+	if (colorp->pixel == background)
+	{
+		colorp->pixel = PictureGetNextColor(colorp->pixel, 1);
+	}
+	return colorp->pixel;
 }
 
 XColor *GetHiliteColor(Pixel background)
@@ -284,7 +288,11 @@ Pixel GetHilite(Pixel background)
   XColor *colorp;
 
   colorp = GetHiliteColor(background);
-  XAllocColor (Pdpy, Pcmap, colorp);
+  PictureAllocColor(Pdpy, Pcmap, colorp, True);
+  if (colorp->pixel == background)
+  {
+	  colorp->pixel = PictureGetNextColor(colorp->pixel, -1);
+  }
   return colorp->pixel;
 }
 
@@ -305,7 +313,7 @@ XColor *GetForeShadowColor(Pixel foreground, Pixel background)
 	fg[1] = color.green;
 	fg[2] = color.blue;
 	bg[0] = bg_color.red;
-	bg[1]=	bg_color.green;
+	bg[1]=  bg_color.green;
 	bg[2] = bg_color.blue;
 
 	for (i=0; i<3; i++)
@@ -335,7 +343,40 @@ Pixel GetForeShadow(Pixel foreground, Pixel background)
   XColor *colorp;
 
   colorp = GetForeShadowColor(foreground, background);
-  XAllocColor (Pdpy, Pcmap, colorp);
+  PictureAllocColor(Pdpy, Pcmap, colorp, True);
+  if (colorp->pixel == background)
+  {
+	  colorp->pixel = PictureGetNextColor(colorp->pixel, 1);
+  }
+  return colorp->pixel;
+}
+
+XColor *GetTintedColor(Pixel in, Pixel tint, int percent)
+{
+	XColor tint_color;
+
+	memset(&color, 0, sizeof(color));
+	memset(&tint_color, 0, sizeof(tint_color));
+	color.pixel = in;
+	XQueryColor(Pdpy, Pcmap, &color);
+	tint_color.pixel = tint;
+	XQueryColor(Pdpy, Pcmap, &tint_color);
+
+	color.red = (unsigned short)
+		(((100-percent)*color.red + tint_color.red * percent) / 100);
+	color.green = (unsigned short)
+		(((100-percent)*color.green + tint_color.green * percent) / 100);
+	color.blue = (unsigned short)
+		(((100-percent)*color.blue + tint_color.blue * percent) / 100);
+	return &color;
+}
+
+Pixel GetTintedPixel(Pixel in, Pixel tint, int percent)
+{
+  XColor *colorp;
+
+  colorp = GetTintedColor(in, tint, percent);
+  PictureAllocColor(Pdpy, Pcmap, colorp, True);
   return colorp->pixel;
 }
 
@@ -344,7 +385,7 @@ Pixel GetForeShadow(Pixel foreground, Pixel background)
  * address 'output'.  It is either in rgb format ("rgb:rrrr/gggg/bbbb") if
  * use_hash is False or in hash notation ("#rrrrggggbbbb") if use_hash is true.
  * The return value is the number of characters used by the string.  The
- * rgb values of the output are undefined if the colorcell is invalid.	The
+ * rgb values of the output are undefined if the colorcell is invalid.  The
  * memory area pointed at by 'output' must be at least 64 bytes (in case of
  * future extensions and multibyte characters).*/
 int pixel_to_color_string(
@@ -413,7 +454,7 @@ Pixel GetSimpleColor(char *name)
 		fprintf(stderr, "Cannot parse color \"%s\"\n",
 			name ? name : "<blank>");
 	}
-	else if (!XAllocColor (Pdpy, Pcmap, &color))
+	else if (!PictureAllocColor(Pdpy, Pcmap, &color, True))
 	{
 		fprintf(stderr, "Cannot allocate color \"%s\"\n", name);
 	}
@@ -472,7 +513,7 @@ Pixel GetColor(char *name)
 			color.pixel = Colorset[cs].shadow;
 			break;
 		}
-		if (!XAllocColor(Pdpy, Pcmap, &color))
+		if (!PictureAllocColor(Pdpy, Pcmap, &color, True))
 		{
 			fprintf(stderr, "Cannot allocate color %d from"
 				" colorset %d\n", i, cs);
@@ -494,7 +535,7 @@ Pixel fvwmlib_clone_color(Pixel p)
 
 	c.pixel = p;
 	XQueryColor(Pdpy, Pcmap, &c);
-	if (!XAllocColor(Pdpy, Pcmap, &c))
+	if (!PictureAllocColor(Pdpy, Pcmap, &c, True))
 	{
 		fprintf(stderr, "Cannot allocate clone Pixel %d\n", (int)p);
 		return 0;
