@@ -44,6 +44,7 @@
 Bool FRenderExtensionSupported = False;
 int FRenderErrorBase = -10000;
 int FRenderMajorOpCode = -10000;
+int FRenderAlphaDepth = 0;
 
 /* ---------------------------- exported variables (globals) ---------------- */
 
@@ -53,16 +54,40 @@ void FRenderInit(Display *dpy)
 {
 	int event_basep;
 
-	if (!XRenderSupport)
-		return;
-
-	if (!(FRenderExtensionSupported = XQueryExtension(dpy, "RENDER",
-							  &FRenderMajorOpCode,
-							  &event_basep,
-							  &FRenderErrorBase)))
+	FRenderAlphaDepth = 8;
+	if (!XRenderSupport || !(FRenderExtensionSupported = XQueryExtension(
+		dpy, "RENDER", &FRenderMajorOpCode, &event_basep,
+		&FRenderErrorBase)))
 	{
+		int *pmf = NULL;
+		int i,n;
+		int alpha_depth = 0;
+
 		FRenderErrorBase = -10000;
 		FRenderMajorOpCode = -10000;
+		FRenderExtensionSupported = 0;
+		pmf = XListDepths(dpy, DefaultScreen(dpy), &n);
+		if (pmf)
+		{
+			i = 0;
+			while(i < n)
+			{
+				if (pmf[i] == 8)
+				{
+					alpha_depth = 8;
+					i = n-1;
+				}
+				else if (pmf[i] >= 8 &&
+					 (pmf[i] < alpha_depth ||
+					  alpha_depth == 0))
+				{
+					alpha_depth = pmf[i];
+				}
+				i++;
+			}
+			XFree(pmf);
+		}
+		FRenderAlphaDepth = alpha_depth;
 	}
 }
 
@@ -82,6 +107,11 @@ Bool FRenderGetExtensionSupported(void)
 	return FRenderExtensionSupported;
 }
 
+int FRenderGetAlphaDepth(void)
+{
+	return FRenderAlphaDepth;
+}
+
 Bool FRenderGetErrorText(int code, char *msg)
 {
 
