@@ -161,38 +161,69 @@ FftFont *FftGetRotatedFont(
 	Display *dpy, FftFont *f, rotation_type rotation)
 {
 	FftPattern *rotated_pat;
-	FftMatrix m;
-	int dummy;
+	FftMatrix r,b;
+	FftMatrix *pm = NULL;
+	FftFont *rf = NULL;
 
 	if (f == NULL)
 		return NULL;
 
 	rotated_pat = FftPatternDuplicate(f->pattern);
-	if (rotated_pat == NULL)
-		return NULL;
 
-	dummy = FftPatternDel(rotated_pat, FFT_MATRIX);
+	if (rotated_pat == NULL)
+	{
+		return NULL;
+	}
 
 	if (rotation == ROTATION_90)
 	{
-		FFT_SET_ROTATED_90_MATRIX(&m);
+		FFT_SET_ROTATED_90_MATRIX(&r);
 	}
 	else if (rotation == ROTATION_180)
 	{
-		FFT_SET_ROTATED_180_MATRIX(&m);
+		FFT_SET_ROTATED_180_MATRIX(&r);
 	}
 	else if (rotation == ROTATION_270)
 	{
-		FFT_SET_ROTATED_270_MATRIX(&m);
+		FFT_SET_ROTATED_270_MATRIX(&r);
 	}
 	else
 	{
-		return NULL;
+		goto bail;
 	}
 
-	if (!FftPatternAddMatrix(rotated_pat, FFT_MATRIX, &m))
-		return NULL;
-	return FftFontOpenPattern(dpy, rotated_pat);
+	if ((FftPatternGetMatrix(
+		rotated_pat, FFT_MATRIX, 0, &pm) == FftResultMatch) && pm)
+	{
+		/* rotate the matrice */
+		b.xx = r.xx * pm->xx + r.xy * pm->yx;
+		b.xy = r.xx * pm->xy + r.xy * pm->yy;
+		b.yx = r.yx * pm->xx + r.yy * pm->yx;
+		b.yy = r.yx * pm->xy + r.yy * pm->yy;
+	}
+	else
+	{
+		b.xx = r.xx;
+		b.xy = r.xy;
+		b.yx = r.yx;
+		b.yy = r.yy;
+	}
+	if (FftPatternDel(rotated_pat, FFT_MATRIX))
+	{
+		/* nothing */
+	}
+	if (!FftPatternAddMatrix(rotated_pat, FFT_MATRIX, &b))
+	{
+		goto bail;
+	}
+	rf = FftFontOpenPattern(dpy, rotated_pat);
+
+ bail:
+	if (!rf && rotated_pat)
+	{
+		FftPatternDestroy(rotated_pat);
+	}
+	return rf;
 }
 
 void FftPDumyFunc(void)
@@ -238,8 +269,12 @@ FftFontType *FftGetFont(Display *dpy, char *fontname, char *module)
 	FftFontType *fftf = NULL;
 	char *fn = NULL, *str_enc = NULL;
 	FlocaleCharset *fc;
+	FftPattern *src_pat = NULL, *load_pat = NULL;
+	FftMatrix *a = NULL;
+	FftResult result;
 
-	if (!FftSupport || !FRenderGetExtensionSupported())
+	result = 0;
+	if (!FftSupport || !(FRenderGetExtensionSupported() || 1) )
 	{
 		return NULL;
 	}
@@ -281,10 +316,53 @@ FftFontType *FftGetFont(Display *dpy, char *fontname, char *module)
 	{
 		fn = fontname;
 	}
-	fftfont = FftFontOpenName(dpy, fftscreen, fn);
+	if ((src_pat = FftNameParse(fn)) == NULL)
+	{
+		goto bail;
+	}
+	if ((load_pat = FftFontMatch(dpy, fftscreen, src_pat, &result)) == NULL)
+	{
+		goto bail;
+	}
+	/* safty check */
+	if (FftPatternGetMatrix(
+		load_pat, FFT_MATRIX, 0, &a) == FftResultMatch && a)
+	{
+		FftMatrix b;
+		Bool cm = False;
+
+		if (a->xx < 0)
+		{
+			a->xx = -a->xx;
+			cm = True;
+		}
+		if (a->yx != 0)
+		{
+			a->yx = 0;
+			cm = True;
+		}
+		if (cm)
+		{
+			b.xx = a->xx;
+			b.xy = a->xy;
+			b.yx = a->yx;
+			b.yy = a->yy;
+			if (FftPatternDel(load_pat, FFT_MATRIX))
+			{
+				/* nothing */
+			}
+			if (!FftPatternAddMatrix(load_pat, FFT_MATRIX, &b))
+			{
+				goto bail;
+			}
+		}
+	}
+	/* FIXME: other safty checking ? */
+	fftfont = FftFontOpenPattern(dpy, load_pat);
+
 	if (!fftfont)
 	{
-		return NULL;
+		goto bail;
 	}
 	fftf = (FftFontType *)safemalloc(sizeof(FftFontType));
 	fftf->fftfont = fftfont;
@@ -293,6 +371,15 @@ FftFontType *FftGetFont(Display *dpy, char *fontname, char *module)
 	fftf->fftfont_rotated_270 = NULL;
 	FftSetupEncoding(dpy, fftf, str_enc, module);
 
+ bail:
+	if (src_pat)
+	{
+		FftPatternDestroy(src_pat);
+	}
+	if (!fftf && load_pat)
+	{
+		FftPatternDestroy(load_pat);
+	}
 	return fftf;
 }
 
@@ -328,7 +415,11 @@ void FftDrawString(
 						  fws->flags.text_rotation);
 		}
 		uf = fftf->fftfont_rotated_90;
+#ifdef FFT_BUGGY_FREETYPE
+		y = fws->y +  FftTextWidth(flf, fws->e_str, len);
+#else
 		y = fws->y;
+#endif
 		x = fws->x - FLF_SHADOW_BOTTOM_SIZE(flf);
 	}
 	else if (fws->flags.text_rotation == ROTATION_180)
@@ -352,7 +443,11 @@ void FftDrawString(
 						  fws->flags.text_rotation);
 		}
 		uf = fftf->fftfont_rotated_270;
+#ifdef FFT_BUGGY_FREETYPE
+		y = fws->y;
+#else
 		y = fws->y + FftTextWidth(flf, fws->e_str, len);
+#endif
 		x = fws->x - FLF_SHADOW_UPPER_SIZE(flf);
 	}
 	else
@@ -512,3 +607,30 @@ int FftTextWidth(FlocaleFont *flf, char *str, int len)
 
 	return result;
 }
+
+
+void FftPrintPatternInfo(FftFont *f, Bool vertical)
+{
+	/* FftPatternPrint use stdout */
+	fflush (stderr);
+	printf("\n        height: %i, ascent: %i, descent: %i, maw: %i\n",
+	       f->height, f->ascent, f->descent, f->max_advance_width);
+	if (!vertical)
+	{
+		printf("        ");
+		FftPatternPrint(f->pattern);
+	}
+	else
+	{
+		FftMatrix *pm = NULL;
+
+		if (FftPatternGetMatrix(
+			f->pattern, FFT_MATRIX, 0, &pm) == FftResultMatch && pm)
+		{
+			printf("         matrix: (%f %f %f %f)\n",
+			       pm->xx, pm->xy, pm->yx, pm->yy);
+		}
+	}
+	fflush (stdout);
+	return;
+}
