@@ -125,6 +125,9 @@
 #define FLF_SHADOW_BOTTOM_SIZE(flf) \
 	(FLF_SHADOW_FULL_SIZE((flf)) * FLF_MULTIDIR_HAS_BOTTOM((flf)))
 
+#define FLF_FONT_HAS_ALPHA(flf,cset) \
+     ((flf && flf->fftf.fftfont != None) ||                      \
+      (0 && cset >= 0 && Colorset[cset].fg_alpha_percent < 100))
 /* ---------------------------- type definitions ---------------------------- */
 
 typedef struct FlocaleCharset
@@ -308,6 +311,11 @@ Bool FlocaleGetShadowTextPosition(
  */
 int FlocaleTextWidth(FlocaleFont *ff, char *str, int sl);
 
+/*
+ * "y" (or "x" position if rotated and Xft font) of the text relatively to 0
+ */
+int FlocaleGetMinOffset(FlocaleFont *flf, rotation_type rotation);
+
 /*
  * Allocate memory for a FlocaleWinString intialized to 0
  */
@@ -343,6 +351,11 @@ Bool FlocaleTextListToTextProperty(
 	Display *dpy, char **list, int count, XICCEncodingStyle style,
 	XTextProperty *text_prop_return);
 
+/* ***************************************************************************
+ * Info
+ * ***************************************************************************/
+void FlocalePrintLocaleInfo(Display *dpy, int verbose);
+
 /* ***************************************************************************
  * Misc
  * ***************************************************************************/
