@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -17,47 +18,141 @@
 ** Parse.c: routines for parsing in fvwm & modules
 */
 
+/* ---------------------------- included header files ----------------------- */
+
 #include "config.h"
 
 #include <stdio.h>
 #include <ctype.h>
-#include <string.h>
 #include <stdlib.h>
 
 #include "safemalloc.h"
 #include "Strings.h"
 #include "Parse.h"
 
+/* ---------------------------- local definitions --------------------------- */
+
+/* ---------------------------- local macros -------------------------------- */
+
+/* ---------------------------- imports ------------------------------------- */
+
+/* ---------------------------- included code files ------------------------- */
+
+/* ---------------------------- local types --------------------------------- */
+
+/* ---------------------------- forward declarations ------------------------ */
+
+/* ---------------------------- local variables ----------------------------- */
+
+/* ---------------------------- exported variables (globals) ---------------- */
+
+/* ---------------------------- local functions ----------------------------- */
+
+/* Copies a token beginning at src to a previously allocated area dest. dest
+ * must be large enough to hold the token. Leading whitespace causes the token
+ * to be NULL. */
+/* NOTE: CopyToken can be called with dest == src. The token will be copied
+ * back down over the src string. */
+static char *CopyToken(
+	char *src, char *dest, char *spaces, int snum, char *delims, int dnum,
+	char *out_delim)
+{
+	int len = 0;
+
+	while (*src != 0 && !(isspace((unsigned char)*src) ||
+			      (snum && strchr(spaces, *src)) ||
+			      (dnum && strchr(delims, *src))))
+	{
+		/* Check for quoted text */
+		if (IsQuote(*src))
+		{
+			char c = *src;
+
+			src++;
+			while ((*src != c)&&(*src != 0))
+			{
+				if ((*src == '\\' && *(src+1) != 0))
+				{
+					/* Skip over backslashes */
+					src++;
+				}
+				if (len < MAX_TOKEN_LENGTH - 1)
+				{
+					len++;
+					*(dest++) = *(src++);
+				}
+				else
+				{
+					/* token too long, just skip rest */
+					src++;
+				}
+			}
+			if (*src == c)
+			{
+				src++;
+			}
+		}
+		else
+		{
+			if ((*src == '\\' && *(src+1) != 0))
+				/* Skip over backslashes */
+				src++;
+			if (len < MAX_TOKEN_LENGTH - 1)
+			{
+				len++;
+				*(dest++) = *(src++);
+			}
+			else
+			{
+				/* token too long, just skip rest of token */
+				src++;
+			}
+		}
+	}
+	if (out_delim)
+	{
+		*out_delim = *src;
+	}
+	*dest = 0;
+	if (*src != 0)
+	{
+		src++;
+	}
+
+	return src;
+}
+
+/* ---------------------------- interface functions ------------------------- */
 
 /* This function escapes all occurences of the characters in the string qchars
  * in the string as with a preceding echar character.  The resulting string is
  * returned in a malloced memory area. */
 char *EscapeString(char *s, const char *qchars, char echar)
 {
-  char *t;
-  char *ret;
-  int len;
+	char *t;
+	char *ret;
+	int len;
 
-  for (len = 1, t = s; *t ; t++, len++)
-  {
-    if (strchr(qchars, *t) != NULL)
-    {
-      len++;
-    }
-  }
-  ret = (char *)safemalloc(len);
-  for (t = ret; *s; s++, t++)
-  {
-    if (strchr(qchars, *s) != NULL)
-    {
-      *t = echar;
-      t++;
-    }
-    *t = *s;
-  }
-  *t = 0;
+	for (len = 1, t = s; *t ; t++, len++)
+	{
+		if (strchr(qchars, *t) != NULL)
+		{
+			len++;
+		}
+	}
+	ret = (char *)safemalloc(len);
+	for (t = ret; *s; s++, t++)
+	{
+		if (strchr(qchars, *s) != NULL)
+		{
+			*t = echar;
+			t++;
+		}
+		*t = *s;
+	}
+	*t = 0;
 
-  return ret;
+	return ret;
 }
 
 /* If the string s begins with a quote chracter SkipQuote returns a pointer
@@ -81,48 +176,64 @@ char *EscapeString(char *s, const char *qchars, char echar)
  *
  * The defaults are used if NULL is passed for the corresponding string.
  */
-char *SkipQuote(char *s, const char *qlong, const char *qstart,
-		const char *qend)
+char *SkipQuote(
+	char *s, const char *qlong, const char *qstart, const char *qend)
 {
-  char *t;
+	char *t;
 
-  if (s == NULL || *s == 0)
-    return s;
-  if (!qlong)
-    qlong = "\"'`";
-  if (!qstart)
-    qstart = "";
-  if (!qend)
-    qend = "";
+	if (s == NULL || *s == 0)
+	{
+		return s;
+	}
+	if (!qlong)
+	{
+		qlong = "\"'`";
+	}
+	if (!qstart)
+	{
+		qstart = "";
+	}
+	if (!qend)
+	{
+		qend = "";
+	}
 
-  if (*s == '\\' && s[1] != 0)
-    return s+2;
-  else if (*qlong && (t = strchr(qlong, *s)))
-  {
-    char c = *t;
+	if (*s == '\\' && s[1] != 0)
+	{
+		return s+2;
+	}
+	else if (*qlong && (t = strchr(qlong, *s)))
+	{
+		char c = *t;
 
-    s++;
-    while(*s && *s != c)
-    {
-      /* Skip over escaped text, ie \quote */
-      if(*s == '\\' && *(s+1) != 0)
-	s++;
-      s++;
-    }
-    if(*s == c)
-      s++;
-    return s;
-  }
-  else if (*qstart && (t = strchr(qstart, *s)))
-  {
-    char c = *((t - qstart) + qend);
+		s++;
+		while (*s && *s != c)
+		{
+			/* Skip over escaped text, ie \quote */
+			if (*s == '\\' && *(s+1) != 0)
+			{
+				s++;
+			}
+			s++;
+		}
+		if (*s == c)
+		{
+			s++;
+		}
+		return s;
+	}
+	else if (*qstart && (t = strchr(qstart, *s)))
+	{
+		char c = *((t - qstart) + qend);
 
-    while (*s && *s != c)
-      s = SkipQuote(s, qlong, "", "");
-    return (*s == c) ? ++s : s;
-  }
+		while (*s && *s != c)
+		{
+			s = SkipQuote(s, qlong, "", "");
+		}
+		return (*s == c) ? ++s : s;
+	}
 
-  return ++s;
+	return ++s;
 }
 
 /* Returns a string up to the first character from the string delims in a
@@ -131,30 +242,37 @@ char *SkipQuote(char *s, const char *qlong, const char *qstart,
  * of this call is a pointer to the first character after the delimiter or
  * to the terminating '\0'. Quoting is handled like in SkipQuote. If sin is
  * NULL, the function returns NULL in *sout. */
-char *GetQuotedString(char *sin, char **sout, const char *delims,
-		      const char *qlong, const char *qstart, const char *qend)
+char *GetQuotedString(
+	char *sin, char **sout, const char *delims, const char *qlong,
+	const char *qstart, const char *qend)
 {
-  char *t = sin;
-  unsigned int len;
+	char *t = sin;
+	unsigned int len;
 
-  if (!sout)
-    return NULL;
-  if (!sin)
-  {
-    *sout = NULL;
-    return NULL;
-  }
+	if (!sout)
+	{
+		return NULL;
+	}
+	if (!sin)
+	{
+		*sout = NULL;
+		return NULL;
+	}
 
-  while (*t && !strchr(delims, *t))
-    t = SkipQuote(t, qlong, qstart, qend);
-  len = t - sin;
-  *sout = (char *)safemalloc(len + 1);
-  memcpy(*sout, sin, len);
-  (*sout)[len] = 0;
-  if (*t)
-    t++;
+	while (*t && !strchr(delims, *t))
+	{
+		t = SkipQuote(t, qlong, qstart, qend);
+	}
+	len = t - sin;
+	*sout = (char *)safemalloc(len + 1);
+	memcpy(*sout, sin, len);
+	(*sout)[len] = 0;
+	if (*t)
+	{
+		t++;
+	}
 
-  return t;
+	return t;
 }
 
 /* SkipSpaces: returns a pointer to the first character in indata that is
@@ -163,78 +281,14 @@ char *GetQuotedString(char *sin, char **sout, const char *delims,
  * in indata. */
 char *SkipSpaces(char *indata, char *spaces, int snum)
 {
-  while (*indata != 0 && (isspace((unsigned char)*indata) ||
-			  (snum && strchr(spaces, *indata))))
-    indata++;
-  return indata;
-}
-
-/* Copies a token beginning at src to a previously allocated area dest. dest
- * must be large enough to hold the token. Leading whitespace causes the token
- * to be NULL. */
-/* NOTE: CopyToken can be called with dest == src. The token will be copied
- * back down over the src string. */
-static char *CopyToken(char *src, char *dest, char *spaces, int snum,
-		       char *delims, int dnum, char *out_delim)
-{
-  int len = 0;
-
-  while ( (*src != 0) && !(isspace((unsigned char)*src) ||
-			   (snum && strchr(spaces, *src)) ||
-			   (dnum && strchr(delims, *src))))
-  {
-    /* Check for quoted text */
-    if (IsQuote(*src))
-    {
-      char c = *src;
-
-      src++;
-      while((*src != c)&&(*src != 0))
-      {
-	if((*src == '\\' && *(src+1) != 0))
-	  /* Skip over backslashes */
-	  src++;
-	if (len < MAX_TOKEN_LENGTH - 1)
+	while (*indata != 0 && (isspace((unsigned char)*indata) ||
+				(snum && strchr(spaces, *indata))))
 	{
-	  len++;
-	  *(dest++) = *(src++);
+		indata++;
 	}
-	else
-	{
-	  /* token too long, just skip rest of token */
-	  src++;
-	}
-      }
-      if(*src == c)
-	src++;
-    }
-    else
-    {
-      if((*src == '\\' && *(src+1) != 0))
-	/* Skip over backslashes */
-	src++;
-      if (len < MAX_TOKEN_LENGTH - 1)
-      {
-	len++;
-	*(dest++) = *(src++);
-      }
-      else
-      {
-	/* token too long, just skip rest of token */
-	src++;
-      }
-    }
-  }
-  if (out_delim)
-    *out_delim = *src;
-  *dest = 0;
-  if (*src != 0)
-    src++;
-
-  return src;
+	return indata;
 }
 
-
 /*
 ** PeekToken:
 ** DoPeekToken: returns next token from string, leaving string intact
@@ -248,44 +302,52 @@ static char *CopyToken(char *src, char *dest, char *spaces, int snum,
 */
 /* NOTE: If indata is the pointer returned by a previous call to PeekToken or
  * DoPeekToken, the input string will be destroyed. */
-
-char *DoPeekToken(char *indata, char **token, char *spaces, char *delims,
-		  char *out_delim)
+char *DoPeekToken(
+	char *indata, char **token, char *spaces, char *delims, char *out_delim)
 {
-  char *end;
-  int snum;
-  int dnum;
-  static char tmptok[MAX_TOKEN_LENGTH];
+	char *end;
+	int snum;
+	int dnum;
+	static char tmptok[MAX_TOKEN_LENGTH];
 
-  snum = (spaces) ? strlen(spaces) : 0;
-  dnum = (delims) ? strlen(delims) : 0;
-  if (indata == NULL)
-  {
-    if (out_delim)
-      *out_delim = '\0';
-    *token = NULL;
-    return NULL;
-  }
-  indata = SkipSpaces(indata, spaces, snum);
-  end = CopyToken(indata, tmptok, spaces, snum, delims, dnum, out_delim);
+	snum = (spaces) ? strlen(spaces) : 0;
+	dnum = (delims) ? strlen(delims) : 0;
+	if (indata == NULL)
+	{
+		if (out_delim)
+		{
+			*out_delim = '\0';
+		}
+		*token = NULL;
+		return NULL;
+	}
+	indata = SkipSpaces(indata, spaces, snum);
+	end = CopyToken(indata, tmptok, spaces, snum, delims, dnum, out_delim);
 
-  if (tmptok[0] == 0)
-    *token = NULL;
-  else
-    *token = tmptok;
-  return end;
+	if (tmptok[0] == 0)
+	{
+		*token = NULL;
+	}
+	else
+	{
+		*token = tmptok;
+	}
+
+	return end;
 }
 
 char *PeekToken(char *indata, char **outdata)
 {
-  char *dummy;
-  char *token;
+	char *dummy;
+	char *token;
 
-  if (!outdata)
-    outdata = &dummy;
+	if (!outdata)
+	{
+		outdata = &dummy;
+	}
+	*outdata = DoPeekToken(indata, &token, NULL, NULL, NULL);
 
-  *outdata = DoPeekToken(indata, &token, NULL, NULL, NULL);
-  return token;
+	return token;
 }
 
 
@@ -300,15 +362,15 @@ char *PeekToken(char *indata, char **outdata)
 #if 0
 int CheckNTokens(char *indata, unsigned int n)
 {
-  unsigned int i;
-  char *token;
+	unsigned int i;
+	char *token;
 
-  for (i = 0, token = (char *)1; i < n && token != NULL; i++)
-  {
-    token = PeekToken(indata, NULL);
-  }
+	for (i = 0, token = (char *)1; i < n && token != NULL; i++)
+	{
+		token = PeekToken(indata, NULL);
+	}
 
-  return i;
+	return i;
 }
 #endif
 
@@ -318,42 +380,54 @@ int CheckNTokens(char *indata, unsigned int n)
 */
 int MatchToken(char *pstr,char *tok)
 {
-  int rc=0;
-  char *ntok;
+	int rc=0;
+	char *ntok;
 
-  DoPeekToken(pstr, &ntok, NULL, NULL, NULL);
-  if (ntok)
-    rc = (strcasecmp(tok,ntok)==0);
-  return rc;
+	DoPeekToken(pstr, &ntok, NULL, NULL, NULL);
+	if (ntok)
+	{
+		rc = (strcasecmp(tok,ntok)==0);
+	}
+
+	return rc;
 }
 
 /* unused at the moment */
 /*
-   function:		XCmpToken
-   description:		compare 1st word of s to 1st word of t
-   returns:		< 0  if s < t
-			= 0  if s = t
-			> 0  if s > t
+  function:	       XCmpToken
+  description:	       compare 1st word of s to 1st word of t
+  returns:	       < 0  if s < t
+  = 0  if s = t
+  > 0  if s > t
 */
-
-
 int XCmpToken(const char *s, const char **t)
 {
 	register const char *w=*t;
 
 	if (w==NULL)
-		return 1;		/* non existant word */
+	{
+		return 1;
+	}
 	if (s==NULL)
-		return -1;		/* non existant string */
+	{
+		return -1;
+	}
 
-	while ( *w && (*s==*w || toupper(*s)==toupper(*w)) )
-		s++,w++;
+	while (*w && (*s==*w || toupper(*s)==toupper(*w)) )
+	{
+		s++;
+		w++;
+	}
 
 	if ((*s=='\0' && (ispunct(*w) || isspace(*w)))||
 	    (*w=='\0' && (ispunct(*s) || isspace(*s))) )
+	{
 		return 0;			/* 1st word equal */
+	}
 	else
-		return toupper(*s)-toupper(*w);	/* smaller/greater */
+	{
+		return toupper(*s)-toupper(*w); /* smaller/greater */
+	}
 }
 
 
@@ -376,50 +450,54 @@ int XCmpToken(const char *s, const char **t)
  * characters (spaces are skipped before a token, delimiters are not).
  *
  **************************************************************************/
-char *DoGetNextToken(char *indata, char **token, char *spaces, char *delims,
-		     char *out_delim)
+char *DoGetNextToken(
+	char *indata, char **token, char *spaces, char *delims, char *out_delim)
 {
-  char *tmptok;
-  char *end;
+	char *tmptok;
+	char *end;
 
-  end = DoPeekToken(indata, &tmptok, spaces, delims, out_delim);
+	end = DoPeekToken(indata, &tmptok, spaces, delims, out_delim);
+	if (tmptok == NULL)
+	{
+		*token = NULL;
+	}
+	else
+	{
+		*token = safestrdup(tmptok);
+	}
 
-  if (tmptok == NULL)
-    *token = NULL;
-  else
-    *token = safestrdup(tmptok);
-
-  return end;
+	return end;
 }
 
 char *GetNextToken(char *indata, char **token)
 {
-  return DoGetNextToken(indata, token, NULL, NULL, NULL);
+	return DoGetNextToken(indata, token, NULL, NULL, NULL);
 }
 
 /* fetch next token and stop at next ',' */
 char *GetNextSimpleOption(char *indata, char **option)
 {
-  return DoGetNextToken(indata, option, NULL, ",", NULL);
+	return DoGetNextToken(indata, option, NULL, ",", NULL);
 }
 
 /* read multiple tokens up to next ',' or end of line */
 char *GetNextFullOption(char *indata, char **option)
 {
-  return GetQuotedString(indata, option, ",\n", NULL, NULL, NULL);
+	return GetQuotedString(indata, option, ",\n", NULL, NULL, NULL);
 }
 
 char *SkipNTokens(char *indata, unsigned int n)
 {
-  for ( ; n > 0 ; n--)
-    PeekToken(indata, &indata);
-  return indata;
+	for ( ; n > 0 ; n--)
+	{
+		PeekToken(indata, &indata);
+	}
+
+	return indata;
 }
 
 /*
- *
  * convenience functions
- *
  */
 
 /****************************************************************************
@@ -435,25 +513,29 @@ char *SkipNTokens(char *indata, unsigned int n)
  **************************************************************************/
 char *GetModuleResource(char *indata, char **resource, char *module_name)
 {
-  char *tmp;
-  char *next;
+	char *tmp;
+	char *next;
 
-  if (!module_name)
-    {
-      *resource = NULL;
-      return indata;
-    }
-  tmp = PeekToken(indata, &next);
-  if (!tmp)
-    return next;
+	if (!module_name)
+	{
+		*resource = NULL;
+		return indata;
+	}
+	tmp = PeekToken(indata, &next);
+	if (!tmp)
+	{
+		return next;
+	}
 
-  if (tmp[0] != '*' || strncasecmp(tmp+1, module_name, strlen(module_name)))
-    {
-      *resource = NULL;
-      return indata;
-    }
-  CopyString(resource, tmp+1+strlen(module_name));
-  return next;
+	if (tmp[0] != '*' ||
+	    strncasecmp(tmp+1, module_name, strlen(module_name)))
+	{
+		*resource = NULL;
+		return indata;
+	}
+	CopyString(resource, tmp+1+strlen(module_name));
+
+	return next;
 }
 
 
@@ -468,61 +550,76 @@ char *GetModuleResource(char *indata, char **resource, char *module_name)
  *
  **************************************************************************/
 int GetSuffixedIntegerArguments(
-  char *action, char **ret_action, int *retvals, int num, char *suffixlist,
-  int *ret_suffixnum)
+	char *action, char **ret_action, int *retvals, int num,
+	char *suffixlist, int *ret_suffixnum)
 {
-  int i;
-  int j;
-  int n;
-  char *token;
-  int suffixes;
+	int i;
+	int j;
+	int n;
+	char *token;
+	int suffixes;
 
-  suffixes = 0;				/* initialize */
-  if (suffixlist != 0) {		/* if passed a suffixlist */
-    suffixes = strlen(suffixlist);	/* save its length */
-  }
-
-  for (i = 0; i < num && action; i++)
-  {
-    token = PeekToken(action, &action);
-    if (token == NULL)
-      break;
-    if (sscanf(token, "%d%n", &(retvals[i]), &n) < 1)
-      break;
-    if (suffixes != 0 && ret_suffixnum != NULL)
-    {
-      int len;
-      char c;
-
-      len = strlen(token) - 1;
-      c = token[len];
-      if (isupper(c))
-	c = tolower(c);
-      for (j = 0; j < suffixes; j++)
-      {
-	char c2 = suffixlist[j];
-
-	if (isupper(c2))
-	  c2 = tolower(c2);
-	if (c == c2)
+	/* initialize */
+	suffixes = 0;
+	if (suffixlist != 0)
 	{
-	  ret_suffixnum[i] = j+1;
-	  break;
+		/* if passed a suffixlist save its length */
+		suffixes = strlen(suffixlist);
 	}
-      }
-      if (j == suffixes)
-	ret_suffixnum[i] = 0;
-    }
-    else if (token[n] != 0 && !isspace(token[n]))
-    {
-      /* there is a suffix but no suffix list was specified */
-      break;
-    }
-  }
-  if (ret_action != NULL)
-    *ret_action = action;
 
-  return i;
+	for (i = 0; i < num && action; i++)
+	{
+		token = PeekToken(action, &action);
+		if (token == NULL)
+		{
+			break;
+		}
+		if (sscanf(token, "%d%n", &(retvals[i]), &n) < 1)
+		{
+			break;
+		}
+		if (suffixes != 0 && ret_suffixnum != NULL)
+		{
+			int len;
+			char c;
+
+			len = strlen(token) - 1;
+			c = token[len];
+			if (isupper(c))
+			{
+				c = tolower(c);
+			}
+			for (j = 0; j < suffixes; j++)
+			{
+				char c2 = suffixlist[j];
+
+				if (isupper(c2))
+				{
+					c2 = tolower(c2);
+				}
+				if (c == c2)
+				{
+					ret_suffixnum[i] = j+1;
+					break;
+				}
+			}
+			if (j == suffixes)
+			{
+				ret_suffixnum[i] = 0;
+			}
+		}
+		else if (token[n] != 0 && !isspace(token[n]))
+		{
+			/* there is a suffix but no suffix list was specified */
+			break;
+		}
+	}
+	if (ret_action != NULL)
+	{
+		*ret_action = action;
+	}
+
+	return i;
 }
 
 /****************************************************************************
@@ -539,7 +636,7 @@ int GetSuffixedIntegerArguments(
  **************************************************************************/
 int SuffixToPercentValue(int value, int suffix, int *unit_table)
 {
-  return (value * unit_table[suffix]) / 100;
+	return (value * unit_table[suffix]) / 100;
 }
 
 /****************************************************************************
@@ -551,8 +648,8 @@ int SuffixToPercentValue(int value, int suffix, int *unit_table)
  **************************************************************************/
 int GetIntegerArguments(char *action, char **ret_action, int *retvals,int num)
 {
-  return GetSuffixedIntegerArguments(
-    action, ret_action, retvals, num, NULL, NULL);
+	return GetSuffixedIntegerArguments(
+		action, ret_action, retvals, num, NULL, NULL);
 }
 
 
@@ -572,33 +669,41 @@ int GetIntegerArguments(char *action, char **ret_action, int *retvals,int num)
  **************************************************************************/
 int GetTokenIndex(char *token, char **list, int len, char **next)
 {
-  int i;
-  int l;
-  int k;
+	int i;
+	int l;
+	int k;
 
-  if (!token || !list)
-  {
-    if (next)
-      *next = NULL;
-    return -1;
-  }
-  l = (len) ? len : strlen(token);
-  for (i = 0; list[i] != NULL; i++)
-  {
-    k = strlen(list[i]);
-    if (len < 0)
-      l = k;
-    if (len == 0 && k != l)
-      continue;
-    if (!strncasecmp(token, list[i], l))
-      break;
-  }
-  if (next)
-  {
-    *next = (list[i]) ? token + l : token;
-  }
+	if (!token || !list)
+	{
+		if (next)
+		{
+			*next = NULL;
+		}
+		return -1;
+	}
+	l = (len) ? len : strlen(token);
+	for (i = 0; list[i] != NULL; i++)
+	{
+		k = strlen(list[i]);
+		if (len < 0)
+		{
+			l = k;
+		}
+		else if (len == 0 && k != l)
+		{
+			continue;
+		}
+		if (!strncasecmp(token, list[i], l))
+		{
+			break;
+		}
+	}
+	if (next)
+	{
+		*next = (list[i]) ? token + l : token;
+	}
 
-  return (list[i]) ? i : -1;
+	return (list[i]) ? i : -1;
 }
 
 /***************************************************************************
@@ -611,21 +716,23 @@ int GetTokenIndex(char *token, char **list, int len, char **next)
  **************************************************************************/
 char *GetNextTokenIndex(char *action, char **list, int len, int *index)
 {
-  char *token;
-  char *next;
+	char *token;
+	char *next;
 
-  if (!index)
-    return action;
+	if (!index)
+	{
+		return action;
+	}
 
-  token = PeekToken(action, &next);
-  if (!token)
-  {
-    *index = -1;
-    return action;
-  }
-  *index = GetTokenIndex(token, list, len, NULL);
+	token = PeekToken(action, &next);
+	if (!token)
+	{
+		*index = -1;
+		return action;
+	}
+	*index = GetTokenIndex(token, list, len, NULL);
 
-  return (*index == -1) ? action : next;
+	return (*index == -1) ? action : next;
 }
 
 
@@ -638,75 +745,85 @@ char *GetNextTokenIndex(char *action, char **list, int len, int *index)
  **************************************************************************/
 int GetRectangleArguments(char *action, int *width, int *height)
 {
-  char *token;
-  int n;
+	char *token;
+	int n;
 
-  token = PeekToken(action, NULL);
-  if (!token)
-    return 0;
-  n = sscanf(token, "%d%*c%d", width, height);
+	token = PeekToken(action, NULL);
+	if (!token)
+	{
+		return 0;
+	}
+	n = sscanf(token, "%d%*c%d", width, height);
 
-  return (n == 2) ? 2 : 0;
+	return (n == 2) ? 2 : 0;
 }
 
 /* unit_io is input as well as output. If action has a postfix 'p' or 'P',
  * *unit_io is set to 100, otherwise it is left untouched. */
 int GetOnePercentArgument(char *action, int *value, int *unit_io)
 {
-  unsigned int len;
-  char *token;
-  int n;
+	unsigned int len;
+	char *token;
+	int n;
 
-  *value = 0;
-  if (!action)
-    return 0;
-  token = PeekToken(action, NULL);
-  if (!token)
-    return 0;
+	*value = 0;
+	if (!action)
+	{
+		return 0;
+	}
+	token = PeekToken(action, NULL);
+	if (!token)
+	{
+		return 0;
+	}
+	len = strlen(token);
+	/* token never contains an empty string, so this is ok */
+	if (token[len - 1] == 'p' || token[len - 1] == 'P')
+	{
+		*unit_io = 100;
+		token[len - 1] = '\0';
+	}
+	n = sscanf(token, "%d", value);
 
-  len = strlen(token);
-  /* token never contains an empty string, so this is ok */
-  if (token[len - 1] == 'p' || token[len - 1] == 'P')
-  {
-    *unit_io = 100;
-    token[len - 1] = '\0';
-  }
-  n = sscanf(token, "%d", value);
-
-  return n;
+	return n;
 }
 
 
-int GetTwoPercentArguments(char *action, int *val1, int *val2, int *val1_unit,
-			   int *val2_unit)
+int GetTwoPercentArguments(
+	char *action, int *val1, int *val2, int *val1_unit, int *val2_unit)
 {
-  char *tok1;
-  char *tok2;
-  char *next;
-  int n = 0;
+	char *tok1;
+	char *tok2;
+	char *next;
+	int n = 0;
 
-  *val1 = 0;
-  *val2 = 0;
+	*val1 = 0;
+	*val2 = 0;
 
-  tok1 = PeekToken(action, &next);
-  action = GetNextToken(action, &tok1);
-  if (!tok1)
-    return 0;
-  GetNextToken(action, &tok2);
-  if (GetOnePercentArgument(tok2, val2, val2_unit) == 1 &&
-      GetOnePercentArgument(tok1, val1, val1_unit) == 1)
-  {
-    free(tok1);
-    free(tok2);
-    return 2;
-  }
+	tok1 = PeekToken(action, &next);
+	action = GetNextToken(action, &tok1);
+	if (!tok1)
+	{
+		return 0;
+	}
+	GetNextToken(action, &tok2);
+	if (GetOnePercentArgument(tok2, val2, val2_unit) == 1 &&
+	    GetOnePercentArgument(tok1, val1, val1_unit) == 1)
+	{
+		free(tok1);
+		free(tok2);
+		return 2;
+	}
 
-  /* now try MxN style number, specifically for DeskTopSize: */
-  n = GetRectangleArguments(tok1, val1, val2);
-  free(tok1);
-  if (tok2)
-    free(tok2);
-  return n;
+	/* now try MxN style number, specifically for DeskTopSize: */
+	n = GetRectangleArguments(tok1, val1, val2);
+	free(tok1);
+	if (tok2)
+	{
+		free(tok2);
+	}
+
+	return n;
 }
 
 
@@ -718,62 +835,65 @@ int GetTwoPercentArguments(char *action, int *val1, int *val2, int *val1_unit,
  * the action itself is passed back in ret_action. If the no_toggle flag is
  * non-zero, the "toggle" string is handled as no match. */
 int ParseToggleArgument(
-  char *action, char **ret_action, int default_ret, char no_toggle)
+	char *action, char **ret_action, int default_ret, char no_toggle)
 {
-  int index;
-  int rc;
-  char *next;
-  char *optlist[] = {
-    "toggle",
-    "yes", "no",
-    "true", "false",
-    "on", "off",
-    "t", "f",
-    "y", "n",
-    NULL
-  };
+	int index;
+	int rc;
+	char *next;
+	char *optlist[] = {
+		"toggle",
+		"yes", "no",
+		"true", "false",
+		"on", "off",
+		"t", "f",
+		"y", "n",
+		NULL
+	};
 
-  next = GetNextTokenIndex(action, optlist, 0, &index);
-  if (index == 0 && no_toggle == 0)
-  {
-    /* toggle requested explicitly */
-    rc = -1;
-  }
-  else if (index == -1 || (index == 0 && no_toggle))
-  {
-    /* nothing selected, use default and don't modify action */
-    rc = default_ret;
-    next = action;
-  }
-  else
-  {
-    /* odd numbers denote True, even numbers denote False */
-    rc = (index & 1);
-  }
+	next = GetNextTokenIndex(action, optlist, 0, &index);
+	if (index == 0 && no_toggle == 0)
+	{
+		/* toggle requested explicitly */
+		rc = -1;
+	}
+	else if (index == -1 || (index == 0 && no_toggle))
+	{
+		/* nothing selected, use default and don't modify action */
+		rc = default_ret;
+		next = action;
+	}
+	else
+	{
+		/* odd numbers denote True, even numbers denote False */
+		rc = (index & 1);
+	}
+	if (ret_action)
+	{
+		*ret_action = next;
+	}
 
-  if (ret_action)
-    *ret_action = next;
-  return rc;
+	return rc;
 }
 
 /* Strips the path from 'path' and returns the last component in a malloc'ed
  * area. */
 char *GetFileNameFromPath(char *path)
 {
-  char *s;
-  char *name;
+	char *s;
+	char *name;
 
-  /* we get rid of the path from program name */
-  s = strrchr(path, '/');
-  if(s)
-  {
-    s++;
-  }
-  else
-  {
-    s = path;
-  }
-  name = (char *)safemalloc(strlen(s)+1);
-  strcpy(name, s);
-  return name;
+	/* we get rid of the path from program name */
+	s = strrchr(path, '/');
+	if (s)
+	{
+		s++;
+	}
+	else
+	{
+		s = path;
+	}
+	name = (char *)safemalloc(strlen(s)+1);
+	strcpy(name, s);
+
+	return name;
 }
