@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #ifndef Picture_H
@@ -32,9 +32,10 @@
  * ImagePath.. If ImagePath is NULL the default image path is used.
  * </description>
  */
-FvwmPicture* PGetFvwmPicture(Display* dpy, Window Root,
-			     char* ImagePath, char* pictureName,
-			     int color_limit);
+FvwmPicture* PGetFvwmPicture(
+	Display* dpy, Window win, char* ImagePath, const char* pictureName,
+	FvwmPictureAttributes fpa);
+
 /* <pubfunc>PCacheFvwmPicture
  * <description>
  * Return the FvwmPicture loaded from the file pictureName found in the
@@ -45,20 +46,19 @@ FvwmPicture* PGetFvwmPicture(Display* dpy, Window Root,
  * If ImagePath is NULL the default image path is used.
  * </description>
  */
-FvwmPicture* PCacheFvwmPicture(Display *dpy, Window Root,
-			       char* ImagePath, char* pictureName,
-			       int color_limit);
+FvwmPicture* PCacheFvwmPicture(
+	Display *dpy, Window win, char* ImagePath, const char* pictureName,
+	FvwmPictureAttributes fpa);
 
 /* <pubfunc>PLoadFvwmPictureFromPixmap
  * <description>
  * Return a FvwmPicture from the given data.
  * </description>
  */
-FvwmPicture *PLoadFvwmPictureFromPixmap(Display *dpy, Window Root,
-					char *name, Pixmap pixmap,
-					Pixmap mask,
-					Pixmap alpha,
-					int width, int height);
+FvwmPicture *PLoadFvwmPictureFromPixmap(
+	Display *dpy, Window win, char *name, Pixmap pixmap, Pixmap mask,
+	Pixmap alpha, int width, int height, int nalloc_pixels,
+	Pixel *alloc_pixels, int no_limit);
 
 /* <pubfunc>PDestroyFvwmPicture
  * <description>
@@ -67,17 +67,18 @@ FvwmPicture *PLoadFvwmPictureFromPixmap(Display *dpy, Window Root,
  * to really cache a picture from the given data.
  * </description>
  */
-FvwmPicture *PCacheFvwmPictureFromPixmap(Display *dpy, Window Root,
-					 char *name, Pixmap pixmap,
-					 Pixmap mask,Pixmap alpha,
-					 int width, int height);
+FvwmPicture *PCacheFvwmPictureFromPixmap(
+	Display *dpy, Window win, char *name, Pixmap pixmap,
+	Pixmap mask, Pixmap alpha, int width, int height, int nalloc_pixels,
+	Pixel *alloc_pixels, int no_limit);
+
 /* <pubfunc>PDestroyFvwmPicture
  * <description>
  * Remove a weight to the FvwmPicture p from the FvwmPicture cache.
  * If the weight is zero the allocated datas from p are freed
  * </description>
  */
-void PDestroyFvwmPicture(Display* dpy, FvwmPicture* p);
+void PDestroyFvwmPicture(Display *dpy, FvwmPicture *p);
 
 /* <pubfunc>PCloneFvwmPicture
  * <description>
