@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /****************************************************************************
@@ -32,7 +32,7 @@
  * as long as the copyright is kept intact.
  *
  * Romano Giannetti - Dipartimento di Ingegneria dell'Informazione
- *		      via Diotisalvi, 2	 PISA
+ *                    via Diotisalvi, 2  PISA
  * mailto:romano@iet.unipi.it
  * http://www.iet.unipi.it/~romano
  *
@@ -53,6 +53,7 @@
 
 #include <fvwmlib.h>
 #include "PictureBase.h"
+#include "PictureUtils.h"
 
 Bool Pdefault;
 Visual *Pvisual;
@@ -61,7 +62,16 @@ Colormap Pcmap;
 static Colormap FvwmCmap;
 unsigned int Pdepth;
 static unsigned int FvwmDepth;
-Display *Pdpy;		  /* Save area for display pointer */
+Display *Pdpy;            /* Save area for display pointer */
+Bool PUseDynamicColors;
+
+Pixel PWhitePixel;
+Pixel PBlackPixel;
+Pixel FvwmWhitePixel;
+Pixel FvwmBlackPixel;
+
+void PictureSetupWhiteAndBlack(void);
+
 
 void PictureInitCMap(Display *dpy) {
 	char *envp;
@@ -88,9 +98,58 @@ void PictureInitCMap(Display *dpy) {
 		Pcmap = DefaultColormap(dpy, screen);
 		Pdefault = True;
 	}
-	FvwmVisual = Pvisual;
-	FvwmDepth = Pdepth;
-	FvwmCmap = Pcmap;
+
+	PictureSetupWhiteAndBlack();
+	PictureSaveFvwmVisual();
+
+	/* initialise color limit */
+	PUseDynamicColors = 0;
+	PictureInitColors(PICTURE_CALLED_BY_MODULE, True, NULL, False, True);
+	return;
+}
+
+void PictureInitCMapRoot(
+	Display *dpy, Bool init_color_limit, PictureColorLimitOption *opt,
+	Bool use_my_color_limit, Bool init_dither)
+{
+	int screen = DefaultScreen(dpy);
+
+	Pdpy = dpy;
+
+	Pvisual = DefaultVisual(dpy, screen);
+	Pdepth = DefaultDepth(dpy, screen);
+	Pcmap = DefaultColormap(dpy, screen);
+	Pdefault = True;
+
+	PictureSetupWhiteAndBlack();
+	PictureSaveFvwmVisual();
+
+	/* initialise color limit */
+	PictureInitColors(
+		PICTURE_CALLED_BY_MODULE, init_color_limit, opt,
+		use_my_color_limit, init_dither);
+	return;
+}
+
+void PictureSetupWhiteAndBlack(void)
+{
+	XColor c;
+
+	if (!Pdefault)
+	{
+		c.flags = DoRed|DoGreen|DoBlue;
+		c.red = c.green = c.blue = 65535;
+		XAllocColor(Pdpy, Pcmap, &c);
+		PWhitePixel = c.pixel;
+		c.red = c.green = c.blue = 0;
+		XAllocColor(Pdpy, Pcmap, &c);
+		PBlackPixel = c.pixel;
+	}
+	else
+	{
+		PWhitePixel = WhitePixel(Pdpy, DefaultScreen(Pdpy));
+		PBlackPixel = BlackPixel(Pdpy, DefaultScreen(Pdpy));
+	}
 
 	return;
 }
@@ -102,7 +161,8 @@ void PictureUseDefaultVisual(void)
 	Pvisual = DefaultVisual(Pdpy, screen);
 	Pdepth = DefaultDepth(Pdpy, screen);
 	Pcmap = DefaultColormap(Pdpy, screen);
-
+	PWhitePixel = WhitePixel(Pdpy, DefaultScreen(Pdpy));
+	PBlackPixel = BlackPixel(Pdpy, DefaultScreen(Pdpy));
 	return;
 }
 
@@ -111,7 +171,8 @@ void PictureUseFvwmVisual(void)
 	Pvisual = FvwmVisual;
 	Pdepth = FvwmDepth;
 	Pcmap = FvwmCmap;
-
+	PWhitePixel = FvwmWhitePixel;
+	PBlackPixel = FvwmBlackPixel;
 	return;
 }
 
@@ -120,10 +181,41 @@ void PictureSaveFvwmVisual(void)
 	FvwmVisual = Pvisual;
 	FvwmDepth = Pdepth;
 	FvwmCmap = Pcmap;
-
+	FvwmWhitePixel = PWhitePixel;
+	FvwmBlackPixel = PBlackPixel;
 	return;
 }
 
+Pixel PictureWhitePixel(void)
+{
+	return PWhitePixel;
+}
+
+Pixel PictureBlackPixel(void)
+{
+	return PBlackPixel;
+}
+
+GC PictureDefaultGC(Display *dpy, Window win)
+{
+	static GC gc = None;
+
+	if (Pdepth == DefaultDepth(dpy, DefaultScreen(dpy)))
+	{
+		return DefaultGC(dpy, DefaultScreen(dpy));
+	}
+	if (gc == None)
+	{
+		XGCValues xgcv;
+
+		xgcv.foreground = PictureBlackPixel();
+		xgcv.background = PictureWhitePixel();
+		gc = fvwmlib_XCreateGC(dpy, win, 0, NULL);
+	}
+
+	return gc;
+}
+
 static char* imagePath = FVWM_IMAGEPATH;
 
 void PictureSetImagePath( const char* newpath )
@@ -149,13 +241,13 @@ char* PictureGetImagePath()
  * Oh well.
  *
  ****************************************************************************/
-char* PictureFindImageFile( const char* icon, const char* pathlist, int type )
+char* PictureFindImageFile(const char* icon, const char* pathlist, int type)
 {
-	if ( pathlist == NULL )
+	if (pathlist == NULL)
 	{
 		pathlist = imagePath;
 	}
 
-	return searchPath( pathlist, icon, ".gz", type );
+	return searchPath(pathlist, icon, ".gz", type);
 }
 
