@@ -6,12 +6,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /* ---------------------------- included header files ----------------------- */
@@ -53,25 +53,25 @@ struct _gravity_offset
 /************************************************************************
  *
  *  Procedure:
- *	gravity_get_offsets - map gravity to (x,y) offset signs for adding
- *		to x and y when window is mapped to get proper placement.
+ *      gravity_get_offsets - map gravity to (x,y) offset signs for adding
+ *              to x and y when window is mapped to get proper placement.
  *
  ************************************************************************/
 void gravity_get_offsets(int grav, int *xp,int *yp)
 {
 	static struct _gravity_offset gravity_offsets[11] =
 		{
-			{  0,  0 },	/* ForgetGravity */
-			{ -1, -1 },	/* NorthWestGravity */
-			{  0, -1 },	/* NorthGravity */
-			{  1, -1 },	/* NorthEastGravity */
-			{ -1,  0 },	/* WestGravity */
-			{  0,  0 },	/* CenterGravity */
-			{  1,  0 },	/* EastGravity */
-			{ -1,  1 },	/* SouthWestGravity */
-			{  0,  1 },	/* SouthGravity */
-			{  1,  1 },	/* SouthEastGravity */
-			{  0,  0 },	/* StaticGravity */
+			{  0,  0 },     /* ForgetGravity */
+			{ -1, -1 },     /* NorthWestGravity */
+			{  0, -1 },     /* NorthGravity */
+			{  1, -1 },     /* NorthEastGravity */
+			{ -1,  0 },     /* WestGravity */
+			{  0,  0 },     /* CenterGravity */
+			{  1,  0 },     /* EastGravity */
+			{ -1,  1 },     /* SouthWestGravity */
+			{  0,  1 },     /* SouthGravity */
+			{  1,  1 },     /* SouthEastGravity */
+			{  0,  0 },     /* StaticGravity */
 		};
 
 	if (grav < ForgetGravity || grav > StaticGravity)
@@ -407,27 +407,29 @@ int gravity_dir_to_sign_one_axis(
  *   5 if it is SE, SouthEast, BottomRight or DownRight
  *   6 if it is SW, SouthWest, BottomLeft or DownLeft
  *   7 if it is NW, NorthWest, TopLeft or UpLeft
+ *   8 if it is C, Center or Centre
  *   default_ret if no string matches.
  *
  * A pointer to the first character in action behind the token is returned
  * through ret_action in this case. ret_action may be NULL. If the token
  * matches none of these strings the default_ret value is returned and the
  * action itself is passed back in ret_action. */
-direction_type ParseDirectionArgument(
+direction_type gravity_parse_dir_argument(
 	char *action, char **ret_action, direction_type default_ret)
 {
 	int index;
 	int rc;
 	char *next;
 	char *optlist[] = {
-		"N",  "North",	   "Top",	  "t", "Up",	     "u",
-		"E",  "East",	   "Right",	  "r", "Right",	     "r",
-		"S",  "South",	   "Bottom",	  "b", "Down",	     "d",
-		"W",  "West",	   "Left",	  "l", "Left",	     "l",
-		"NE", "NorthEast", "TopRight",	  "tr", "UpRight",   "ur",
+		"N",  "North",     "Top",         "t", "Up",         "u",
+		"E",  "East",      "Right",       "r", "Right",      "r",
+		"S",  "South",     "Bottom",      "b", "Down",       "d",
+		"W",  "West",      "Left",        "l", "Left",       "l",
+		"NE", "NorthEast", "TopRight",    "tr", "UpRight",   "ur",
 		"SE", "SouthEast", "BottomRight", "br", "DownRight", "dr",
 		"SW", "SouthWest", "BottomLeft",  "bl", "DownLeft",  "dl",
-		"NW", "NorthWest", "TopLeft",	  "tl", "UpLeft",    "ul",
+		"NW", "NorthWest", "TopLeft",     "tl", "UpLeft",    "ul",
+		"C",  "Center",    "Centre",      NULL, NULL,        NULL,
 		NULL
 	};
 
@@ -450,12 +452,12 @@ direction_type ParseDirectionArgument(
 	return (direction_type)rc;
 }
 
-multi_direction_type ParseMultiDirectionArgument(
+multi_direction_type gravity_parse_multi_dir_argument(
 	char *action, char **ret_action)
 {
 	int rc = MULTI_DIR_NONE;
 	char *token, *str;
-	direction_type dir = ParseDirectionArgument(action, ret_action, -1);
+	direction_type dir = gravity_parse_dir_argument(action, ret_action, -1);
 
 	if (dir != -1)
 	{
@@ -474,22 +476,78 @@ multi_direction_type ParseMultiDirectionArgument(
 			rc = MULTI_DIR_NONE;
 		}
 	}
+
 	return (multi_direction_type)rc;
 }
 
-void GetNextMultiDirection(int dir_set, multi_direction_type *dir)
+void gravity_get_next_multi_dir(int dir_set, multi_direction_type *dir)
 {
 	if (*dir == MULTI_DIR_NONE)
 	{
-		*dir = FIRST_MULTI_DIR;
+		*dir = MULTI_DIR_FIRST;
 		if (dir_set & *dir)
+		{
 			return;
+		}
 	}
-	while(*dir != LAST_MULTI_DIR)
+	while(*dir != MULTI_DIR_LAST)
 	{
 		*dir = (*dir << 1);
 		if (dir_set & *dir)
+		{
 			return;
+		}
 	}
 	*dir = MULTI_DIR_NONE;
+
+	return;
+}
+
+direction_type gravity_multi_dir_to_dir(multi_direction_type mdir)
+{
+	direction_type dir = DIR_NONE;
+
+	for ( ; mdir != 0; dir++)
+	{
+		mdir = (mdir >> 1);
+	}
+	if (dir > DIR_ALL_MASK)
+	{
+		dir = DIR_NONE;
+	}
+
+	return dir;
+}
+
+void gravity_rotate_xy(rotation_type rot, int x, int y, int *ret_x, int *ret_y)
+{
+	int tx;
+	int ty;
+
+	switch (rot)
+	{
+	case ROTATION_90:
+		/* CW */
+		tx = -y;
+		ty = x;
+		break;
+	case ROTATION_180:
+		tx = -x;
+		ty = -y;
+		break;
+	case ROTATION_270:
+		/* CCW */
+		tx = y;
+		ty = -x;
+		break;
+	default:
+	case ROTATION_0:
+		tx = x;
+		ty = y;
+		break;
+	}
+	*ret_x = tx;
+	*ret_y = ty;
+
+	return;
 }
