@@ -67,6 +67,7 @@
 #include "parse.h" /* ParseConfiguration(), parse_window_geometry() */
 #include "icons.h" /* CreateIconWindow(), ConfigureIconWindow() */
 #include "draw.h"
+#include "dynamic.h"
 
 
 #define MW_EVENTS   (ExposureMask |\
@@ -93,10 +94,10 @@ static RETSIGTYPE TerminateHandler(int sig);
 void SetButtonSize(button_info*,int,int);
 /* main */
 void Loop(void);
-void RedrawWindow(button_info*);
+void RedrawWindow();
 void RecursiveLoadData(button_info*,int*,int*);
 void CreateUberButtonWindow(button_info*,int,int);
-int My_XNextEvent(Display *dpy, XEvent *event);
+int My_FNextEvent(Display *dpy, XEvent *event);
 void process_message(unsigned long type,unsigned long *body);
 extern void send_clientmessage (Display *disp, Window w, Atom a,
 				Time timestamp);
@@ -140,7 +141,7 @@ GC  NormalGC;
 /* needed for relief drawing only */
 GC  ShadowGC;
 
-static int Width,Height;
+int Width, Height;
 static int x= -30000,y= -30000;
 int w = -1;
 int h = -1;
@@ -207,7 +208,7 @@ static Window SwallowedWindow(button_info *b)
 int IsThereADestroyEvent(button_info *b)
 {
   XEvent event;
-  return XCheckIfEvent(Dpy,&event,DestroyedWindow,
+  return FCheckIfEvent(Dpy,&event,DestroyedWindow,
 		       (char*)SwallowedWindow(b));
 }
 
@@ -581,7 +582,7 @@ void SetTransparentBackground(button_info *ub,int w,int h)
       {
 	if (Ffont->font)
 	  XSetFont(Dpy,trans_gc,Ffont->font->fid);
-	DrawTitle(b,pmap_mask,trans_gc);
+	DrawTitle(b,pmap_mask,trans_gc,NULL,True);
       }
     }
     XSetRegion(Dpy, trans_gc, shape_r);
@@ -635,6 +636,7 @@ int main(int argc, char **argv)
   unsigned int depth;
   button_info *b,*ub;
   int geom_option_argc = 0;
+  XSetWindowAttributes xswa;
 
   FlocaleInit(LC_CTYPE, "", "", "FvwmButtons");
 
@@ -803,6 +805,16 @@ int main(int argc, char **argv)
   ShuffleButtons(UberButton);
   NumberButtons(UberButton);
 
+#ifdef DEBUG_INIT
+  fprintf(stderr,"OK\n%s: Creating Main Window ...",MyName);
+#endif
+
+  xswa.colormap = Pcmap;
+  xswa.border_pixel = 0;
+  xswa.background_pixmap = None;
+  MyWindow = XCreateWindow(Dpy, Root, 0,0,1,1,0, Pdepth, InputOutput, Pvisual,
+			   CWColormap|CWBackPixmap|CWBorderPixel, &xswa);
+
 #ifdef DEBUG_INIT
   fprintf(stderr,"OK\n%s: Loading data...\n",MyName);
 #endif
@@ -820,7 +832,7 @@ int main(int argc, char **argv)
   }
 
 #ifdef DEBUG_INIT
-  fprintf(stderr,"%s: Creating main window...",MyName);
+  fprintf(stderr,"%s: Configuring main window...",MyName);
 #endif
 
   CreateUberButtonWindow(UberButton,maxx,maxy);
@@ -857,7 +869,7 @@ int main(int argc, char **argv)
   i=-1;
   ub=UberButton;
   while(NextButton(&ub,&b,&i,0))
-    ConfigureIconWindow(b);
+    ConfigureIconWindow(b, NULL);
 
   if (FShapesSupported)
   {
@@ -919,17 +931,14 @@ void Loop(void)
   int x;
   int y;
   button_info *ub,*b;
-#ifndef OLD_EXPOSE
-  static int ex=10000,ey=10000,ex2=0,ey2=0;
   button_info *tmpb;
-#endif
   FlocaleNameString tmp;
 
   tmp.name = NULL;
   tmp.name_list = NULL;
   while( !isTerminated )
   {
-    if(My_XNextEvent(Dpy,&Event))
+    if(My_FNextEvent(Dpy,&Event))
     {
       if (FShapesSupported)
       {
@@ -949,70 +958,67 @@ void Loop(void)
       switch(Event.type)
       {
       case Expose:
-#ifdef OLD_EXPOSE
-	if(Event.xexpose.count == 0)
-	{
-	  if(Event.xexpose.window != MyWindow)
-	    break;
-	  button=-1;ub=UberButton;
-	  while(NextButton(&ub,&b,&button,1))
-	  {
-	    if(!ready && !(b->flags&b_Container))
-	      MakeButton(b);
-	    RedrawButton(b,1);
-	  }
-	  if(!ready)
-	    ready++;
-	}
-#else
-	if(Event.xexpose.window != MyWindow)
-	  break;
-	ex=min(ex,Event.xexpose.x);
-	ey=min(ey,Event.xexpose.y);
-	ex2=max(ex2,Event.xexpose.x+Event.xexpose.width);
-	ey2=max(ey2,Event.xexpose.y+Event.xexpose.height);
+      {
+	      int ex,ey,ex2,ey2;
 
-	if(Event.xexpose.count==0)
-	{
-	  button=-1;ub=UberButton;
-	  while(NextButton(&ub,&b,&button,1))
-	  {
-	    Bool r = False;
+	      if(Event.xexpose.window != MyWindow)
+	      {
+		      break;
+	      }
+	      ex = Event.xexpose.x;
+	      ey = Event.xexpose.y;
+	      ex2= Event.xexpose.x+Event.xexpose.width;
+	      ey2= Event.xexpose.y+Event.xexpose.height;
+	      while (FCheckTypedWindowEvent(Dpy, MyWindow, Expose, &Event))
+	      {
+		      /* maybe we should not purge here, this may interfere
+		       * with configure notify */
+		      ex = min(ex, Event.xexpose.x);
+		      ey = min(ey, Event.xexpose.y);
+		      ex2 = max(ex2, Event.xexpose.x + Event.xexpose.width);
+		      ey2= max(ey2, Event.xexpose.y + Event.xexpose.height);
+	      }
+	      Event.xexpose.x = ex;
+	      Event.xexpose.y = ey;
+	      Event.xexpose.width = ex2-ex;
+	      Event.xexpose.height = ey2-ey;
+	      button=-1;
+	      ub=UberButton;
+	      while(NextButton(&ub,&b,&button,1))
+	      {
+		      Bool r = False;
 
-	    tmpb = b;
-	    while(tmpb->parent != NULL)
-	    {
-	      if(tmpb->flags&b_Container)
-	      {
-		x=buttonXPos(tmpb,buttonNum(tmpb));
-		y=buttonYPos(tmpb,buttonNum(tmpb));
+		      tmpb = b;
+		      while(tmpb->parent != NULL)
+		      {
+			      if(tmpb->flags&b_Container)
+			      {
+				      x=buttonXPos(tmpb,buttonNum(tmpb));
+				      y=buttonYPos(tmpb,buttonNum(tmpb));
+			      }
+			      else
+			      {
+				      x=buttonXPos(tmpb,button);
+				      y=buttonYPos(tmpb,button);
+			      }
+			      if (!(ex > x + buttonWidth(tmpb) || ex2 < x ||
+				    ey > y + buttonHeight(tmpb) || ey2 < y))
+			      {
+				      r = True;
+			      }
+			      tmpb = tmpb->parent;
+		      }
+		      if(r)
+		      {
+			      if(ready<1 && !(b->flags&b_Container))
+				      MakeButton(b);
+			      RedrawButton(b, DRAW_ALL, &Event);
+		      }
 	      }
-	      else
-	      {
-		x=buttonXPos(tmpb,button);
-		y=buttonYPos(tmpb,button);
-	      }
-	      if (!(ex > x + buttonWidth(tmpb) || ex2 < x ||
-		 ey > y + buttonHeight(tmpb) || ey2 < y))
-	      {
-		r = True;
-	      }
-	      tmpb = tmpb->parent;
-	    }
-	    if(r)
-	    {
-	      if(ready<1 && !(b->flags&b_Container))
-		MakeButton(b);
-	      RedrawButton(b,1);
-	    }
-	  }
-	  if(ready<1)
-	    ready++;
-
-	  ex=ey=10000;ex2=ey2=0;
-	}
-#endif
-	break;
+	      if(ready<1)
+		      ready++;
+      }
+      break;
 
       case ConfigureNotify:
       {
@@ -1020,7 +1026,7 @@ void Loop(void)
 	unsigned int depth,tw,th,border_width;
 	Window root;
 
-	while (XCheckTypedWindowEvent(Dpy, MyWindow, ConfigureNotify, &event))
+	while (FCheckTypedWindowEvent(Dpy, MyWindow, ConfigureNotify, &event))
 	{
 	  if (!event.xconfigure.send_event &&
 	      Event.xconfigure.window != MyWindow)
@@ -1045,17 +1051,19 @@ void Loop(void)
 	  {
 	    SetTransparentBackground(UberButton, Width, Height);
 	  }
-	  for (i = 0; i < nColorsets; i++)
-	    change_colorset(i, &Event);
+	  /* what follow can be optimized */
 	  while(NextButton(&ub,&b,&button,0))
 	    MakeButton(b);
-	  XClearWindow(Dpy, MyWindow);
-	  RedrawWindow(NULL);
+	  for (i = 0; i < nColorsets; i++)
+		  change_colorset(i, &Event);
+	  if (!(UberButton->c->flags & b_Colorset))
+	  {
+		  XClearWindow(Dpy, MyWindow);
+		  RedrawWindow();
+	  }
 	}
 	else if (Event.xconfigure.window == MyWindow &&
-		 Event.xconfigure.send_event &&
-		 (UberButton->c->flags & b_Colorset) &&
-		 Colorset[UberButton->c->colorset].pixmap == ParentRelative)
+		 Event.xconfigure.send_event)
 	{
 	  update_root_transparency(&Event);
 	}
@@ -1071,7 +1079,7 @@ void Loop(void)
 	     select_button(UberButton, Event.xmotion.x, Event.xmotion.y));
 	  if (CurrentButton && is_pointer_in_current_button != f)
 	  {
-	    RedrawButton(b, 0);
+	    RedrawButton(b, DRAW_RELIEF, NULL);
 	  }
 	}
 	break;
@@ -1086,7 +1094,7 @@ void Loop(void)
 	{
 	  b = CurrentButton;
 	  CurrentButton = 0;
-	  RedrawButton(b, 0);
+	  RedrawButton(b, DRAW_RELIEF, NULL);
 	  break;
 	}
 	if (Event.xbutton.state & DEFAULT_ALL_BUTTONS_MASK)
@@ -1119,7 +1127,7 @@ void Loop(void)
 	  CurrentButton=NULL;
 	  break;
 	}
-	RedrawButton(b,0);
+	RedrawButton(b, DRAW_RELIEF, NULL);
 	if (!act)
 	{
 	  break;
@@ -1270,7 +1278,7 @@ void Loop(void)
 	b = CurrentButton;
 	CurrentButton=NULL;
 	if (b)
-	  RedrawButton(b, 0);
+	  RedrawButton(b, DRAW_RELIEF, NULL);
 	break;
 
       case ClientMessage:
@@ -1324,7 +1332,7 @@ void Loop(void)
 		}
 	      }
 	    }
-	    RedrawButton(b,1);
+	    RedrawButton(b, DRAW_FORCE, NULL);
 	  }
 	}
 	break;
@@ -1339,7 +1347,7 @@ void Loop(void)
 	  {
 	    /* A panel has been unmapped, update the button */
 	    b->newflags.panel_mapped = (Event.type == MapNotify);
-	    RedrawButton(b, 1);
+	    RedrawButton(b, DRAW_FORCE, NULL);
 	    break;
 	  }
 	}
@@ -1391,7 +1399,7 @@ void Loop(void)
 		SendText(fd, p, 0);
 		free(p);
 	      }
-	      RedrawButton(b,1);
+	      RedrawButton(b, DRAW_CLEAN, NULL); /* ? */
 	      if (is_transient_panel)
 	      {
 		XWithdrawWindow(Dpy, MyWindow, screen);
@@ -1400,7 +1408,7 @@ void Loop(void)
 	    else
 	    {
 	      b->flags &= ~(b_Swallow | b_Panel);
-	      RedrawButton(b,2);
+	      RedrawButton(b, DRAW_FORCE, NULL);
 #ifdef DEBUG_HANGON
 	      fprintf(stderr,"\n");
 #endif
@@ -1415,21 +1423,21 @@ void Loop(void)
 	fprintf(stderr,"%s: Event fell through unhandled\n",MyName);
 #endif
 	break;
-      }
-    }
-  }
+      } /* switch */
+    } /* event handling */
+  } /* while */
+
 }
 
 /**
 *** RedrawWindow()
-*** Draws the window by traversing the button tree, draws all if NULL is given,
-*** otherwise only the given button.
+*** Draws the window by traversing the button tree
 **/
-void RedrawWindow(button_info *b)
+void RedrawWindow(void)
 {
   int button;
   XEvent dummy;
-  button_info *ub;
+  button_info *ub,*b;
   static Bool initial_redraw = True;
   Bool clear_buttons;
 
@@ -1437,15 +1445,9 @@ void RedrawWindow(button_info *b)
     return;
 
   /* Flush expose events */
-  while (XCheckTypedWindowEvent (Dpy, MyWindow, Expose, &dummy))
+  while (FCheckTypedWindowEvent(Dpy, MyWindow, Expose, &dummy))
     ;
 
-  if(b)
-  {
-    RedrawButton(b,0);
-    return;
-  }
-
   /* Clean out the entire window first */
   if (initial_redraw == False)
   {
@@ -1461,13 +1463,13 @@ void RedrawWindow(button_info *b)
   button=-1;
   ub=UberButton;
   while(NextButton(&ub,&b,&button,1))
-    RedrawButton(b,1);
+    RedrawButton(b, DRAW_ALL, NULL);
 }
 
 /**
 *** LoadIconFile()
 **/
-int LoadIconFile(char *s, FvwmPicture **p, int cset)
+int LoadIconFile(const char *s, FvwmPicture **p, int cset)
 {
 	FvwmPictureAttributes fpa;
 
@@ -1477,8 +1479,12 @@ int LoadIconFile(char *s, FvwmPicture **p, int cset)
 
 		fpa.mask |= FPAM_DITHER;
 	}
-	*p = PCacheFvwmPicture(Dpy, Root, imagePath, s, fpa);
-	if(*p)
+	if (UberButton->c->flags&b_TransBack)
+	{
+		fpa.mask |= FPAM_NO_ALPHA;
+	}
+	*p = PCacheFvwmPicture(Dpy, MyWindow, imagePath, s, fpa);
+	if (*p)
 	{
 		return 1;
 	}
@@ -1501,7 +1507,7 @@ void RecursiveLoadData(button_info *b,int *maxx,int *maxy)
   fprintf(stderr,"%s: Loading: Button 0x%06x: colors",MyName,(ushort)b);
 #endif
 
-  
+
   /* initialise button colours and background */
   if (b->flags & b_Colorset)
   {
@@ -1659,7 +1665,7 @@ void RecursiveLoadData(button_info *b,int *maxx,int *maxy)
   j=0;
 
   /* Load the icon */
-  if(b->flags&b_Icon && LoadIconFile(b->icon_file,&b->icon, b->colorset))
+  if(b->flags&b_Icon && LoadIconFile(b->icon_file,&b->icon, buttonColorset(b)))
   {
 #ifdef DEBUG_LOADDATA
     fprintf(stderr,", icon \"%s\"",b->icon_file);
@@ -1778,7 +1784,7 @@ static void HandlePanelPress(button_info *b)
 	/* the window has been destroyed */
 	XUnmapWindow(Dpy, b->PanelWin);
 	XSync(Dpy, 0);
-	RedrawButton(b, 1);
+	RedrawButton(b, DRAW_CLEAN, NULL);
 	return;
       }
       if (xwa.map_state == IsViewable)
@@ -1795,7 +1801,7 @@ static void HandlePanelPress(button_info *b)
       /* give up after one second */
       XUnmapWindow(Dpy, b->PanelWin);
       XSync(Dpy, 0);
-      RedrawButton(b, 1);
+      RedrawButton(b, DRAW_FORCE, NULL);
       return;
     }
   }
@@ -1867,7 +1873,7 @@ static void HandlePanelPress(button_info *b)
   }
   tmp = CurrentButton;
   CurrentButton = NULL;
-  RedrawButton(b, 1);
+  RedrawButton(b, DRAW_FORCE, NULL);
   CurrentButton = tmp;
   XSync(Dpy, 0);
   /* Give fvwm a chance to update the window.  Otherwise the window may end up
@@ -1889,7 +1895,6 @@ void CreateUberButtonWindow(button_info *ub,int maxx,int maxy)
   XGCValues gcv;
   unsigned long gcm;
   XClassHint myclasshints;
-  XSetWindowAttributes xswa;
 
   x = UberButton->x; /* Geometry x where to put the panel */
   y = UberButton->y; /* Geometry y where to put the panel */
@@ -1967,13 +1972,11 @@ void CreateUberButtonWindow(button_info *ub,int maxx,int maxy)
 	    (ushort)fore_pix,(ushort)back_pix);
 #endif
 
-  xswa.colormap = Pcmap;
-  xswa.border_pixel = 0;
-  xswa.background_pixmap = None;
-  MyWindow = XCreateWindow(Dpy,Root,mysizehints.x,mysizehints.y,
-			   mysizehints.width,mysizehints.height,0,Pdepth,
-			   InputOutput,Pvisual,
-			   CWColormap|CWBackPixmap|CWBorderPixel,&xswa);
+  XMoveResizeWindow(Dpy, MyWindow, mysizehints.x, mysizehints.y,
+	      mysizehints.width, mysizehints.height);
+
+  XSetWMNormalHints(Dpy,MyWindow,&mysizehints);
+
   if (is_transient)
   {
     XSetTransientForHint(Dpy, MyWindow, Root);
@@ -2106,17 +2109,17 @@ void DebugEvents(XEvent *event)
 #endif
 
 /**
-*** My_XNextEvent()
+*** My_FNextEvent()
 *** Waits for next X event, or for an auto-raise timeout.
 **/
-int My_XNextEvent(Display *Dpy, XEvent *event)
+int My_FNextEvent(Display *Dpy, XEvent *event)
 {
   fd_set in_fdset;
   static int miss_counter = 0;
 
-  if(XPending(Dpy))
+  if(FPending(Dpy))
   {
-    XNextEvent(Dpy,event);
+    FNextEvent(Dpy,event);
 #ifdef DEBUG_EVENTS
     DebugEvents(event);
 #endif
@@ -2132,9 +2135,9 @@ int My_XNextEvent(Display *Dpy, XEvent *event)
 
     if(FD_ISSET(x_fd, &in_fdset))
     {
-      if(XPending(Dpy))
+      if(FPending(Dpy))
       {
-	XNextEvent(Dpy,event);
+	FNextEvent(Dpy,event);
 	miss_counter = 0;
 #ifdef DEBUG_EVENTS
 	DebugEvents(event);
@@ -2232,7 +2235,7 @@ static void send_bg_change_to_module(button_info *b, XEvent *Event)
     Event->xconfigure.width = b->icon_w;
     Event->xconfigure.height = b->icon_h;
     Event->xconfigure.window = SwallowedWindow(b);
-    XSendEvent(Dpy, SwallowedWindow(b), False, NoEventMask, Event);
+    FSendEvent(Dpy, SwallowedWindow(b), False, NoEventMask, Event);
   }
   else
     {
@@ -2245,125 +2248,205 @@ static void send_bg_change_to_module(button_info *b, XEvent *Event)
 
 static void update_root_transparency(XEvent *Event)
 {
-  button_info *ub,*b;
-  int button;
+	button_info *ub,*b;
+	int button;
+	Bool bg_is_transparent = CSET_IS_TRANSPARENT(UberButton->c->colorset);
 
-  if (!(UberButton->c->flags & b_Colorset) ||
-      Colorset[UberButton->c->colorset].pixmap != ParentRelative)
-  {
-    return;
-  }
+	if (bg_is_transparent)
+	{
+		SetWindowBackground(
+			Dpy, MyWindow, UberButton->c->width,
+			UberButton->c->height,
+			&Colorset[UberButton->c->colorset],
+			Pdepth, NormalGC, True);
+		/* the window is redraw by exposure. It is very difficult
+		 * the clear only the needed part and then redraw the
+		 * cleared buttons (pbs with containers which contain containers
+		 * ...) */
+	}
 
-  XClearWindow(Dpy, MyWindow);
-  RedrawWindow(NULL);
-  /* Handle the swallowed window */
-  ub = UberButton;
-  button=-1;
-  while(NextButton(&ub,&b,&button,0))
-  {
-    if (buttonSwallowCount(b) == 3 && (b->flags & b_Swallow))
-    {
-      /* should handle 2 cases: module or not */
-      if (b->swallow & b_FvwmModule)
-      {
-	send_bg_change_to_module(b, Event);
-      }
-      else if ((b->flags & b_Colorset) &&
-	       Colorset[buttonColorset(b)].pixmap == ParentRelative)
-      {
-	/* it is an X application */
-	change_swallowed_window_colorset(b, True);
-      }
-    }
-  }
+	if (!bg_is_transparent)
+	{
+		/* update root transparent button (and its childs) */
+		ub = UberButton;
+		button=-1;
+		while(NextButton(&ub,&b,&button,1))
+		{
+			button_info *tmpb;
+			Bool redraw = False;
+
+			tmpb = b;
+			/* see if _a_ parent (!= UberButton) is root trans */
+			while(tmpb->parent != NULL && !redraw)
+			{
+				if (CSET_IS_TRANSPARENT_ROOT(
+					buttonColorset(tmpb)))
+				{
+					redraw = True;
+				}
+				tmpb = tmpb->parent;
+			}
+			if (redraw)
+			{
+				RedrawButton(b,DRAW_ALL,NULL);
+			}
+		}
+	}
+
+
+	/* Handle the swallowed window */
+	ub = UberButton;
+	button=-1;
+	while(NextButton(&ub,&b,&button,0))
+	{
+		if (buttonSwallowCount(b) == 3 && (b->flags & b_Swallow))
+		{
+			/* should handle 2 cases: module or not */
+			if (b->swallow & b_FvwmModule)
+			{
+				if (bg_is_transparent)
+				{
+					XSync(Dpy, 0);
+					send_bg_change_to_module(b, Event);
+				}
+			}
+			else if ((b->flags & b_Colorset) &&
+				 CSET_IS_TRANSPARENT(b->colorset))
+			{
+				change_swallowed_window_colorset(b, True);
+			}
+		}
+	}
 }
 
-static void recursive_change_colorset(container_info *c, int colorset,
-				      XEvent *Event)
+
+static void recursive_change_colorset(
+	container_info *c, int colorset, XEvent *Event)
 {
-  int i;
-  Bool bg_change = False;
+	int button;
+	button_info *ub,*b;
+	button_info *tmpb;
 
-  for (i = c->num_buttons; --i >= 0; )
-  {
-    button_info *b = c->buttons[i];
-
-    if (!b)
-      continue;
-
-    bg_change = False;
-    if (buttonColorset(b) == colorset)
-    {
-      bg_change = True;
-    }
-    else  if (buttonColorset(b) > -1 &&
-	      Colorset[buttonColorset(b)].pixmap == ParentRelative &&
-	      (UberButton->c->flags & b_Colorset &&
-	       colorset == UberButton->c->colorset))
-    {
-      bg_change = True;
-    }
-    else  if ((b->flags & b_Swallow) && (b->swallow & b_FvwmModule) &&
-	      (UberButton->c->flags & b_Colorset &&
-	       colorset == UberButton->c->colorset))
-    {
-      /* swallowed module */
-      bg_change = True;
-    }
-
-    if (b->flags & b_Container)
-    {
-      if (bg_change)
-      {
-	/* re-apply colorset to button */
-	RedrawButton(b, True);
-      }
-      /* recursively update containers */
-      recursive_change_colorset(b->c, colorset, Event);
-    }
-    else if (b->flags & b_Swallow)
-    {
-      /* swallowed window */
-      if (buttonSwallowCount(b) == 3 && b->IconWin != None && bg_change)
-      {
-	RedrawButton(b, True);
-	if (Event == NULL && (b->swallow & b_FvwmModule))
+	button=-1;
+	ub=UberButton;
+	while(NextButton(&ub,&b,&button,1))
 	{
-	  XSync(Dpy, 0);
-	  send_bg_change_to_module(b, Event);
-	}
-	else if (b->flags & b_Colorset)
-	{
-	  change_swallowed_window_colorset(b, True);
-	}
-      }
-    }
-    else
-    {
-      /* simple button */
-      if (bg_change)
-      {
-	/* re-apply colorset to button */
-	RedrawButton(b, True);
-      }
-    }
-  }
+		Bool redraw = False;
 
-  return;
+		tmpb = b;
+		/* see if _a_ parent (!= UberButton) has colorset as cset */
+		while(tmpb->parent != NULL && !redraw)
+		{
+			if (buttonColorset(tmpb) == colorset)
+			{
+				redraw = True;
+			}
+			tmpb = tmpb->parent;
+		}
+		if(!redraw)
+		{
+			continue;
+		}
+		if (b->flags & b_Swallow)
+		{
+			/* swallowed window */
+			if (buttonSwallowCount(b) == 3 && b->IconWin != None)
+			{
+				RedrawButton(b, DRAW_ALL, NULL);
+				if (b->flags&b_Colorset &&
+				    !(b->swallow&b_FvwmModule))
+				{
+					change_swallowed_window_colorset(
+						b, True);
+				}
+			}
+		}
+		else
+		{
+			if (Event == NULL && b->flags&b_Icon)
+			{
+				/* FIXME do that only if we have an icon
+				 * colorset */
+				DestroyIconWindow(b);
+				CreateIconWindow(b);
+				if (b->flags&b_Icon)
+				{
+					if (!(b->flags&b_IconAlpha))
+					{
+						ConfigureIconWindow(b, NULL);
+						XMapWindow(Dpy,b->IconWin);
+					}
+				}
+			}
+			RedrawButton(b, DRAW_ALL, NULL);
+		}
+	}
+
+	return;
 }
 
 static void change_colorset(int colorset, XEvent *Event)
 {
-  if (UberButton->c->flags & b_Colorset && colorset == UberButton->c->colorset)
-  {
-    SetWindowBackground(
-      Dpy, MyWindow, UberButton->c->width, UberButton->c->height,
-      &Colorset[colorset], Pdepth, NormalGC, True);
-  }
+	if (UberButton->c->flags & b_Colorset &&
+	    colorset == UberButton->c->colorset)
+	{
+		button_info *ub,*b;
+		int button;
 
-  recursive_change_colorset(UberButton->c, colorset, Event);
+		SetWindowBackground(
+			Dpy, MyWindow, UberButton->c->width,
+			UberButton->c->height,
+			&Colorset[colorset], Pdepth, NormalGC, True);
 
-  return;
+		/* some change are done by exposing. Do the other one */
+		button=-1;
+		ub=UberButton;
+		while(NextButton(&ub,&b,&button,0))
+		{
+			if (b->flags&b_Swallow && buttonSwallowCount(b) == 3)
+			{
+				if (b->swallow&b_FvwmModule)
+				{
+					/* the bg has changed send the info to
+					 * modules */
+					XSync(Dpy, 0);
+					send_bg_change_to_module(b, Event);
+				}
+				else if (CSET_IS_TRANSPARENT_PR(b->colorset) &&
+					!buttonBackgroundButton(b->parent,NULL))
+				{
+					/* the swallowed app has a ParentRel bg
+					 * and the bg of the button is the bg */
+					change_swallowed_window_colorset(
+						b, True);
+				}
+			}
+			else if (Event == NULL && b->flags&b_Icon &&
+				 buttonColorset(b) == colorset)
+			{
+				/* FIXME do that only if we have an icon
+				 * colorset */
+				DestroyIconWindow(b);
+				CreateIconWindow(b);
+				if (b->flags&b_Icon)
+				{
+					if (!(b->flags&b_IconAlpha))
+					{
+						ConfigureIconWindow(b, NULL);
+						XMapWindow(Dpy,b->IconWin);
+					}
+				}
+			}
+		}
+
+		return;
+	}
+
+	recursive_change_colorset(
+		UberButton->c, colorset, Event);
+
+
+	return;
 }
 
 static void handle_config_info_packet(unsigned long *body)
@@ -2398,7 +2481,16 @@ void process_message(unsigned long type,unsigned long *body)
   {
   case M_NEW_DESK:
     new_desk = body[0];
-    RedrawWindow(NULL);
+    {
+	    int button=-1;
+	    button_info *b, *ub;
+
+	    ub=UberButton;
+	    while(NextButton(&ub,&b,&button,1))
+	    {
+		    RedrawButton(b, DRAW_RELIEF, NULL);
+	    }
+    }
     break;
   case M_END_WINDOWLIST:
     SpawnSome();
@@ -2442,7 +2534,10 @@ void process_message(unsigned long type,unsigned long *body)
     if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
 	((!swallowed && body[2] == 0) || (swallowed && body[2] == MyWindow)))
     {
-      update_root_transparency(NULL);
+	    if (CSET_IS_TRANSPARENT_PR(UberButton->c->colorset))
+	    {
+		update_root_transparency(NULL);
+	    }
     }
     else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == MyWindow)
     {
@@ -2514,7 +2609,7 @@ void CheckForHangon(unsigned long *body)
 	b->flags&=~b_Hangon;
 	free(b->hangon);
 	b->hangon=NULL;
-	RedrawButton(b,0);
+	RedrawButton(b, DRAW_RELIEF, NULL);
       }
       break;
     }
@@ -2898,7 +2993,7 @@ void swallow(unsigned long *body)
 	}
 	/* Redraw and force cleaning the background to erase the old button
 	 * title. */
-	RedrawButton(b, 2);
+	RedrawButton(b, DRAW_FORCE, NULL);
       }
       else /* (b->flags & b_Panel) */
       {
@@ -2915,137 +3010,3 @@ void swallow(unsigned long *body)
     }
   }
 }
-
-
-/* SendToModule options */
-static void change_title(button_info *b, char *line)
-{
-  if (!(b->flags & b_Title)) {
-    fprintf(stderr, "%s: cannot create a title, only change one\n", MyName);
-    return;
-  }
-  free(b->title);
-  CopyString(&b->title, line);
-  RedrawButton(b, 2);
-  return;
-}
-
-static void change_icon(button_info *b, char *line)
-{
-  FvwmPicture *new_icon;
-  int l;
-
-  if (line == NULL) {
-    return;
-  }
-  if (!(b->flags & b_Icon)) {
-    fprintf(stderr, "%s: cannot create an icon, only change one\n", MyName);
-    return;
-  }
-  l = strlen(line);
-  if (l > 0 && line[l - 1] == '\n') {
-    line[l - 1] = 0;
-  }
-  if (LoadIconFile(line, &new_icon, b->colorset) == 0) {
-    fprintf(stderr, "%s: cannot load icon %s\n", MyName, line);
-    return;
-  }
-  free(b->icon_file);
-  CopyString(&b->icon_file, line);
-  PDestroyFvwmPicture(Dpy, b->icon);
-  XDestroyWindow(Dpy, b->IconWin);
-  b->IconWin = None;
-  b->icon = new_icon;
-  CreateIconWindow(b);
-  ConfigureIconWindow(b);
-  XMapWindow(Dpy, b->IconWin);
-  RedrawButton(b, 2);
-  return;
-}
-
-static char *message_options[] = {"Title", "Icon", NULL};
-
-void parse_message_line(char *line)
-{
-  char *rest;
-  char *next;
-  char *s;
-  int type;
-  int count, i;
-  int n;
-  int mask;
-  int x;
-  int y;
-  button_info *b, *ub = UberButton;
-
-  type = GetTokenIndex(line, message_options, -1, &rest);
-  if (type == -1) {
-    fprintf(stderr, "%s: Message not understood: %s\n", MyName, line);
-    return;
-  }
-
-  /* find out which button */
-  s = PeekToken(rest, &next);
-  if (s != NULL && *s == '+')
-  {
-    unsigned int JunkWidth;
-    unsigned int JunkHeight;
-
-    mask = XParseGeometry(s, &x, &y, &JunkWidth, &JunkHeight);
-    if (!(mask & XValue) || (mask & XNegative) ||
-	!(mask & YValue) || (mask & YNegative))
-    {
-      fprintf(stderr, "%s: illegal button position '%s'\n", MyName, s);
-      return;
-    }
-    rest = next;
-    if (x < 0 || y < 0) {
-      fprintf(stderr, "%s: button column/row must not be negative\n", MyName);
-      return;
-    }
-    b = get_xy_button(ub, y, x);
-    if (b == NULL) {
-      fprintf(stderr, "%s: button at column %d row %d not found\n", MyName,
-	      x, y);
-      return;
-    }
-  }
-  else
-  {
-    n = GetIntegerArguments(rest, &rest, &x, 1);
-    if (n < 1) {
-      fprintf(stderr, "%s: missing button number\n", MyName);
-      return;
-    }
-    if (x < 0) {
-      fprintf(stderr, "%s: button number must not be negative\n", MyName);
-      return;
-    }
-    i = count = -1;
-    /* find the button */
-    while (NextButton(&ub, &b, &i, 0)) {
-      if (++count == x)
-	break;
-    }
-    if (count != x || b == NULL) {
-      fprintf(stderr, "%s: button number %d not found\n", MyName, x);
-      return;
-    }
-  }
-
-  switch(type) {
-  case 0:
-    change_title(b, rest);
-    break;
-  case 1:
-    change_icon(b, rest);
-    break;
-  }
-  if (FShapesSupported)
-  {
-    if (UberButton->c->flags&b_TransBack)
-      SetTransparentBackground(UberButton,Width,Height);
-  }
-
-  return;
-}
