@@ -149,7 +149,7 @@ main_loop (void)
     }
     else {
 
-      if (FD_ISSET(x_fd, &readset) || XPending(theDisplay)) {
+      if (FD_ISSET(x_fd, &readset) || FPending(theDisplay)) {
 	xevent_loop();
       }
       if (FD_ISSET(Fvwm_fd[1], &readset)) {
@@ -290,11 +290,10 @@ main(int argc, char **argv)
 
   SetMessageMask(Fvwm_fd, M_CONFIGURE_WINDOW | M_RES_CLASS | M_RES_NAME |
 		 M_ADD_WINDOW | M_DESTROY_WINDOW | M_ICON_NAME |
-		 M_DEICONIFY | M_ICONIFY | M_END_WINDOWLIST |
+		 M_DEICONIFY | M_ICONIFY | M_ICON_LOCATION | M_END_WINDOWLIST |
 		 M_NEW_DESK | M_NEW_PAGE | M_FOCUS_CHANGE | M_WINDOW_NAME |
 		 M_CONFIG_INFO | M_SENDCONFIG | M_VISIBLE_NAME |
-		 M_MINI_ICON |
-		 M_STRING);
+		 M_MINI_ICON | M_STRING | M_WINDOWSHADE | M_DEWINDOWSHADE);
   /* extended messages */
   SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME | MX_PROPERTY_CHANGE);
 
