@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "config.h"
@@ -18,7 +18,7 @@
 #include <assert.h>
 #include <stdio.h>
 
-#include <sys/time.h>
+#include "libs/ftime.h"
 
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
@@ -80,9 +80,9 @@ typedef enum {
   SHOW_DESKTOP,
   SHOW_PAGE,
   SHOW_SCREEN,
-  NO_SHOW_DESKTOP,	/* "!desk" Show windows not on the current desk */
-  NO_SHOW_PAGE,		/* "!page" Show windows not on the current page */
-  NO_SHOW_SCREEN	/* "!screen" Show windows not on the current screen */
+  NO_SHOW_DESKTOP,      /* "!desk" Show windows not on the current desk */
+  NO_SHOW_PAGE,         /* "!page" Show windows not on the current page */
+  NO_SHOW_SCREEN        /* "!screen" Show windows not on the current screen */
 } Resolution;
 
 typedef enum {
@@ -108,24 +108,25 @@ typedef enum {
   NUM_ACTIONS
 } Action;
 
+
 typedef enum {
-  PLAIN_CONTEXT = 0,
+  DEFAULT = 0,
   FOCUS_CONTEXT = 1,
   SELECT_CONTEXT = 2,
   FOCUS_SELECT_CONTEXT = 3, /* had better be FOCUS_CONTEXT | SELECT_CONTEXT */
-  TITLE_CONTEXT = 4,
-  ICON_CONTEXT = 5,
-  DEFAULT = 6,
+  PLAIN_CONTEXT = 4,
+  TITLE_CONTEXT = 5,
+  ICON_CONTEXT = 6,
   NUM_CONTEXTS
 } Contexts;
 
 typedef enum {
-  NO_NAME	= 0,
-  TITLE_NAME	= 1,
-  ICON_NAME	= 2,
+  NO_NAME       = 0,
+  TITLE_NAME    = 1,
+  ICON_NAME     = 2,
   RESOURCE_NAME = 4,
-  CLASS_NAME	= 8,
-  ALL_NAME	= 15
+  CLASS_NAME    = 8,
+  ALL_NAME      = 15
 } NameType;
 
 typedef struct win_list {
@@ -198,6 +199,7 @@ typedef struct win_data {
 
   Ulong desknum;
   long x, y, width, height;
+  rectangle icon_g;
   Ulong app_id;
 /*  Ulong fvwm_flags; */
   window_flags flags;
@@ -218,15 +220,16 @@ typedef struct win_data {
 } WinData;
 
 typedef struct button {
-  int index;	  /* index into button array */
+  int index;      /* index into button array */
   int x, y, w, h; /* current coords of button */
   struct {
-    int dirty_flags;
-    FvwmPicture pic;
-    WinData *win;
-    char *display_string;
-    int x, y, w, h;
-    Uchar iconified, state;
+	  int dirty_flags;
+	  FvwmPicture pic;
+	  WinData *win;
+	  char *display_string;
+	  int x, y, w, h;
+	  Uchar iconified, state;
+	  int ex, ey, ew, eh; /* expose damage relatively the main win */
   } drawn_state;
 } Button;
 
@@ -263,11 +266,11 @@ typedef struct {
 } ShapeState;
 
 typedef enum {
-  SortNone,	     /* no sorting */
-  SortId,	     /* sort by window id */
-  SortName,	     /* case insensitive name sorting */
-  SortNameCase,	     /* case sensitive name sorting */
-  SortWeighted	     /* custom sort order */
+  SortNone,          /* no sorting */
+  SortId,            /* sort by window id */
+  SortName,          /* case insensitive name sorting */
+  SortNameCase,      /* case sensitive name sorting */
+  SortWeighted       /* custom sort order */
 } SortType;
 
 typedef struct {
@@ -313,7 +316,8 @@ typedef struct win_manager {
   int weighted_sorts_len, weighted_sorts_size;
   char *AnimCommand;
   Uchar showonlyiconic;
-  rectangle managed_g;	  /* dimensions of managed screen portion */
+  Uchar showtransient;
+  rectangle managed_g;    /* dimensions of managed screen portion */
 
   /* X11 state */
   Window theWindow, theFrame;
@@ -337,14 +341,19 @@ typedef struct win_manager {
   /* Fvwm state */
   int we_are_drawing, configures_expected;
   Bool swallowed;
+  struct
+  {
+    unsigned is_shaded : 1;
+    unsigned needs_resize_after_unshade : 1;
+  } flags;
 } WinManager;
 
 #define MANAGER_EMPTY(man) ((man)->buttons.num_windows == 0)
 
 typedef struct {
   Ulong desknum;
-  Ulong x, y;		  /* of the view window */
-  rectangle screen_g;	  /* screen dimensions */
+  Ulong x, y;             /* of the view window */
+  rectangle screen_g;     /* screen dimensions */
   WinManager *managers;
   int num_managers;
   int transient;
