@@ -11,12 +11,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "types.h"
@@ -29,16 +29,16 @@
 #include "libs/FRender.h"
 #include "libs/FRenderInit.h"
 
-#ifdef MEMDEBUG			/* For debugging */
+#ifdef MEMDEBUG                 /* For debugging */
 #include <unchecked.h>
 #endif
 
 /* Variables utilise par l'analyseur syntaxique */
 extern ScriptProp *scriptprop;
-extern int nbobj;			/* Nombre d'objets */
-extern int numligne;			/* Numero de ligne */
-extern TabObj *tabobj;			/* Tableau d'objets, limite=1000 */
-extern char **TabVVar;			/* Tableau des variables du sript */
+extern int nbobj;                       /* Nombre d'objets */
+extern int numligne;                    /* Numero de ligne */
+extern TabObj *tabobj;                  /* Tableau d'objets, limite=1000 */
+extern char **TabVVar;                  /* Tableau des variables du sript */
 extern int TabIdObj[1001];
 extern Bloc **TabIObj;
 extern CaseObj *TabCObj;
@@ -54,13 +54,13 @@ extern int __bounds_debug_no_checking;
 
 
 /* Variables globales */
-char *ScriptName;	/* Nom du fichier contenat le script decrivant le GUI */
+char *ScriptName;       /* Nom du fichier contenat le script decrivant le GUI */
 char *ScriptBaseName;
 char *ScriptPath = "";
 char *ModuleName;
-int fd[2];			/* pipe pair */
+int fd[2];                      /* pipe pair */
 int fd_err;
-int x_fd;			/* fd for X */
+int x_fd;                       /* fd for X */
 Window ref;
 FlocaleWinString *FwinString;
 
@@ -69,16 +69,16 @@ extern void (*TabCom[25]) (int NbArg,long *TabArg);
 
 Display *dpy;
 int screen;
-Window	Root;
-X11base *x11base;		/* Pour le serveur X */
-TypeBuffSend BuffSend;		/* Pour les communication entre script */
+Window  Root;
+X11base *x11base;               /* Pour le serveur X */
+TypeBuffSend BuffSend;          /* Pour les communication entre script */
 int grab_server = 0;
 struct XObj *tabxobj[1000];
 char *Scrapt;
 Atom propriete,type;
 static Atom wm_del_win;
 char *imagePath = NULL;
-int save_color_limit = 0;		    /* color limit from config */
+int save_color_limit = 0;                   /* color limit from config */
 static Bool is_dead_pipe = False;
 KeySym shift_tab_ks;  /* shift-Tab keysym */
 char *LastString = NULL;  /* last string send by a SendString ot Key cmd */
@@ -100,7 +100,7 @@ ShutdownX(void)
   int NbEssai=0;
   struct timeval tv;
 
-#ifdef DEBUG			/* For debugging */
+#ifdef DEBUG                    /* For debugging */
   XSync(dpy,0);
 #endif
 
@@ -130,7 +130,7 @@ ShutdownX(void)
     {
       if (FD_ISSET(x_fd, &in_fdset))
       {
-	if (XCheckTypedEvent(dpy, SelectionRequest, &event))
+	if (FCheckTypedEvent(dpy, SelectionRequest, &event))
 	  SendMsgToScript(event);
 	else
 	  NbEssai++;
@@ -140,7 +140,7 @@ ShutdownX(void)
   XFlush(dpy);
 
   /* Attente de deux secondes afin d'etre sur que tous */
-  /* les messages soient arrives a destination	       */
+  /* les messages soient arrives a destination         */
   /* On quitte proprement le serveur X */
   for (i=0; i<nbobj; i++)
     tabxobj[i]->DestroyObj(tabxobj[i]);
@@ -177,8 +177,8 @@ void ReadConfig (char *ScriptName)
      Not very pretty, dje 12/26/99 */
   sprintf(s,"%s%s%s",ScriptPath,(!*ScriptPath ? "" : "/"),ScriptName);
   yyin = fopen(s,"r");
-  if (yyin == NULL) {			/* file not found yet, */
-    TryToFind(ScriptName);		     /* look in some other places */
+  if (yyin == NULL) {                   /* file not found yet, */
+    TryToFind(ScriptName);                   /* look in some other places */
   }
   if (yyin == NULL)
   {
@@ -206,7 +206,7 @@ static void TryToFind(char *filename) {
   extern FILE *yyin;
   char path[FILENAME_MAX];
 
-  if (filename[0] == '/') {		/* if absolute path */
+  if (filename[0] == '/') {             /* if absolute path */
     yyin = fopen(filename,"r");
     return;
   }
@@ -357,10 +357,13 @@ void LoadIcon(struct XObj *xobj)
 {
 	char *path;
 	int depth;
-	FvwmPictureFlags fpf;
+	FvwmPictureAttributes fpa;
 
-	fpf.alloc_pixels = 0;
-	fpf.alpha = 1;
+	fpa.mask = 0;
+	if (xobj->colorset >= 0 && Colorset[xobj->colorset].do_dither_icon)
+	{
+		fpa.mask |= FPAM_DITHER;
+	}
 	if (xobj->iconPixmap != None)
 	{
 		XFreePixmap(dpy, xobj->iconPixmap);
@@ -376,18 +379,25 @@ void LoadIcon(struct XObj *xobj)
 		XFreePixmap(dpy, xobj->icon_alphaPixmap);
 		xobj->icon_alphaPixmap = None;
 	}
-
+	if (xobj->alloc_pixels != NULL && xobj->nalloc_pixels > 0)
+	{
+		PictureFreeColors(
+			dpy, Pcmap, xobj->alloc_pixels, xobj->nalloc_pixels, 0,
+			False);
+		xobj->alloc_pixels = NULL;
+		xobj->nalloc_pixels = 0;
+	}
 	if ((xobj->icon) == NULL)
 		return;
 	if ((path = PictureFindImageFile(xobj->icon,imagePath,R_OK)) == NULL)
 	{
 		return;
 	}
-	if (!PImageLoadPixmapFromFile(dpy, x11base->root, path, save_color_limit,
-				      &xobj->iconPixmap, &xobj->icon_maskPixmap,
-				      &xobj->icon_alphaPixmap,
-				      &xobj->icon_w, &xobj->icon_h, &depth,
-				      0, NULL, fpf))
+	if (!PImageLoadPixmapFromFile(
+		dpy, x11base->win, path, &xobj->iconPixmap,
+		&xobj->icon_maskPixmap, &xobj->icon_alphaPixmap, &xobj->icon_w,
+		&xobj->icon_h, &depth, &xobj->nalloc_pixels,
+		&xobj->alloc_pixels, 0, fpa))
 	{
 		fprintf(stderr,"[%s][LoadIcon]: <<WARNING>> Unable to "
 			"load pixmap %s\n",
@@ -639,7 +649,7 @@ void BuildGUI(int IsFather)
     tabxobj[i]->iconPixmap = None;
     tabxobj[i]->icon_maskPixmap = None;
 
-    LoadIcon(tabxobj[i]);		   /* Chargement de l'icone du widget */
+    LoadIcon(tabxobj[i]);                  /* Chargement de l'icone du widget */
 
     tabxobj[i]->InitObj(tabxobj[i]);
   }
@@ -777,32 +787,60 @@ void SendMsgToScript(XEvent event)
       /* Cas ou le recepteur demande un message et qu'il n'y en a pas */
       evnt_sel.xselection.property = None;
     }
-    XSendEvent(dpy,evnt_sel.xselection.requestor, False, 0, &evnt_sel);
+    FSendEvent(dpy,evnt_sel.xselection.requestor, False, 0, &evnt_sel);
   }
 }
 
 static
-void UpdateRootTransparency(void)
+void UpdateRootTransparency(Bool pr_only)
 {
-  int i;
+	int i;
 
-  if (x11base->colorset > 0 &&
-      Colorset[x11base->colorset].pixmap == ParentRelative)
-  {
-    XClearArea(dpy, x11base->win, 0, 0, 0, 0, True);
-    for (i=0; i<nbobj; i++)
-    {
-      if (Rectangle != tabxobj[i]->TypeWidget &&
-	  SwallowExec != tabxobj[i]->TypeWidget &&
-	  ((tabxobj[i]->colorset > 0 &&
-	   Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)))
-      {
-	XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
-      }
-    }
-  }
+	if (CSET_IS_TRANSPARENT(x11base->colorset))
+	{
+		if (CSET_IS_TRANSPARENT_PR_PURE(x11base->colorset))
+		{
+			XClearArea(dpy, x11base->win, 0, 0, 0, 0, True);
+		}
+		else if (!pr_only || CSET_IS_TRANSPARENT_PR(x11base->colorset))
+		{
+			SetWindowBackground(
+				dpy, x11base->win, x11base->size.width,
+				x11base->size.height,
+				&Colorset[x11base->colorset], Pdepth,
+				x11base->gc, True);
+		}
+	}
+	for (i=0; i<nbobj; i++)
+	{
+		if (Rectangle == tabxobj[i]->TypeWidget ||
+		    SwallowExec == tabxobj[i]->TypeWidget)
+		{
+			continue;
+		}
+		if (!CSET_IS_TRANSPARENT(tabxobj[i]->colorset) ||
+		    (CSET_IS_TRANSPARENT_PR(tabxobj[i]->colorset) &&
+		     !CSET_IS_TRANSPARENT(x11base->colorset)))
+		{
+			continue;
+		}
+		if (CSET_IS_TRANSPARENT_PR_PURE(tabxobj[i]->colorset))
+		{
+			XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
+		}
+		else if (!pr_only ||
+			 CSET_IS_TRANSPARENT_PR(tabxobj[i]->colorset))
+		{
+			SetWindowBackground(
+				dpy, tabxobj[i]->win, tabxobj[i]->width,
+				tabxobj[i]->height,
+				&Colorset[tabxobj[i]->colorset], Pdepth,
+				tabxobj[i]->gc, True);
+		}
+	}
 }
 
+
 /* read an X event */
 void ReadXServer (void)
 {
@@ -811,36 +849,61 @@ void ReadXServer (void)
   int isTab = 0;
   char *octet;
   KeySym ks;
-  static unsigned char buf[10];		/* unsigned for international */
+  static unsigned char buf[10];         /* unsigned for international */
   Bool find = False;
   char *action;
+  int ex, ey, ex2, ey2;
 
-  while (XEventsQueued(dpy, QueuedAfterReading))
+  while (FEventsQueued(dpy, QueuedAfterReading))
   {
-    XNextEvent(dpy, &event);
+    FNextEvent(dpy, &event);
     switch (event.type)
     {
     case Expose:
-      if (event.xexpose.count == 0) {
-	for (i=0; i<nbobj; i++)
-	  if (event.xexpose.window == tabxobj[i]->win) {
-	    tabxobj[i]->DrawObj(tabxobj[i]);
-	    break;
-	  }
-	/* handle exposes on x11base that need an object to render */
-	if (event.xexpose.window == x11base->win) {
-	  /* redraw first menu item to get the 3d menubar */
-	  for (i=0; i<nbobj; i++)
-	    if (Menu == tabxobj[i]->TypeWidget) {
-	      tabxobj[i]->DrawObj(tabxobj[i]);
-	      break;
-	    }
-	  /* redraw all Rectangles and SwallowExec's */
-	  for (i=0; i<nbobj; i++)
-	    if ((Rectangle == tabxobj[i]->TypeWidget)
-		||(SwallowExec == tabxobj[i]->TypeWidget))
-	      tabxobj[i]->DrawObj(tabxobj[i]);
-	}
+      ex = event.xexpose.x;
+      ey = event.xexpose.y;
+      ex2 = event.xexpose.x + event.xexpose.width;
+      ey2 = event.xexpose.y + event.xexpose.height;
+      while (FCheckTypedWindowEvent(dpy, event.xany.window, Expose, &event))
+      {
+	      ex = min(ex, event.xexpose.x);
+	      ey = min(ey, event.xexpose.y);
+	      ex2 = max(ex2, event.xexpose.x + event.xexpose.width);
+	      ey2 = max(ey2 , event.xexpose.y + event.xexpose.height);
+      }
+      event.xexpose.x = ex;
+      event.xexpose.y = ey;
+      event.xexpose.width = ex2 - ex;
+      event.xexpose.height = ey2 - ey;  
+      for (i=0; i<nbobj; i++)
+      {
+	      if (event.xexpose.window == tabxobj[i]->win)
+	      {
+		      tabxobj[i]->DrawObj(tabxobj[i], &event);
+		      break;
+	      }
+	      /* handle exposes on x11base that need an object to render */
+	      if (event.xexpose.window == x11base->win)
+	      {
+		      /* redraw first menu item to get the 3d menubar */
+		      for (i=0; i<nbobj; i++)
+		      {
+			      if (Menu == tabxobj[i]->TypeWidget)
+			      {
+				      tabxobj[i]->DrawObj(tabxobj[i], &event);
+				      break;
+			      }
+		      }
+		      /* redraw all Rectangles and SwallowExec's */
+		      for (i=0; i<nbobj; i++)
+		      {
+			      if ((Rectangle == tabxobj[i]->TypeWidget)
+				  ||(SwallowExec == tabxobj[i]->TypeWidget))
+			      {
+				      tabxobj[i]->DrawObj(tabxobj[i], &event);
+			      }
+		      }
+	      }
       }
       break;
     case ConfigureNotify:
@@ -848,7 +911,7 @@ void ReadXServer (void)
       Bool moved = False;
 
       moved = event.xconfigure.send_event;
-      while (XCheckTypedEvent(dpy, ConfigureNotify, &event))
+      while (FCheckTypedEvent(dpy, ConfigureNotify, &event))
       {
 	/* check for movement */
 	if (event.xconfigure.send_event)
@@ -856,7 +919,7 @@ void ReadXServer (void)
       }
       if (moved)
       {
-	UpdateRootTransparency();
+	UpdateRootTransparency(False);
       }
     }
     break;
@@ -869,10 +932,10 @@ void ReadXServer (void)
 	XKeysymToKeycode(dpy,XKeycodeToKeysym(dpy,event.xkey.keycode,0));
 
       /* check for bindings defined by the Key instruction */
-      if ((action = CheckBinding(BindingsList, STROKE_ARG(0)
-				 event.xkey.keycode,
-				 event.xkey.state, LockMask,
-				 C_WINDOW, KEY_BINDING)) != NULL)
+      if ((action = CheckBinding(
+		   BindingsList, STROKE_ARG(0) event.xkey.keycode,
+		   event.xkey.state, LockMask, C_WINDOW, BIND_KEYPRESS)) !=
+	  NULL)
       {
 	SendMsgAndString(action, "CheckBinding");
 	break;
@@ -923,7 +986,7 @@ void ReadXServer (void)
 	    find = True;
 	}
 	if (find)
-	  XWarpPointer(dpy, x11base->win, tabxobj[i]->win, 0, 0, 0, 0,
+	  FWarpPointer(dpy, x11base->win, tabxobj[i]->win, 0, 0, 0, 0,
 		       tabxobj[i]->width/2, 10);
       }
       break;
@@ -970,7 +1033,7 @@ void ReadXServer (void)
 	  break;
 	default:evnt_sel.xselection.property = None;
 	}
-	XSendEvent(dpy,evnt_sel.xselection.requestor,
+	FSendEvent(dpy,evnt_sel.xselection.requestor,
 		   False,0,&evnt_sel);
       }
       else
@@ -1034,7 +1097,7 @@ void MainLoop (void)
 
   while ( !isTerminated )
   {
-    while (XPending(dpy))
+    while (FPending(dpy))
       ReadXServer();
 
     FD_ZERO(&in_fdset);
@@ -1072,7 +1135,7 @@ void MainLoop (void)
 				      tabxobj[i]->gc, False);
 		  XClearWindow(dpy, tabxobj[i]->win);
 		}
-		tabxobj[i]->DrawObj(tabxobj[i]);
+		tabxobj[i]->DrawObj(tabxobj[i],NULL);
 	      }
 	    }
 	    if (n == x11base->colorset) {
@@ -1087,8 +1150,7 @@ void MainLoop (void)
 	      {
 		if (Rectangle != tabxobj[i]->TypeWidget &&
 		    SwallowExec != tabxobj[i]->TypeWidget &&
-		    ((tabxobj[i]->colorset > 0 &&
-		      Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)))
+		    CSET_IS_TRANSPARENT_PR(tabxobj[i]->colorset))
 		{
 		  XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
 		}
@@ -1107,7 +1169,7 @@ void MainLoop (void)
 	      ((!x11base->swallowed && packet->body[2] == 0) ||
 	       (x11base->swallowed && packet->body[2] == x11base->win)))
 	  {
-	    UpdateRootTransparency();
+	    UpdateRootTransparency(True);
 	  }
 	  else if  (packet->body[0] == MX_PROPERTY_CHANGE_SWALLOW &&
 		    packet->body[2] == x11base->win)
@@ -1145,19 +1207,19 @@ void ReadFvwmScriptArg(int argc, char **argv,int IsFather)
   Atom myatom;
   int FisrtArg;
 
-  BuffSend.NbMsg=0;			/* Aucun message dans le buffer */
+  BuffSend.NbMsg=0;                     /* Aucun message dans le buffer */
 
   for (i=2; i<98; i++)
     x11base->TabScriptId[i]=NULL;
 
-  if (IsFather)			/* Cas du pere */
+  if (IsFather)                 /* Cas du pere */
   {
     myatom = XInternAtom(dpy,x11base->TabScriptId[1],True);
     XSetSelectionOwner(dpy, myatom, x11base->win, CurrentTime);
     FisrtArg = 9;
   }
   else
-  {				/* Cas du fils */
+  {                             /* Cas du fils */
     x11base->TabScriptId[0] = (char*)safecalloc(sizeof(char),strlen(argv[7]));
     x11base->TabScriptId[0] = strncpy(x11base->TabScriptId[0],argv[7],
 				    strlen(argv[7])-2);
@@ -1212,7 +1274,7 @@ int main (int argc, char **argv)
   fd[0] = atoi(argv[1]);
   fd[1] = atoi(argv[2]);
   SetMessageMask(fd, M_NEW_DESK | M_END_WINDOWLIST| M_STRING |
-		 M_MAP|	 M_RES_NAME| M_RES_CLASS| M_CONFIG_INFO|
+		 M_MAP|  M_RES_NAME| M_RES_CLASS| M_CONFIG_INFO|
 		 M_END_CONFIG_INFO| M_WINDOW_NAME | M_SENDCONFIG);
   SetMessageMask(fd, MX_PROPERTY_CHANGE);
   /* Enregistrement des arguments du script */
@@ -1236,7 +1298,7 @@ int main (int argc, char **argv)
 
   SendText(fd,"Send_WindowList",0);
 
-  ReadConfig(ScriptName);	/* Lecture et analyse du script */
+  ReadConfig(ScriptName);       /* Lecture et analyse du script */
 
   /* Fonction d'initialisation de TabCom et TabFunc   */
   InitCom();
@@ -1278,8 +1340,8 @@ int main (int argc, char **argv)
   signal (SIGTERM, TerminateHandler);
 #ifdef HAVE_SIGINTERRUPT
   siginterrupt(SIGPIPE, 1);
-  siginterrupt(SIGINT,	1);
-  siginterrupt(SIGHUP,	1);
+  siginterrupt(SIGINT,  1);
+  siginterrupt(SIGHUP,  1);
   siginterrupt(SIGQUIT, 1);
   siginterrupt(SIGTERM, 1);
 #endif
