@@ -5,16 +5,16 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "Tools.h"
-#include <time.h>
+#include "libs/ftime.h"
 
 /******************************************************/
 /******************************************************/
@@ -46,7 +46,7 @@ void InitPopupMenu(struct XObj *xobj)
   Attr.background_pixel = xobj->TabColor[back];
   mask |= CWBackPixel;
   Attr.cursor = XCreateFontCursor(dpy, XC_hand2);
-  mask |= CWCursor;		/* Curseur pour la fenetre / window cursor */
+  mask |= CWCursor;             /* Curseur pour la fenetre / window cursor */
 
   xobj->win = XCreateWindow(dpy, *xobj->ParentWin,
 		xobj->x, xobj->y, xobj->width, xobj->height, 0,
@@ -95,7 +95,7 @@ void DestroyPopupMenu(struct XObj *xobj)
   XDestroyWindow(dpy, xobj->win);
 }
 
-void DrawPopupMenu(struct XObj *xobj)
+void DrawPopupMenu(struct XObj *xobj, XEvent *evp)
 {
   XSegment segm[4];
   char* str;
@@ -141,7 +141,7 @@ void DrawPopupMenu(struct XObj *xobj)
 		       FlocaleTextWidth(xobj->Ffont,str,strlen(str)),
 		       25,8,8);
   MyDrawString(dpy, xobj, xobj->win, x, y, str, fore, hili, back,
-	     !xobj->flags[1]);
+	     !xobj->flags[1], NULL, evp);
 
   free(str);
 }
@@ -186,7 +186,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
   Attr.colormap = Pcmap;
   mask |= CWColormap;
   Attr.cursor = XCreateFontCursor(dpy, XC_hand2);
-  mask |= CWCursor;		/* Curseur pour la fenetre / Window cursor */
+  mask |= CWCursor;             /* Curseur pour la fenetre / Window cursor */
   Attr.override_redirect = True;
   mask |= CWOverrideRedirect;
   WinPop = XCreateWindow(dpy, Root, x, y, xobj->width, hMenu, 0,
@@ -211,7 +211,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
 
   while (End)
   {
-    XQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
+    FQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
     /* Determiner l'option courante / Current option */
     y2 = y2 - y;
     x2 = x2 - x;
@@ -228,7 +228,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
       oldvalue = newvalue;
     }
 
-    XNextEvent(dpy, &event);
+    FNextEvent(dpy, &event);
     switch (event.type)
     {
     case KeyPress:
@@ -241,10 +241,10 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
 	End = 0;
       }
       else if (ks == XK_Up && y2 >= hOpt) {
-	XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, -hOpt);
+	FWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, -hOpt);
       }
       else if (ks == XK_Down && y2 + hOpt <= hMenu) {
-	XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, hOpt);
+	FWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, hOpt);
       }
       break;
     case ButtonPress:
@@ -264,7 +264,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
 #if 0
   do
   {
-    XQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
+    FQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
     /* Determiner l'option courante / Current option */
     y2 = y2 - y;
     x2 = x2 - x;
@@ -286,7 +286,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
     FD_SET(x_fd, &in_fdset);
     select(32, SELECT_FD_SET_CAST &in_fdset, NULL, NULL, NULL);
   }
-  while (!XCheckTypedEvent(dpy, while_mask, &event));
+  while (!FCheckTypedEvent(dpy, while_mask, &event));
 #endif
  XUngrabPointer(dpy, CurrentTime);
  XSync(dpy,0);
