@@ -5,56 +5,109 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "Tools.h"
 
 /***********************************************/
-/* Fonction d'ecriture en relief	       */
+/* Fonction d'ecriture en relief               */
 /***********************************************/
-void MyDrawString(Display *dpy, struct XObj *xobj, Window win, int x, int y,
-		  char *str, unsigned long ForeC,unsigned long HiC,
-		  unsigned long BackC, int WithRelief)
+void MyDrawString(
+	Display *dpy, struct XObj *xobj, Window win, int x, int y,
+	char *str, unsigned long ForeC,unsigned long HiC,
+	unsigned long BackC, int WithRelief, XRectangle *clip, XEvent *evp)
 {
-  FwinString->win = win;
-  FwinString->str = str;
-  FwinString->gc = xobj->gc;
-  FwinString->flags.has_colorset = False;
-  if (xobj->colorset >= 0)
-  {
-    FwinString->colorset = &Colorset[xobj->colorset];
-    FwinString->flags.has_colorset = True;
-  }
-  if (WithRelief && xobj->Ffont->shadow_size == 0)
-  {
-    XSetBackground(dpy, xobj->gc, xobj->TabColor[BackC]);
-    XSetForeground(dpy, xobj->gc, xobj->TabColor[HiC]);
-    FwinString->x = x+1;
-    FwinString->y = y+1;
-    FlocaleDrawString(dpy, xobj->Ffont, FwinString, 0);
-    XSetForeground(dpy, xobj->gc, xobj->TabColor[ForeC]);
-    FwinString->x = x;
-    FwinString->y = y;
-    FlocaleDrawString(dpy, xobj->Ffont, FwinString, 0);
-  }
-  else
-  {
-    XSetBackground(dpy, xobj->gc, xobj->TabColor[BackC]);
-    XSetForeground(dpy, xobj->gc, xobj->TabColor[ForeC]);
-    FwinString->x = x;
-    FwinString->y = y+1;
-    FlocaleDrawString(dpy, xobj->Ffont, FwinString, 0);
-  }
+	Region region = None;
+	XRectangle inter;
+	Bool do_draw = True;
+
+	if (evp && clip)
+	{
+		if (!frect_get_intersection(
+			clip->x, clip->y, clip->width, clip->height,
+			evp->xexpose.x, evp->xexpose.y, 
+			evp->xexpose.width, evp->xexpose.height,
+			&inter))
+		{
+			do_draw = False;
+		}
+	}
+	else if (clip)
+	{
+		inter.x = clip->x;
+		inter.y = clip->y;
+		inter.width = clip->width;
+		inter.height = clip->height;
+	}
+	else if (evp)
+	{
+		inter.x = evp->xexpose.x;
+		inter.y = evp->xexpose.y;
+		inter.width = evp->xexpose.width;
+		inter.height = evp->xexpose.height;	
+	}
+	if (!do_draw)
+	{
+		return;
+	}
+	if (clip || evp)
+	{
+		region = XCreateRegion();
+		XUnionRectWithRegion (&inter, region, region);
+		XSetClipRectangles(dpy, xobj->gc, 0, 0, &inter, 1, Unsorted);
+		FwinString->flags.has_clip_region = True;
+		FwinString->clip_region = region;
+	}
+	else
+	{
+		FwinString->flags.has_clip_region = False;
+	}
+
+	FwinString->win = win;
+	FwinString->str = str;
+	FwinString->gc = xobj->gc;
+	FwinString->flags.has_colorset = False;
+	if (xobj->colorset >= 0)
+	{
+		FwinString->colorset = &Colorset[xobj->colorset];
+		FwinString->flags.has_colorset = True;
+	}
+	if (WithRelief && xobj->Ffont->shadow_size == 0)
+	{
+		XSetBackground(dpy, xobj->gc, xobj->TabColor[BackC]);
+		XSetForeground(dpy, xobj->gc, xobj->TabColor[HiC]);
+		FwinString->x = x+1;
+		FwinString->y = y+1;
+		FlocaleDrawString(dpy, xobj->Ffont, FwinString, 0);
+		XSetForeground(dpy, xobj->gc, xobj->TabColor[ForeC]);
+		FwinString->x = x;
+		FwinString->y = y;
+		FlocaleDrawString(dpy, xobj->Ffont, FwinString, 0);
+	}
+	else
+	{
+		XSetBackground(dpy, xobj->gc, xobj->TabColor[BackC]);
+		XSetForeground(dpy, xobj->gc, xobj->TabColor[ForeC]);
+		FwinString->x = x;
+		FwinString->y = y+1;
+		FlocaleDrawString(dpy, xobj->Ffont, FwinString, 0);
+	}
+	if (region)
+	{
+		XDestroyRegion(region);
+		XSetClipMask(dpy, xobj->gc, None);
+	}
+	FwinString->flags.has_clip_region = False;
 }
 
 /**************************************************/
-/* Return the x text position of the widget	  */
+/* Return the x text position of the widget       */
 /**************************************************/
 int GetXTextPosition(struct XObj *xobj, int obj_width, int str_len,
 		     int left_offset, int center_offset, int right_offset)
@@ -96,7 +149,7 @@ int GetXTextPosition(struct XObj *xobj, int obj_width, int str_len,
 }
 
 /**************************************************/
-/* Retourne le titre de l'option id du menu	  */
+/* Retourne le titre de l'option id du menu       */
 /**************************************************/
 char* GetMenuTitle(char *str, int id)
 {
@@ -124,7 +177,7 @@ char* GetMenuTitle(char *str, int id)
 }
 
 /***********************************************************/
-/* Dessine le contenu de la fenetre du popup-menu	   */
+/* Dessine le contenu de la fenetre du popup-menu          */
 /***********************************************************/
 void DrawPMenu(struct XObj *xobj, Window WinPop, int h, int StrtOpt)
 {
@@ -181,12 +234,12 @@ void UnselectMenu(struct XObj *xobj, Window WinPop, int hOpt, int value,
   x = GetXTextPosition(xobj, width, FlocaleTextWidth(xobj->Ffont,str,len),
 		       8, 0, 8);
   MyDrawString(dpy, xobj, WinPop, x, y, str, fore, hili, back,
-	       !xobj->flags[1]);
+	       !xobj->flags[1], NULL, NULL);
   free(str);
 }
 
 /***********************************************************/
-/* Dessine l'option active d'un menu			   */
+/* Dessine l'option active d'un menu                       */
 /***********************************************************/
 void SelectMenu(struct XObj *xobj, Window WinPop, int hOpt, int value)
 {
@@ -248,45 +301,156 @@ int CountOption(char *str)
 /*****************************************/
 /* Dessine l'icone et le titre du widget */
 /*****************************************/
-void DrawIconStr(int offset, struct XObj *xobj, int DoRedraw,
-		 int l_offset, int c_offset, int r_offset)
+void DrawIconStr(
+	int offset, struct XObj *xobj, int DoRedraw,
+	int l_offset, int c_offset, int r_offset,
+	XRectangle *str_clip, XRectangle *icon_clip, XEvent *evp)
 {
-  int i,j;
-  char *str;
-  int len = 0;
+	int i,j;
+	char *str;
+	int len = 0;
+	XRectangle clear_r, inter;
+	Bool do_clear = True;
 
-  if (DoRedraw)
-    XClearArea(dpy, xobj->win, 4, 4, xobj->width-8, xobj->height-8, False);
+	inter.x = clear_r.x = 4;
+	inter.y = clear_r.y = 4;
+	inter.width = clear_r.width  = xobj->width-8;
+	inter.height = clear_r.height = xobj->height-8;
 
-  str = GetMenuTitle(xobj->title,1);
-  len = strlen(str);
-  i = GetXTextPosition(xobj, xobj->width, FlocaleTextWidth(xobj->Ffont,str,len),
-		       l_offset, c_offset, r_offset);
+	if (evp)
+	{
+		if (!frect_get_intersection(
+			clear_r.x, clear_r.y, clear_r.width, clear_r.height,
+			evp->xexpose.x, evp->xexpose.y, 
+			evp->xexpose.width, evp->xexpose.height,
+			&inter))
+		{
+			do_clear = False;
+		}
+	}
+#if 0
+	else if (clip)
+	{
+		inter.x = clip->x;
+		inter.y = clip->y;
+		inter.width = clip->width;
+		inter.height = clip->height;
+	}
+	else if (evp)
+	{
+		inter.x = evp->xexpose.x;
+		inter.y = evp->xexpose.y;
+		inter.width = evp->xexpose.width;
+		inter.height = evp->xexpose.height;	
+	}
+#endif
+	if (DoRedraw && do_clear)
+	{
+		XClearArea(
+			dpy, xobj->win,
+			inter.x, inter.y, inter.width, inter.height, False);
+	}
 
-  if (len > 0 && xobj->iconPixmap==None)       /* Si l'icone n'existe pas */
-  {
-    j = xobj->height/2 + (xobj->Ffont->height)/2 + offset - 3;
-    MyDrawString(dpy,xobj,xobj->win,i,j,str,fore,hili,back,!xobj->flags[1]);
-  }
-  else					/* Si l'icone existe */
-  {
-    if (len > 0)
-    {
-      j = ((xobj->height - xobj->icon_h)/4)*3 + xobj->icon_h + offset +
-	(xobj->Ffont->height)/2 - 3;
-      MyDrawString(dpy,xobj,xobj->win,i,j,str,fore,hili,back,!xobj->flags[1]);
-    }
-    /* Dessin de l'icone */
-    j=(xobj->height - xobj->icon_h)/2+offset;
-    i=(xobj->width - xobj->icon_w)/2+offset;
-    PGraphicsCopyPixmaps(dpy, xobj->iconPixmap,
-			 xobj->icon_maskPixmap,
-			 xobj->icon_alphaPixmap,
-			 Pdepth, xobj->win, xobj->gc,
-			 0, 0, xobj->icon_w, xobj->icon_h,
-			 i, j);
-  }
-  free(str);
+	str = GetMenuTitle(xobj->title,1);
+	len = strlen(str);
+	i = GetXTextPosition(
+		xobj, xobj->width, FlocaleTextWidth(xobj->Ffont,str,len),
+		l_offset, c_offset, r_offset);
+
+	if (len > 0 && xobj->iconPixmap==None)
+	{
+		/* Si l'icone n'existe pas */
+		j = xobj->height/2 - (xobj->Ffont->height)/2 +
+			xobj->Ffont->ascent + offset;
+		MyDrawString(
+			dpy,xobj,xobj->win,i,j,str,fore,hili,back,
+			!xobj->flags[1], str_clip, evp);
+	}
+	else
+	{
+		/* Si l'icone existe */
+		FvwmRenderAttributes fra;
+		Bool do_draw_icon = True;
+		XRectangle ir;
+		int iy = (xobj->height - xobj->icon_h)/2+offset;
+		int ix = (xobj->width - xobj->icon_w)/2+offset;
+
+		if (evp && icon_clip)
+		{
+			if (!frect_get_intersection(
+				icon_clip->x, icon_clip->y,
+				icon_clip->width, icon_clip->height,
+				evp->xexpose.x, evp->xexpose.y, 
+				evp->xexpose.width, evp->xexpose.height,
+				&inter))
+			{
+				do_draw_icon = False;
+			}
+			else if (!frect_get_intersection(
+				inter.x, inter.y,
+				inter.width, inter.height,
+				ix, iy, xobj->icon_w, xobj->icon_h,
+				&ir))
+			{
+				do_draw_icon = False;
+			}
+		}
+		else if (icon_clip)
+		{
+			if (!frect_get_intersection(
+				icon_clip->x, icon_clip->y,
+				icon_clip->width, icon_clip->height,
+				ix, iy, xobj->icon_w, xobj->icon_h,
+				&ir))
+			{
+				do_draw_icon = False;
+			}
+		}
+		else if (evp)
+		{
+			if (!frect_get_intersection(
+				evp->xexpose.x, evp->xexpose.y, 
+				evp->xexpose.width, evp->xexpose.height,
+				ix, iy, xobj->icon_w, xobj->icon_h,
+				&ir))
+			{
+				do_draw_icon = False;
+			}
+		}
+		else
+		{
+			ir.x = ix;
+			ir.y = iy;
+			ir.width = xobj->icon_w;
+			ir.height = xobj->icon_h;
+		}
+
+		if (len > 0)
+		{
+			j = ((xobj->height - xobj->icon_h)/4)*3 +
+				xobj->icon_h + offset + xobj->Ffont->ascent; 
+			MyDrawString(
+				dpy,xobj,xobj->win,i,j,str,fore,hili,
+				back,!xobj->flags[1], str_clip, evp);
+		}
+		/* Dessin de l'icone */
+		fra.mask = FRAM_DEST_IS_A_WINDOW;
+		if (xobj->colorset >= 0)
+		{
+			fra.mask |= FRAM_HAVE_ICON_CSET;
+			fra.colorset = &Colorset[xobj->colorset];
+		}
+		if (do_draw_icon)
+		{
+			PGraphicsRenderPixmaps(
+				dpy, xobj->win, xobj->iconPixmap,
+				xobj->icon_maskPixmap, xobj->icon_alphaPixmap,
+				Pdepth, &fra, xobj->win, xobj->gc, None, None,
+				ir.x - ix, ir.y - iy, ir.width, ir.height,
+				ir.x, ir.y, ir.width, ir.height, False);
+		}
+	}
+	free(str);
 }
 
 /***********************************************/
@@ -384,7 +548,7 @@ void UnselectAllTextField(struct XObj **txobj)
       if (txobj[i]->value2 != txobj[i]->value)
       {
 	txobj[i]->value2 = txobj[i]->value;
-	txobj[i]->DrawObj(txobj[i]);
+	txobj[i]->DrawObj(txobj[i],NULL);
 	return;
       }
 }
@@ -398,13 +562,13 @@ void SelectOneTextField(struct XObj *xobj)
       if (tabxobj[i]->value2 != tabxobj[i]->value)
       {
 	tabxobj[i]->value2 = tabxobj[i]->value;
-	tabxobj[i]->DrawObj(tabxobj[i]);
+	tabxobj[i]->DrawObj(tabxobj[i],NULL);
 	return;
       }
 }
 
 /************************************************************/
-/* Dessine une fleche direction nord			    */
+/* Dessine une fleche direction nord                        */
 /************************************************************/
 void DrawArrowN(struct XObj *xobj, int x, int y, int Press)
 {
@@ -449,7 +613,7 @@ void DrawArrowN(struct XObj *xobj, int x, int y, int Press)
 }
 
 /************************************************************/
-/* Dessine une fleche direction sud			    */
+/* Dessine une fleche direction sud                         */
 /************************************************************/
 void DrawArrowS(struct XObj *xobj, int x, int y, int Press)
 {
@@ -603,5 +767,5 @@ int IsItDoubleClic(struct XObj *xobj)
   XEvent Event;
   XFlush(dpy);
   Wait(12);
-  return (XCheckTypedEvent(dpy, ButtonPress, &Event));
+  return (FCheckTypedEvent(dpy, ButtonPress, &Event));
 }
